@@ -1,8 +1,26 @@
+/* $Id: ai_python.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/*
+   Copyright (C) 2007 - 2008
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
+
+   See the COPYING file for more details.
+*/
+
+//! @file ai_python.hpp
+//!
+
 #ifndef AI_PYTHON_HPP_INCLUDED
 #define AI_PYTHON_HPP_INCLUDED
 
 #include "ai_interface.hpp"
 #include "menu_events.hpp"
+#undef _POSIX_C_SOURCE	// avoids a spurious compiler warning
 #include <Python.h>
 
 typedef struct {
@@ -20,6 +38,7 @@ typedef struct {
 	const unit* unit_;
 } wesnoth_unit;
 
+#define W(name) static PyObject *wrapper_##name(PyObject* self, PyObject* args)
 class python_ai : public ai_interface
 {
 public:
@@ -32,38 +51,43 @@ public:
     static PyObject* wrapper_unittype_movement_cost(wesnoth_unittype*, PyObject* args);
     static PyObject* wrapper_unittype_defense_modifier(wesnoth_unittype*, PyObject* args);
 
-	static PyObject* wrapper_get_units(PyObject* self, PyObject* args);
-	static PyObject* wrapper_log_message(PyObject* self, PyObject* args);
-	static PyObject* wrapper_get_location(PyObject* self, PyObject* args);
-	static PyObject* wrapper_get_map(PyObject* self, PyObject* args);
-	static PyObject* wrapper_get_teams(PyObject* self, PyObject* args);
-	static PyObject* wrapper_get_current_team(PyObject* self, PyObject* args);
-	static PyObject* wrapper_get_src_dst(PyObject* self, PyObject* args);
-	static PyObject* wrapper_get_dst_src(PyObject* self, PyObject* args);
-	static PyObject* wrapper_get_enemy_src_dst(PyObject* self, PyObject* args);
-	static PyObject* wrapper_get_enemy_dst_src(PyObject* self, PyObject* args);
-	static PyObject* wrapper_move_unit(PyObject* self, PyObject* args);
-	static PyObject* wrapper_attack_unit(PyObject* self, PyObject* args);
-	static PyObject* wrapper_get_adjacent_tiles(PyObject* self, PyObject* args);
-	static PyObject* wrapper_recruit_unit(PyObject* self, PyObject* args);
-	static PyObject* wrapper_get_gamestatus(PyObject* self, PyObject* args);
-	static PyObject* wrapper_set_variable(PyObject*, PyObject* args);
-	static PyObject* wrapper_get_variable(PyObject*, PyObject* args);
-	static PyObject* wrapper_get_version(PyObject*, PyObject* args);
+    W(team_targets);
+    W(get_units);
+    W(log_message);
+    W(get_location);
+    W(get_map);
+    W(get_teams);
+    W(get_current_team);
+    W(get_src_dst);
+    W(get_dst_src);
+    W(get_enemy_src_dst);
+    W(get_enemy_dst_src);
+    W(move_unit);
+    W(attack_unit);
+    W(get_adjacent_tiles);
+    W(recruit_unit);
+    W(get_gamestatus);
+    W(set_variable);
+    W(get_variable);
+    W(get_version);
+    W(raise_user_interact);
+    W(test_move);
 
 	static PyObject* unittype_advances_to( wesnoth_unittype* type, PyObject* args );
 	static PyObject* wrapper_team_recruits( wesnoth_team* team, PyObject* args );
 	static PyObject* wrapper_unit_find_path( wesnoth_unit* unit, PyObject* args );
 	static PyObject* wrapper_unit_attack_statistics(wesnoth_unit* unit, PyObject* args);
 
-	static void set_error(const char *fmt, ...);
-
-	static bool is_unit_valid(const unit* unit, bool do_set_error = true);
+	static bool is_unit_valid(const unit* unit);
 	std::vector<team>& get_teams() { return get_info().teams; }
     static std::vector<std::string> get_available_scripts();
-protected:
-	static bool init_;
-	static PyObject* python_error_;
+    static void initialize_python();
+    static void invoke(std::string name);
+
+    friend void recalculate_movemaps();
+private:
+    static bool init_;
+
     end_level_exception exception;
 	ai_interface::move_map src_dst_;
 	ai_interface::move_map dst_src_;
@@ -72,5 +96,6 @@ protected:
 	ai_interface::move_map enemy_dst_src_;
 	std::map<location,paths> enemy_possible_moves_;
 };
+#undef W
 
 #endif
