@@ -1,16 +1,20 @@
-/* $Id: array.hpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: array.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file array.hpp 
+//! Template for arrays.
+
 #ifndef ARRAY_HPP_INCLUDED
 #define ARRAY_HPP_INCLUDED
 
@@ -62,7 +66,7 @@ private:
 	T a[N];
 };
 
-}
+} // end namespace util
 
 #endif
 
