@@ -1,17 +1,21 @@
-/* $Id: config.cpp 21169 2007-10-23 08:39:31Z soliton $ */
+/* $Id: config.cpp 23936 2008-02-18 19:17:49Z suokko $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
-   Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2003 by David White <dave@whitevine.net>
+   Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file config.cpp
+//! Routines related to configuration-files / WML.
+
 #include "global.hpp"
 
 #include <algorithm>
@@ -20,10 +24,6 @@
 #include "gettext.hpp"
 #include "log.hpp"
 #include "util.hpp"
-#include "wassert.hpp"
-#include "serialization/string_utils.hpp"
-
-#include <iostream>
 
 #define ERR_CF LOG_STREAM(err, config)
 
@@ -83,7 +83,7 @@ void config::merge_children(const std::string& key)
 		merged_children.append(**i);
 	}
 
-	detach_children(key);
+	clear_children(key);
 	add_child(key,merged_children);
 }
 
@@ -200,12 +200,6 @@ private:
 
 }
 
-void config::detach_children(const std::string& key)
-{
-	ordered_children.erase(std::remove_if(ordered_children.begin(),ordered_children.end(),remove_ordered(key)),ordered_children.end());
-	children.erase(key);
-}
-
 void config::clear_children(const std::string& key)
 {
 	ordered_children.erase(std::remove_if(ordered_children.begin(),ordered_children.end(),remove_ordered(key)),ordered_children.end());
@@ -219,23 +213,58 @@ void config::clear_children(const std::string& key)
 	}
 }
 
+void config::recursive_clear_value(const std::string& key)
+{
+	values.erase(key);
+
+	for (all_children_iterator it = ordered_begin(); it != ordered_end(); ++it)
+	{
+		const_cast<config*>(it->second)->recursive_clear_value(key);
+	}
+}
+
+config::all_children_iterator config::erase(const config::all_children_iterator& i)
+{
+	config* found_config = NULL;
+	std::vector<child_pos>::iterator erase_pos, j, j_end = ordered_children.end();
+	for(j = ordered_children.begin(); j != j_end; ++j) {
+		if(i.get_key() == j->pos->first) {
+			if(i.get_index() == j->index) {
+				erase_pos = j;
+				found_config = *(j->pos->second.begin() + j->index);
+			} else if(i.get_index() < j->index) {
+				//decrement subsequent child indeces of the same key
+				j->index--;
+			}
+		}
+	}
+	child_list& vec = children[i.get_key()];
+	if(!found_config || erase_pos->index >= vec.size()) {
+		ERR_CF << "Error: attempting to delete non-existing child: "
+			<< i.get_key() << "[" << i.get_index() << "]\n";
+		return ordered_end();
+	}
+	delete found_config;
+	vec.erase(vec.begin()+i.get_index());
+	return all_children_iterator(ordered_children.erase(erase_pos));
+}
+
 void config::remove_child(const std::string& key, size_t index)
 {
-	//remove from the ordering
+	// Remove from the ordering
 	const child_pos pos(children.find(key),index);
 	ordered_children.erase(std::remove(ordered_children.begin(),ordered_children.end(),pos),ordered_children.end());
 
-	//decrement all indices in the ordering that are above this index, since everything
-	//is getting shifted back by 1.
+	// Decrement all indices in the ordering that are above this index,
+	// since everything is getting shifted back by 1.
 	for(std::vector<child_pos>::iterator i = ordered_children.begin(); i != ordered_children.end(); ++i) {
 		if(i->pos->first == key && i->index > index) {
 			i->index--;
 		}
 	}
 
-	//remove from the child map
+	// Remove from the child map
 	child_list& v = children[key];
-	//wassert(index < v.size());
 	if(index >= v.size()) {
 		ERR_CF << "Error: attempting to delete non-existing child: "
 			<< key << "[" << index << "]\n";
@@ -254,15 +283,6 @@ t_string& config::operator[](const std::string& key)
 const t_string& config::operator[](const std::string& key) const
 {
 	return get_attribute(key);
-#if 0
-	const std::string& str = get_attribute(key);
-	//see if the value is a variable
-	if (!str.empty() && str[0] == '$') {
-		return game_events::get_variable_const(std::string(str.begin() + 1, str.end()));
-	} else {
-		return str;
-	}
-#endif
 }
 
 const t_string& config::get_attribute(const std::string& key) const
@@ -289,7 +309,7 @@ private:
 	const std::string name_, value_;
 };
 
-}
+} // end namespace
 
 config* config::find_child(const std::string& key,
                            const std::string& name,
@@ -347,7 +367,7 @@ bool config::empty() const
 config::all_children_iterator::all_children_iterator(config::all_children_iterator::Itor i) : i_(i)
 {}
 
-config::all_children_iterator config::all_children_iterator::operator++()
+config::all_children_iterator& config::all_children_iterator::operator++()
 {
 	++i_;
 	return *this;
@@ -365,6 +385,26 @@ std::pair<const std::string*,const config*> config::all_children_iterator::opera
 	return std::pair<const std::string*,const config*>(&(i_->pos->first),i_->pos->second[i_->index]);
 }
 
+config::all_children_iterator::pointer config::all_children_iterator::operator->() const
+{
+	return pointer(new std::pair<const std::string*,const config*>(&(i_->pos->first),i_->pos->second[i_->index]));
+}
+
+const std::string& config::all_children_iterator::get_key() const
+{
+	return i_->pos->first;
+}
+
+const config& config::all_children_iterator::get_child() const
+{
+	return *(i_->pos->second[i_->index]);
+}
+
+size_t config::all_children_iterator::get_index() const
+{
+	return i_->index;
+}
+
 bool config::all_children_iterator::operator==(all_children_iterator i) const
 {
 	return i_ == i.i_;
@@ -436,24 +476,24 @@ config config::get_diff(const config& c) const
 
 		static const child_list dummy;
 
-		//get the two child lists. 'b' has to be modified to look like 'a'
+		// Get the two child lists. 'b' has to be modified to look like 'a'.
 		const child_list& a = itor_a != children.end() ? itor_a->second : dummy;
 		const child_list& b = itor_b != c.children.end() ? itor_b->second : dummy;
 
 		size_t ndeletes = 0;
 		size_t ai = 0, bi = 0;
 		while(ai != a.size() || bi != b.size()) {
-			//if the two elements are the same, nothing needs to be done
+			// If the two elements are the same, nothing needs to be done.
 			if(ai < a.size() && bi < b.size() && *a[ai] == *b[bi]) {
 				++ai;
 				++bi;
 			} else {
-				//we have to work out what the most appropriate operation --
-				//delete, insert, or change is the best to get b[bi] looking like a[ai]
+				// We have to work out what the most appropriate operation --
+				// delete, insert, or change is the best to get b[bi] looking like a[ai].
 				std::stringstream buf;
 
-				//if b has more elements than a, then we assume this element is an
-				//element that needs deleting
+				// If b has more elements than a, then we assume this element
+				// is an element that needs deleting.
 				if(b.size() - bi > a.size() - ai) {
 					config& new_delete = res.add_child("delete_child");
 					buf << bi - ndeletes;
@@ -464,8 +504,8 @@ config config::get_diff(const config& c) const
 					++bi;
 				}
 
-				//if b has less elements than a, then we assume this element is an
-				//element that needs inserting
+				// If b has less elements than a, then we assume this element
+				// is an element that needs inserting.
 				else if(b.size() - bi < a.size() - ai) {
 					config& new_insert = res.add_child("insert_child");
 					buf << ai;
@@ -475,8 +515,8 @@ config config::get_diff(const config& c) const
 					++ai;
 				}
 
-				//otherwise, they have the same number of elements, so try just
-				//changing this element to match
+				// Otherwise, they have the same number of elements,
+				// so try just changing this element to match.
 				else {
 					config& new_change = res.add_child("change_child");
 					buf << bi;
@@ -550,33 +590,78 @@ void config::apply_diff(const config& diff)
 	}
 }
 
-// Create a new config tree as a copy of 'this' overridden by 'c'.
-// Nodes are matched up by name and with name by order. Nodes in 'c',
-// but not in 'this' are added at the end in the order they appeared in 'c'.
-config config::merge_with(const config& c) const
+void config::merge_with(const config& c)
 {
-	config n;
-	for(string_map::const_iterator j = this->values.begin();
-	    j != this->values.end(); ++j) {
-		n.values[j->first] = j->second;
+	std::map<std::string, unsigned> visitations;
+
+	// Merge attributes first
+	string_map::const_iterator attrib_it, attrib_end = c.values.end();
+	for(attrib_it = c.values.begin(); attrib_it != attrib_end; ++attrib_it) {
+		values[attrib_it->first] = attrib_it->second;
 	}
-	// This ends up copying values twice (in config initialization and
-	// append), but is simpler than dealing with the guts of m.
-	config m(c);
-	for(all_children_iterator i = this->ordered_begin();
-	    i != this->ordered_end(); ++i) {
-		const std::pair<const std::string*,const config*>& value = *i;
-		config *mc = m.child(*value.first);
-		if (mc == NULL) {
-			n.add_child(*value.first, *value.second);
-		}
-		else {
-			n.add_child(*value.first, value.second->merge_with(*mc));
-			m.remove_child(*value.first, 0);
+
+	// Now merge shared tags
+	all_children_iterator::Itor i, i_end = ordered_children.end();
+	for(i = ordered_children.begin(); i != i_end; ++i) {
+		const std::string& tag = i->pos->first;
+		child_map::const_iterator j = c.children.find(tag);
+		if (j != c.children.end()) {
+			unsigned &visits = visitations[tag];
+			if(visits < j->second.size()) {
+				(i->pos->second[i->index])->merge_with(*j->second[visits++]);
+			}
 		}
 	}
-	n.append(m);
-	return n;
+
+	// Now add any unvisited tags
+	for(child_map::const_iterator j = c.children.begin(); j != c.children.end(); ++j) {
+		const std::string& tag = j->first;
+		unsigned &visits = visitations[tag];
+		while(visits < j->second.size()) {
+			add_child(tag, *j->second[visits++]);
+		}
+	}
+}
+
+bool config::matches(const config &filter) const
+{
+	// First match values. all values should match.
+	for(string_map::const_iterator j = filter.values.begin(); j != filter.values.end(); ++j) {
+		if(!this->values.count(j->first)) return false;
+		const t_string& test_val = this->values.find(j->first)->second;
+		if(test_val != j->second) {
+			const std::string& boolcheck = j->second.base_str();
+			if(boolcheck == "yes" || boolcheck == "true" || boolcheck == "on") {
+				if(!utils::string_bool(test_val.base_str(), false)) {
+					return false;
+				}
+			} else if(boolcheck == "no" || boolcheck == "false" || boolcheck == "off") {
+				if(utils::string_bool(test_val.base_str(), true)) {
+					return false;
+				}
+			} else {
+				return false;
+			}
+		}
+	}
+
+	// Now, match the kids
+	for(all_children_iterator i2 = filter.ordered_begin(); i2 != filter.ordered_end(); ++i2) {
+		if(*(*i2).first == "not") continue;
+		child_list interesting_children = get_children(*(*i2).first);
+		bool found = false;
+		for(child_list::iterator j2 = interesting_children.begin(); j2 != interesting_children.end(); ++j2) {
+			if((*j2)->matches(*(*i2).second)) {
+				found = true;
+			}
+		}
+		if(!found) return false;
+	}
+	child_list negative_children = filter.get_children("not");
+	for(child_list::iterator j3 = negative_children.begin() ; j3 != negative_children.end() ; j3++) {
+		if(matches(**j3)) return false;
+	}
+	return true;
 }
 
 void config::prune() {
@@ -610,24 +695,29 @@ void config::reset_translation() const
 	}
 }
 
-void config::debug() const{
+std::string config::debug() const
+{
+	std::ostringstream outstream;
+	outstream << *this;
+	return outstream.str();
+}
+
+std::ostream& operator << (std::ostream& outstream, const config& cfg) {
 	static int i = 0;
 	i++;
-	for(string_map::const_iterator val = values.begin(); val != values.end(); ++val) {
-		for (int j = 0; j < i; j++){ std::cerr << char(9); }
-		std::cerr << val->first << ", " << val->second << "\n";
+	for(string_map::const_iterator val = cfg.values.begin(); val != cfg.values.end(); ++val) {
+		for (int j = 0; j < i-1; j++){ outstream << char(9); }
+		outstream << val->first << " = " << val->second << "\n";
 	}
-
-	for(child_map::const_iterator list = children.begin(); list != children.end(); ++list) {
-		{ for (int j = 0; j < i-1; j++){ std::cerr << char(9); } }
-		std::cerr << "[" << list->first << "]\n";
-		for(child_list::const_iterator child = list->second.begin(); child != list->second.end(); ++child) {
-			(*child)->debug();
-		}
-		{ for (int j = 0; j < i-1; j++){ std::cerr << char(9); } }
-		std::cerr << "[/" << list->first << "]\n";
+	for(config::all_children_iterator list = cfg.ordered_begin(); list != cfg.ordered_end(); ++list) {
+		{ for (int j = 0; j < i-1; j++){ outstream << char(9); } }
+		outstream << "[" << *(*list).first << "]\n";
+		outstream << *(*list).second;
+		{ for (int j = 0; j < i-1; j++){ outstream << char(9); } }
+		outstream << "[/" << *(*list).first << "]\n";
 	}
 	i--;
+    return outstream;
 }
 
 std::string config::hash() const
@@ -657,7 +747,8 @@ std::string config::hash() const
 					i = 0;
 				}
 			}
-			for(c = val->second.value().begin(); c != val->second.value().end(); ++c) {
+			const std::string &base_str = val->second.base_str();
+			for(c = base_str.begin(); c != base_str.end(); ++c) {
 				if(utils::portable_isspace(*c)) {
 					continue;
 				}
@@ -683,7 +774,8 @@ std::string config::hash() const
 	}
 
 	for(i = 0; i != hash_length; ++i) {
-		hash_str[i] = hash_string[(unsigned)hash_str[i]%strlen(hash_string)];
+		hash_str[i] = hash_string[
+			static_cast<unsigned>(hash_str[i]) % strlen(hash_string)];
 	}
 
 	return std::string(hash_str);
