@@ -1,9 +1,11 @@
+/* $Id: editor.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-  Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+  Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License.
+  it under the terms of the GNU General Public License version 2
+  or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.
 
@@ -19,6 +21,7 @@
 #include "editor_palettes.hpp"
 #include "editor_layout.hpp"
 #include "editor_undo.hpp"
+#include "map_manip.hpp"
 
 #include "../display.hpp"
 #include "../events.hpp"
@@ -37,11 +40,13 @@
 
 namespace map_editor {
 
+bool check_data(std::string &data, std::string &filename, bool &from_scenario, config &game_cfg);
+
 /// A map editor. Receives SDL events and can execute hotkey commands.
 class map_editor : public events::handler,
 				   public hotkey::command_executor {
 public:
-	map_editor(display &gui, gamemap &map, config &theme, config &game_config);
+	map_editor(editor_display &gui, editormap &map, config &theme, config &game_config);
 	virtual ~map_editor();
 
 	/// Enter the main loop. The loop runs until set_abort() is called
@@ -49,7 +54,7 @@ public:
 	void main_loop();
 
 	/// Set the filename that map should be saved as.
-	void set_file_to_save_as(const std::string);
+	void set_file_to_save_as(const std::string, bool from_scenario);
 
 	/// How to abort the map editor.
 	/// DONT_ABORT is set during normal operation.
@@ -73,11 +78,6 @@ public:
 	/// Exception thrown when the loading of a map failed.
 	struct load_map_exception {};
 
-	/// Load the map from filename. Return the string representation of
-	/// the map. Show a message if the load failed. Throw
-	/// load_map_exception if the file could not be loaded.
-	std::string load_map(const std::string filename);
-
 	virtual void handle_event(const SDL_Event &event);
 
 	/// Handle a keyboard event. mousex and mousey is the current
@@ -116,6 +116,7 @@ public:
 	virtual void edit_set_start_pos();
 	virtual void edit_flood_fill();
 	virtual void edit_fill_selection();
+	virtual void edit_rotate_selection();
 	virtual void edit_cut();
 	virtual void edit_copy();
 	virtual void edit_paste();
@@ -127,19 +128,25 @@ public:
 	virtual void edit_select_all();
 	virtual void edit_draw();
 	virtual void edit_refresh();
+	virtual void edit_update();
+	virtual void edit_auto_update();
 
-	void perform_flood_fill();
+	void perform_flood_fill(const t_translation::t_terrain fill_with);
 	void perform_paste();
 	void perform_set_starting_pos();
 
-	virtual bool can_execute_command(hotkey::HOTKEY_COMMAND command) const;
+	virtual bool can_execute_command(hotkey::HOTKEY_COMMAND command, int) const;
+	//Does the action control a toggle switch? If so, return the state of the action (on or off)
+	virtual hotkey::ACTION_STATE get_action_state(hotkey::HOTKEY_COMMAND command) const;
+
 
 	/// Exception thrown when new map is to be loaded.
 	struct new_map_exception {
-		new_map_exception(const std::string &map, const std::string filename="")
-			: new_map(map), new_filename(filename) {}
+		new_map_exception(const std::string &map, const std::string filename="", const bool scenario = false)
+			: new_map(map), new_filename(filename), from_scenario(scenario) {}
 		const std::string new_map;
 		const std::string new_filename;
+		const bool from_scenario;
 	};
 
 private:
@@ -193,7 +200,7 @@ private:
 
 	/// Draw terrain at a location. The operation is saved in the undo
 	/// stack. Update the map to reflect the change.
-	void draw_terrain(const gamemap::TERRAIN terrain,
+	void draw_terrain(const t_translation::t_terrain terrain,
 					  const gamemap::location hex);
 
 
@@ -226,12 +233,10 @@ private:
 	/// Insert the currently selected locations in the clipboard.
 	void insert_selection_in_clipboard();
 
-	/// Return the hex with the given offset from loc. Make calculations
-	/// so the result with have the same _appearance_ as when the offset
-	/// was calculated ,not the same representation.
-	gamemap::location get_hex_with_offset(const gamemap::location loc,
-										  const int x_offset, const int y_offset);
-
+	/// Commit hexes filling.
+	void perform_fill_hexes(std::set<gamemap::location> &fill_hexes,
+		const t_translation::t_terrain terrain, map_undo_action &undo_action);
+	
 	/// Commit the movement of a selection.
 	void perform_selection_move();
 
@@ -246,17 +251,16 @@ private:
 	/// refreshed.
 	void clear_highlighted_hexes_in_gui();
 
+	// Sets the special overlay with the selection of fore and background
+	void set_mouseover_overlay();
+	// Clears the special overlay
+	void reset_mouseover_overlay() { gui_.clear_mouseover_hex_overlay(); }
+
 	/// Terrain has changed at the specified hex through user drawing
-	/// (not undo/redo or other special things). If the hex was a
-	/// starting position, remove this position. Save additional
-	/// information in the undo_action and invalidate the hex and the
-	/// adjacent ones.
-	void terrain_changed(const gamemap::location &hex,
-						 map_undo_action &undo_action);
-	void terrain_changed(const std::vector<gamemap::location> &hexes,
-						 map_undo_action &undo_action);
-	void terrain_changed(const std::set<gamemap::location> &hexes,
-						 map_undo_action &undo_action);
+	/// (not undo/redo or other special things).
+	void terrain_changed(const gamemap::location &hex);
+	void terrain_changed(const std::vector<gamemap::location> &hexes);
+	void terrain_changed(const std::set<gamemap::location> &hexes);
 
 	/// Save an action so that it may be undone. Add an operation to the
 	/// number done since save.
@@ -282,27 +286,35 @@ private:
 
 	/// Draw the terrain on the hexes the mouse is over, taking account
 	/// for brush size.
-	void draw_on_mouseover_hexes(const gamemap::TERRAIN t);
+	void draw_on_mouseover_hexes(const t_translation::t_terrain t);
 
 	// Load the tooltips for each button
 	void load_tooltips(void);
 
-	/// An item in the clipboard. Consists of the copied terrain and an
+	/// An item in the buffer. Consists of the copied terrain and an
 	/// offset. When pasting stuff, the offset is used to calculate
-	/// where to put the pasted hex when calculating from the one
-	/// selected when the paste takes place.
-	struct clipboard_item {
-		clipboard_item(int xo, int yo, gamemap::TERRAIN t, int start_side) :
-			x_offset(xo), y_offset(yo), terrain(t),
-			starting_side(start_side){}
-		int x_offset, y_offset;
-		gamemap::TERRAIN terrain;
+	/// where to put the pasted hex
+	struct buffer_item {
+		buffer_item(const gamemap::location &o, t_translation::t_terrain t, int start_side) :
+			offset(o), terrain(t), starting_side(start_side) {}
+		gamemap::location offset;
+		t_translation::t_terrain terrain;
 		int starting_side;
 	};
 
-	display &gui_;
-	gamemap &map_;
-	std::string filename_;
+	// map_buffer is used for clipboard and other map operations
+	typedef std::vector<buffer_item> map_buffer;
+
+	void copy_buffer(map_buffer& buffer, const std::set<gamemap::location> &locs,
+			 const gamemap::location &origin);
+ 	void paste_buffer(const map_buffer& buffer, const gamemap::location &loc,
+ 			map_undo_action &undo_action);
+ 	void clear_buffer(map_buffer& buffer) {buffer.clear();};
+
+	editor_display &gui_;
+	editormap &map_;
+	std::string filename_, original_filename_;
+	bool from_scenario_;
 	ABORT_MODE abort_;
 	// Keep track of the number of operations performed since the last
 	// save. If this is zero when the editor is exited there is no need
@@ -315,8 +327,9 @@ private:
 	gamemap::location selected_hex_;
 	// When map_dirty_ is true, schedule redraw of the minimap and
 	// perform some updates like recalculating labels of starting
-	// positions.
+	// positions. But only it the option auto_update is enabled
 	bool map_dirty_;
+	bool auto_update_;
 	bool l_button_palette_dirty_;
 	bool everything_dirty_;
 	terrain_palette palette_;
@@ -324,8 +337,8 @@ private:
 	std::vector<gamemap::location> starting_positions_;
 	std::set<gamemap::location> mouse_over_hexes_;
 	std::set<gamemap::location> selected_hexes_;
-	std::vector<clipboard_item> clipboard_;
-	gamemap::location clipboard_offset_loc_;
+	map_buffer clipboard_;
+	
 	LEFT_BUTTON_HELD_FUNC l_button_held_func_;
 	gamemap::location selection_move_start_;
 
@@ -341,7 +354,7 @@ private:
 	static config hotkeys_;
 	static bool first_time_created_;
 	static LEFT_BUTTON_FUNC l_button_func_;
-	static gamemap::TERRAIN old_fg_terrain_, old_bg_terrain_;
+	static t_translation::t_terrain old_fg_terrain_, old_bg_terrain_;
 	static int old_brush_size_;
 	bool all_hexes_selected_;
 
