@@ -1,23 +1,26 @@
-/* $Id: filesystem.cpp 16920 2007-04-18 21:38:48Z noyga $ */
+/* $Id: filesystem.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file filesystem.cpp 
+//! File-IO
+
 #include "global.hpp"
 
-#include "loadscreen.hpp"
-
-//include files for opendir(3), readdir(3), etc. These files may vary
-//from platform to platform, since these functions are NOT ANSI-conforming
-//functions. They may have to be altered to port to new platforms
+// Include files for opendir(3), readdir(3), etc. 
+// These files may vary from platform to platform, 
+// since these functions are NOT ANSI-conforming functions. 
+// They may have to be altered to port to new platforms
 #include <sys/types.h>
 
 //for mkdir
@@ -26,21 +29,234 @@
 
 #ifdef _WIN32
 
+/* /////////////////////////////////////////////////////////////////////////
+ * This code swiped from dirent.c in the unixem library, version 1.7.3.
+ * See http://synesis.com.au/software/unixem.html for full sources.
+ * It's under BSD license.
+ */
+
 #include <direct.h>
-
 #include <io.h>
+#include <errno.h>
+#include <stdlib.h>
+#include <windows.h>
 
-//#define mkdir(a,b) (_mkdir(a))
 
-#define mode_t int
+/* /////////////////////////////////////////////////////////////////////////
+ * Compiler differences
+ */
 
+#if defined(__BORLANDC__)
+# define DIRENT_PROVIDED_BY_COMPILER
+#elif defined(__DMC__)
+# define DIRENT_PROVIDED_BY_COMPILER
+#elif defined(__GNUC__)
+# define DIRENT_PROVIDED_BY_COMPILER
+#elif defined(__INTEL_COMPILER)
+#elif defined(_MSC_VER)
+#elif defined(__MWERKS__)
+#elif defined(__WATCOMC__)
 #else
+# error Compiler not discriminated
+#endif /* compiler */
+
+#if defined(DIRENT_PROVIDED_BY_COMPILER)
+#include <dirent.h>
+#else
+
+/* ////////////////////////////////////////////////////////////////////// */
+
+#include <stddef.h>
+
+#ifndef NAME_MAX
+# define NAME_MAX   (260)
+#endif /* !NAME_MAX */
+
+struct dirent
+{
+	char	d_name[NAME_MAX + 1];	   /*!< file name (null-terminated) */
+	int 	d_mode;
+};
+
+struct DIR
+{
+	char			  directory[_MAX_DIR+1];   /* . */
+	WIN32_FIND_DATAA  find_data;			   /* The Win32 FindFile data. */
+	HANDLE			  hFind;				   /* The Win32 FindFile handle. */
+	struct dirent	  dirent;				   /* The handle's entry. */
+};
+
+#ifndef FILE_ATTRIBUTE_ERROR
+# define FILE_ATTRIBUTE_ERROR			(0xFFFFFFFF)
+#endif /* FILE_ATTRIBUTE_ERROR */
+
+/* /////////////////////////////////////////////////////////////////////////
+ * Helper functions
+ */
+
+static HANDLE dirent__findfile_directory(char const *name, LPWIN32_FIND_DATAA data)
+{
+	char	search_spec[_MAX_PATH +1];
+
+	// Simply add the *.*, ensuring the path separator is included.
+	(void)lstrcpyA(search_spec, name);
+	if( '\\' != search_spec[lstrlenA(search_spec) - 1] &&
+		'/' != search_spec[lstrlenA(search_spec) - 1])
+	{
+		(void)lstrcatA(search_spec, "\\*.*");
+	}
+	else
+	{
+		(void)lstrcatA(search_spec, "*.*");
+	}
+
+	return FindFirstFileA(search_spec, data);
+}
+
+/* /////////////////////////////////////////////////////////////////////////
+ * API functions
+ */
+
+DIR *opendir(char const *name)
+{
+	DIR 	*result =	NULL;
+	DWORD	dwAttr;
+
+	// Must be a valid name 
+	if( !name ||
+		!*name ||
+		(dwAttr = GetFileAttributes(name)) == 0xFFFFFFFF)
+	{
+		errno = ENOENT;
+	}
+	// Must be a directory 
+	else if(!(dwAttr & FILE_ATTRIBUTE_DIRECTORY))
+	{
+		errno = ENOTDIR;
+	}
+	else
+	{
+		result = (DIR*)malloc(sizeof(DIR));
+
+		if(result == NULL)
+		{
+			errno = ENOMEM;
+		}
+		else
+		{
+			result->hFind=dirent__findfile_directory(name, &result->find_data);
+
+			if(result->hFind == INVALID_HANDLE_VALUE)
+			{
+				free(result);
+
+				result = NULL;
+			}
+			else
+			{
+				// Save the directory, in case of rewind. 
+				(void)lstrcpyA(result->directory, name);
+				(void)lstrcpyA(result->dirent.d_name, result->find_data.cFileName);
+				result->dirent.d_mode	=	(int)result->find_data.dwFileAttributes;
+			}
+		}
+	}
+
+	return result;
+}
+
+int closedir(DIR *dir)
+{
+	int ret;
+
+	if(dir == NULL)
+	{
+		errno = EBADF;
+
+		ret = -1;
+	}
+	else
+	{
+		// Close the search handle, if not already done.
+		if(dir->hFind != INVALID_HANDLE_VALUE)
+		{
+			(void)FindClose(dir->hFind);
+		}
+
+		free(dir);
+
+		ret = 0;
+	}
+
+	return ret;
+}
+
+struct dirent *readdir(DIR *dir)
+{
+	// The last find exhausted the matches, so return NULL.
+	if(dir->hFind == INVALID_HANDLE_VALUE)
+	{
+		if(FILE_ATTRIBUTE_ERROR == dir->find_data.dwFileAttributes)
+		{
+			errno = EBADF;
+		}
+		else
+		{
+			dir->find_data.dwFileAttributes = FILE_ATTRIBUTE_ERROR;
+		}
+
+		return NULL;
+	}
+	else
+	{
+		// Copy the result of the last successful match to dirent.
+		(void)lstrcpyA(dir->dirent.d_name, dir->find_data.cFileName);
+
+		// Attempt the next match.
+		if(!FindNextFileA(dir->hFind, &dir->find_data))
+		{
+			// Exhausted all matches, so close and null the handle.
+			(void)FindClose(dir->hFind);
+			dir->hFind = INVALID_HANDLE_VALUE;
+		}
+
+		return &dir->dirent;
+	}
+}
+
+/*
+ * Microsoft C uses _stat instead of stat,
+ * for both the function name and the structure name.
+ * See <http://svn.ghostscript.com:8080/ghostscript/trunk/gs/src/stat_.h>
+ */
+#ifdef _MSC_VER
+#  define stat _stat
+namespace {
+	typedef int mode_t;
+}
+#endif
+
+#ifndef S_IFMT
+#define S_IFMT	(S_IFDIR|S_IFREG)
+#endif
+#ifndef S_ISREG
+#define S_ISREG(x) (((x) & S_IFMT) == S_IFREG)
+#endif
+#ifndef S_ISDIR
+#define S_ISDIR(x) (((x) & S_IFMT) == S_IFDIR)
+#endif
+
+#endif /* !DIRENT_PROVIDED_BY_COMPILER */
+
+#define mkdir(a,b) (_mkdir(a))
+
+#else /* !_WIN32 */
 
 #include <unistd.h>
 
 #include <dirent.h>
 
-#endif
+#endif /* !_WIN32 */
 
 #ifdef __BEOS__
 #include <Directory.h>
@@ -49,7 +265,7 @@
 BPath be_path;
 #endif
 
-//for getenv
+// for getenv
 #include <cstdlib>
 #include <cerrno>
 #include <cstring>
@@ -67,206 +283,175 @@ BPath be_path;
 #include "log.hpp"
 #include "scoped_resource.hpp"
 #include "util.hpp"
+#include "loadscreen.hpp"
 
+#define DBG_FS LOG_STREAM(debug, filesystem)
 #define LOG_FS LOG_STREAM(info, filesystem)
 #define ERR_FS LOG_STREAM(err, filesystem)
 
-#ifdef USE_ZIPIOS
-#include <zipios++/collcoll.h>
-#include <zipios++/dircoll.h>
-#include <zipios++/zipfile.h>
-#include <zipios++/simplesmartptr.h>
-#include <zipios++/basicentry.h>
-#include "zipios++/xcoll.hpp"
-
-namespace {
-	xzipios::XCColl* the_collection = NULL;
-
-	void register_zipdir(const std::string directory) {
-		// look for zip files
-		zipios::DirectoryCollection dir(directory,false);
-		zipios::ConstEntries entries = dir.entries();
-		for (zipios::ConstEntries::iterator i = entries.begin(); i != entries.end(); ++i) {
-			if ((**i).isValid()) {
-				const std::string fname = (**i).getName();
-				const std::string suffix = ".zip";
-				if ((fname.size() > suffix.size()) &&
-				    (0 == fname.compare(fname.size() - suffix.size(), suffix.size(), suffix))) {
-					zipios::ZipFile zip(game_config::path + "/" + fname);
-					the_collection->addCollection(zip);
-					LOG_FS << "zip collection " << fname
-					       << " has " << zip.size() << " elements\n";
-				}
-			}
-			else LOG_FS << "skipping invalid entry\n";
-		}
-
-	}
-}
-#endif
-
-bool filesystem_init()
-{
-#ifdef USE_ZIPIOS
-	if (the_collection != NULL) {
-		// this is a re-read, cleanup first !
-		free (the_collection);
-	}
-
-	the_collection = new xzipios::XCColl;
-
-	if (!get_user_data_dir().empty()) {
-		LOG_FS << "looking at user dir " << get_user_data_dir() << '\n';
-		zipios::DirectoryCollection dir(get_user_data_dir());
-		LOG_FS << "user collection has " << dir.size() << " elements\n";
-		the_collection->addCollection(dir);
-		register_zipdir(get_user_data_dir());
-	}
-	if (!game_config::path.empty()) {
-		LOG_FS << "looking at system dir " << game_config::path << '\n';
-		zipios::DirectoryCollection dir(game_config::path);
-		LOG_FS << "system collection has " << dir.size() << " elements\n";
-		the_collection->addCollection(dir);
-		register_zipdir(game_config::path);
-	}
-#endif
-	return true;
-}
-
 namespace {
 	const mode_t AccessMode = 00770;
 }
 
-#ifdef _WIN32
-#define DIR_INVALID(d) (d == -1)
-#else
-#define DIR_INVALID(d) (d == NULL)
-#endif
-
 #ifdef __APPLE__
 #include <CoreFoundation/CoreFoundation.h>
 #include <CoreFoundation/CFString.h>
 #include <CoreFoundation/CFBase.h>
 #endif
 
-void get_files_in_dir(const std::string& directory,
-                      std::vector<std::string>* files,
-                      std::vector<std::string>* dirs,
-                      FILE_NAME_MODE mode)
+bool ends_with(const std::string& str, const std::string& suffix)
 {
-#ifdef USE_ZIPIOS
-	if (the_collection->hasSubdir(directory)) {
-		the_collection->childrenOf(directory, files, dirs);
-	}
-	else
-#endif
-	{
+	return str.size() >= suffix.size() && std::equal(suffix.begin(),suffix.end(),str.end()-suffix.size());
+}
 
-		//if we have a path to find directories in, then convert relative
-		//pathnames to be rooted on the wesnoth path
+// These are the filenames that get special processing
+#define MAINCFG "_main.cfg"
+#define FINALCFG	"_final.cfg"
+
+// Don't pass directory as reference, it seems to break on 
+// arklinux with GCC-4.3.
+void get_files_in_dir(const std::string directory,
+					  std::vector<std::string>* files,
+					  std::vector<std::string>* dirs,
+					  FILE_NAME_MODE mode,
+					  FILE_FILTER filter,
+					  FILE_REORDER_OPTION reorder)
+{
+	// If we have a path to find directories in, 
+	// then convert relative pathnames to be rooted 
+	// on the wesnoth path
 #ifndef __AMIGAOS4__
-		if(!directory.empty() && directory[0] != '/' && !game_config::path.empty()){
-			const std::string& dir = game_config::path + "/" + directory;
-			if(is_directory(dir)) {
-				get_files_in_dir(dir,files,dirs,mode);
-				return;
-			}
-		}
-#endif
-
-#ifdef _WIN32
-		_finddata_t fileinfo;
-		long dir = _findfirst((directory + "/*.*").c_str(),&fileinfo);
-#else
-
-		DIR* dir = opendir(directory.c_str());
-#endif
-
-		if(DIR_INVALID(dir)) {
+	if(!directory.empty() && directory[0] != '/' && !game_config::path.empty()){
+		const std::string& dir = game_config::path + "/" + directory;
+		if(is_directory(dir)) {
+			get_files_in_dir(dir,files,dirs,mode,filter,reorder);
 			return;
 		}
+	}
+#endif /* __AMIGAOS4__ */
 
-#ifdef _WIN32
+	struct stat st;
 
-		int res = dir;
-		do {
-			if(fileinfo.name[0] != '.') {
-				const std::string path = (mode == ENTIRE_FILE_PATH ?
-							  directory + "/" : std::string("")) + fileinfo.name;
+	if (reorder == DO_REORDER) {
+		LOG_FS << "searching _main.cfg in directory " << directory << '\n';
+		std::string maincfg;
+		if (directory.empty() || directory[directory.size()-1] == '/'
+#ifdef __AMIGAOS4__
+			|| (directory[directory.size()-1]==':')
+#endif /* __AMIGAOS4__ */
+		)
+			maincfg = directory + MAINCFG;
+		else
+			maincfg = (directory + "/") + MAINCFG;
 
-				if(fileinfo.attrib&_A_SUBDIR) {
-					if(dirs != NULL)
-						dirs->push_back(path);
-				} else {
-					if(files != NULL)
-						files->push_back(path);
-				}
+		if (::stat(maincfg.c_str(), &st) != -1) {
+			LOG_FS << "_main.cfg found : " << maincfg << '\n';
+			if (files != NULL) {
+				if (mode == ENTIRE_FILE_PATH)
+					files->push_back(maincfg);
+				else
+					files->push_back(MAINCFG);
 			}
+			return;
+		}
+	}
 
-			res = _findnext(dir,&fileinfo);
-		} while(!DIR_INVALID(res));
+	DIR* dir = opendir(directory.c_str());
 
-		_findclose(dir);
+	if(dir == NULL) {
+		return;
+	}
 
-#else
-		struct dirent* entry;
-		while((entry = readdir(dir)) != NULL) {
-			if(entry->d_name[0] == '.')
-				continue;
+	struct dirent* entry;
+	while((entry = readdir(dir)) != NULL) {
+		if(entry->d_name[0] == '.')
+			continue;
 #ifdef __APPLE__
-		  /* HFS Mac OS X decompose filenames using combining unicode
-		  characters. Try to get the precomposed form.
-		  */
-		  char filename[MAXNAMLEN+1];
-		  CFStringRef cstr = CFStringCreateWithCString(NULL, entry->d_name,
-							       kCFStringEncodingUTF8);
-		  CFMutableStringRef mut_str = CFStringCreateMutableCopy(NULL, 0, cstr);
-		  CFStringNormalize(mut_str, kCFStringNormalizationFormC);
-		  CFStringGetCString(mut_str,filename,sizeof(filename)-1,kCFStringEncodingUTF8);
-		  CFRelease(cstr);
-		  CFRelease(mut_str);
+		// HFS Mac OS X decomposes filenames using combining unicode characters. 
+		// Try to get the precomposed form.
+		char macname[MAXNAMLEN+1];
+		CFStringRef cstr = CFStringCreateWithCString(NULL,
+							 entry->d_name,
+							 kCFStringEncodingUTF8);
+		CFMutableStringRef mut_str = CFStringCreateMutableCopy(NULL,
+							 0, cstr);
+		CFStringNormalize(mut_str, kCFStringNormalizationFormC);
+		CFStringGetCString(mut_str,
+				macname,sizeof(macname)-1,
+				kCFStringEncodingUTF8);
+		CFRelease(cstr);
+		CFRelease(mut_str);
+		const std::string basename = macname;
 #else
-		  char *filename = entry->d_name;
-#endif
-#ifndef __AMIGAOS4__
-			const std::string name((directory + "/") + filename);
-#else
-			std::string name;
-			if (directory.empty() || directory[directory.size()-1] == ':')
-				name = directory + filename;
-			else
-				name = (directory + "/") + filename;
-#endif
+		// generic Unix 
+		const std::string basename = entry->d_name;
+#endif /* !APPLE */
 
-			struct stat st;
-			if (::stat(name.c_str(), &st) != -1) {
-				if (S_ISREG(st.st_mode)) {
+		std::string fullname;
+		if (directory.empty() || directory[directory.size()-1] == '/'
+#ifdef __AMIGAOS4__
+			|| (directory[directory.size()-1]==':')
+#endif /* __AMIGAOS4__ */
+		)
+			fullname = directory + basename;
+		else
+			fullname = directory + "/" + basename;
+
+		if (::stat(fullname.c_str(), &st) != -1) {
+			if (S_ISREG(st.st_mode)) {
+				if (files != NULL) {
+					if (mode == ENTIRE_FILE_PATH)
+						files->push_back(fullname);
+					else
+						files->push_back(basename);
+				}
+			} else if (S_ISDIR(st.st_mode)) {
+				if (filter == SKIP_MEDIA_DIR
+						&& (basename == "images"|| basename == "sounds"))
+					continue;
+			
+				if (reorder == DO_REORDER &&
+						::stat((fullname+"/"+MAINCFG).c_str(), &st)!=-1 &&
+						S_ISREG(st.st_mode)) {
+					LOG_FS << "_main.cfg found : ";
 					if (files != NULL) {
-						if (mode == ENTIRE_FILE_PATH)
-							files->push_back(name);
-						else
-							files->push_back(filename);
+						if (mode == ENTIRE_FILE_PATH) {
+							files->push_back(fullname + "/" + MAINCFG);
+							LOG_FS << fullname << "/" << MAINCFG << '\n';
+						} else {
+							files->push_back(basename + "/" + MAINCFG);
+							LOG_FS << basename << "/" << MAINCFG << '\n';
 					}
-				} else if (S_ISDIR(st.st_mode)) {
-					if (dirs != NULL) {
-						if (mode == ENTIRE_FILE_PATH)
-							dirs->push_back(name);
-						else
-							dirs->push_back(filename);
+					} else {
+					// Show what I consider strange
+						LOG_FS << fullname << "/" << MAINCFG << " not used now but skip the directory \n";
 					}
+				} else if (dirs != NULL) {
+					if (mode == ENTIRE_FILE_PATH)
+						dirs->push_back(fullname);
+					else
+						dirs->push_back(basename);
 				}
 			}
 		}
-
-		closedir(dir);
-#endif
 	}
 
+	closedir(dir);
+
 	if(files != NULL)
 		std::sort(files->begin(),files->end());
 
-	if(dirs != NULL)
+	if (dirs != NULL)
 		std::sort(dirs->begin(),dirs->end());
+
+	if (files != NULL && reorder == DO_REORDER) {
+		for (unsigned int i = 0; i < files->size(); i++)
+			if (ends_with((*files)[i], FINALCFG)) {
+				files->push_back((*files)[i]);
+				files->erase(files->begin()+i);
+				break;
+			}
+	}
 }
 
 std::string get_prefs_file()
@@ -324,10 +509,6 @@ std::string get_upload_dir()
 
 std::string get_dir(const std::string& dir_path)
 {
-#ifdef _WIN32
-	_mkdir(dir_path.c_str());
-#else
-
 	DIR* dir = opendir(dir_path.c_str());
 	if(dir == NULL) {
 		const int res = mkdir(dir_path.c_str(),AccessMode);
@@ -342,22 +523,17 @@ std::string get_dir(const std::string& dir_path)
 		return "";
 
 	closedir(dir);
-#endif
 
 	return dir_path;
 }
 
-void make_directory(const std::string& path)
+bool make_directory(const std::string& path)
 {
-#ifdef _WIN32
-	_mkdir(path.c_str());
-#else
-	mkdir(path.c_str(),AccessMode);
-#endif
+	return (mkdir(path.c_str(),AccessMode) == 0);
 }
 
-//this deletes a directory with no hidden files and subdirectories
-//also deletes a single file
+// This deletes a directory with no hidden files and subdirectories.
+// Also deletes a single file.
 bool delete_directory(const std::string& path)
 {
 	bool ret = true;
@@ -478,7 +654,7 @@ std::string get_user_data_dir()
 	if(dir == NULL) {
 		const int res = mkdir(dir_path.c_str(),AccessMode);
 
-		//also create the maps directory
+		// Also create the maps directory
 		mkdir((dir_path + "/editor").c_str(),AccessMode);
 		mkdir((dir_path + "/editor/maps").c_str(),AccessMode);
 		mkdir((dir_path + "/data").c_str(),AccessMode);
@@ -504,29 +680,16 @@ std::string get_user_data_dir()
 #endif
 }
 
-std::string read_stream(std::istream& s)
+static std::string read_stream(std::istream& s)
 {
 	std::stringstream ss;
 	ss << s.rdbuf();
 	return ss.str();
 }
 
-std::string read_stdin()
-{
-	return read_stream(std::cin);
-}
-
 std::istream *istream_file(std::string const &fname)
 {
 	LOG_FS << "streaming " << fname << " for reading.\n";
-#ifdef USE_ZIPIOS
-	if (!fname.empty() && fname[0] != '/' && the_collection) {
-		zipios::ConstEntryPointer p = the_collection->getEntry(fname);
-		if (p != 0)
-			if (std::istream *s = the_collection->getInputStream(p))
-				return s;
-	}
-#else
 	if (!fname.empty() && fname[0] != '/' && !game_config::path.empty()) {
 		std::ifstream *s = new std::ifstream((game_config::path + "/" + fname).c_str(),std::ios_base::binary);
 		if (s->is_open())
@@ -534,10 +697,10 @@ std::istream *istream_file(std::string const &fname)
 		LOG_FS << "could not open " << fname << " for reading.\n";
 		delete s;
 	}
-#endif
 
-	// FIXME: why do we rely on this even with relative paths ?
-	// still useful with zipios, for things like cache and prefs
+	//! @todo FIXME: why do we rely on this even with relative paths ?
+	// Still useful with zipios, for things like cache and prefs.
+	// NOTE zipios has been removed - not sure what to do with this code.
 	std::istream *s = new std::ifstream(fname.c_str(), std::ios_base::binary);
 	if (s->fail())
 		LOG_FS << "streaming " << fname << " failed.\n";
@@ -556,7 +719,7 @@ std::ostream *ostream_file(std::string const &fname)
 	return new std::ofstream(fname.c_str(), std::ios_base::binary);
 }
 
-//throws io_exception if an error occurs
+// Throws io_exception if an error occurs
 void write_file(const std::string& fname, const std::string& data)
 {
 	//const util::scoped_resource<FILE*,close_FILE> file(fopen(fname.c_str(),"wb"));
@@ -593,9 +756,7 @@ std::string read_map(const std::string& name)
 	return res;
 }
 
-namespace {
-
-bool is_directory_internal(const std::string& fname)
+static bool is_directory_internal(const std::string& fname)
 {
 #ifdef _WIN32
 	_finddata_t info;
@@ -617,36 +778,27 @@ bool is_directory_internal(const std::string& fname)
 #endif
 }
 
-}
-
 bool is_directory(const std::string& fname)
 {
-#ifdef USE_ZIPIOS
-	if(!fname.empty() && fname[0] != '/') {
-		return the_collection->hasSubdir(fname);
+	if(fname.empty()) {
+		return false;
 	}
-
-#else
-	if(!fname.empty() && fname[0] != '/' && !game_config::path.empty()) {
+	if(fname[0] != '/' && !game_config::path.empty()) {
 		if(is_directory_internal(game_config::path + "/" + fname))
 			return true;
 	}
-#endif
 
 	return is_directory_internal(fname);
 }
 
 bool file_exists(const std::string& name)
 {
-#ifdef USE_ZIPIOS
-	if (the_collection->getEntry(name))
-		return true;
-#endif
 	std::ifstream file(name.c_str(),std::ios_base::binary);
-	if (file.rdstate() != 0)
-	        return false;
+	if (file.rdstate() != 0) {
+		return false;
+	}
 	file.close();
-        return true;
+	return true;
 }
 
 time_t file_create_time(const std::string& fname)
@@ -658,7 +810,7 @@ time_t file_create_time(const std::string& fname)
 	return buf.st_mtime;
 }
 
-//return the next ordered full filename within this directory
+//! Return the next ordered full filename within this directory.
 std::string next_filename(const std::string &dirname, unsigned int max)
 {
 	std::vector<std::string> files;
@@ -691,15 +843,24 @@ std::string next_filename(const std::string &dirname, unsigned int max)
 	return dirname + "/" + fname.str();
 }
 
+//! Returns true if the file ends with '.gz'.
+//! 
+//! @param filename     The name to test.
+bool is_gzip_file(const std::string& filename)
+{ 
+	return (filename.length() > 3 
+		&& filename.substr(filename.length() - 3) == ".gz"); 
+}
+
 file_tree_checksum::file_tree_checksum()
-    : nfiles(0), sum_size(0), modified(0)
+	: nfiles(0), sum_size(0), modified(0)
 {}
 
-file_tree_checksum::file_tree_checksum(const config& cfg)
+file_tree_checksum::file_tree_checksum(const config& cfg) :
+	nfiles	(lexical_cast_default<size_t>(cfg["nfiles"])),
+	sum_size(lexical_cast_default<size_t>(cfg["size"])),
+	modified(lexical_cast_default<time_t>(cfg["modified"]))
 {
-	nfiles = lexical_cast_default<size_t>(cfg["nfiles"]);
-	sum_size = lexical_cast_default<size_t>(cfg["size"]);
-	modified = lexical_cast_default<time_t>(cfg["modified"]);
 }
 
 void file_tree_checksum::write(config& cfg) const
@@ -712,7 +873,7 @@ void file_tree_checksum::write(config& cfg) const
 bool operator==(const file_tree_checksum& lhs, const file_tree_checksum& rhs)
 {
 	return lhs.nfiles == rhs.nfiles && lhs.sum_size == rhs.sum_size &&
-	       lhs.modified == rhs.modified;
+		   lhs.modified == rhs.modified;
 }
 
 bool operator!=(const file_tree_checksum& lhs, const file_tree_checksum& rhs)
@@ -720,12 +881,10 @@ bool operator!=(const file_tree_checksum& lhs, const file_tree_checksum& rhs)
 	return !operator==(lhs,rhs);
 }
 
-namespace {
-
-void get_file_tree_checksum_internal(const std::string& path, file_tree_checksum& res)
+static void get_file_tree_checksum_internal(const std::string& path, file_tree_checksum& res)
 {
 	std::vector<std::string> files, dirs;
-	get_files_in_dir(path,&files,&dirs,ENTIRE_FILE_PATH);
+	get_files_in_dir(path,&files,&dirs, ENTIRE_FILE_PATH, SKIP_MEDIA_DIR);
 	increment_filesystem_progress();
 	for(std::vector<std::string>::const_iterator i = files.begin(); i != files.end(); ++i) {
 		++res.nfiles;
@@ -745,15 +904,6 @@ void get_file_tree_checksum_internal(const std::string& path, file_tree_checksum
 	}
 }
 
-} //end anonymous namespace
-
-file_tree_checksum get_file_tree_checksum(const std::string& path)
-{
-	file_tree_checksum res;
-	get_file_tree_checksum_internal(path,res);
-	return res;
-}
-
 const file_tree_checksum& data_tree_checksum()
 {
 	static file_tree_checksum checksum;
@@ -761,8 +911,8 @@ const file_tree_checksum& data_tree_checksum()
 		get_file_tree_checksum_internal("data/",checksum);
 		get_file_tree_checksum_internal(get_user_data_dir() + "/data/",checksum);
 		LOG_FS << "calculated data tree checksum: "
-		       << checksum.nfiles << " files; "
-		       << checksum.sum_size << " bytes\n";
+			   << checksum.nfiles << " files; "
+			   << checksum.sum_size << " bytes\n";
 	}
 
 	return checksum;
@@ -778,6 +928,7 @@ int file_size(const std::string& fname)
 }
 
 std::string file_name(const std::string& file)
+// Analogous to POSIX basename(3), but for C++ string-object pathnames
 {
 #ifdef _WIN32
 	static const std::string dir_separators = "\\/:";
@@ -796,6 +947,7 @@ std::string file_name(const std::string& file)
 }
 
 std::string directory_name(const std::string& file)
+// Analogous to POSIX dirname(3), but for C++ string-object pathnames
 {
 #ifdef _WIN32
 	static const std::string dir_separators = "\\/:";
@@ -818,19 +970,19 @@ std::set<std::string> binary_paths;
 typedef std::map<std::string,std::vector<std::string> > paths_map;
 paths_map binary_paths_cache;
 
-void init_binary_paths()
+}
+
+static void init_binary_paths()
 {
 	if(binary_paths.empty()) {
 		binary_paths.insert("");
 	}
 }
 
-}
-
-binary_paths_manager::binary_paths_manager()
+binary_paths_manager::binary_paths_manager() : paths_()
 {}
 
-binary_paths_manager::binary_paths_manager(const config& cfg)
+binary_paths_manager::binary_paths_manager(const config& cfg) : paths_()
 {
 	set_paths(cfg);
 }
@@ -847,7 +999,8 @@ void binary_paths_manager::set_paths(const config& cfg)
 
 	const config::child_list& items = cfg.get_children("binary_path");
 	for(config::child_list::const_iterator i = items.begin(); i != items.end(); ++i) {
-		const std::string path = (**i)["path"].str() + "/";
+		std::string path = (**i)["path"].str();
+		if (!path.empty() && path[path.size()-1] != '/') path += "/";
 		if(binary_paths.count(path) == 0) {
 			binary_paths.insert(path);
 			paths_.push_back(path);
@@ -896,20 +1049,32 @@ const std::vector<std::string>& get_binary_paths(const std::string& type)
 std::string get_binary_file_location(const std::string& type, const std::string& filename)
 {
 	const std::vector<std::string>& paths = get_binary_paths(type);
+	if(!filename.empty()) {
+		DBG_FS << "Looking for " << filename << " in  '.'\n";
+		if(file_exists(filename) || is_directory(filename)) {
+		  DBG_FS << "  Found at " << filename << "\n";
+			return filename;
+		}
+	}
+
 	for(std::vector<std::string>::const_iterator i = paths.begin(); i != paths.end(); ++i) {
 		const std::string file = *i + filename;
+		DBG_FS << "  Checking " << *i << "\n";
 		if(file_exists(file) || is_directory(file)) {
+		  DBG_FS << "  Found at " << file << "\n";
 			return file;
 		}
 	}
 
+	DBG_FS << "  " << filename << " not found.\n";
 	return "";
 }
 
-void scoped_istream::operator=(std::istream *s)
+scoped_istream& scoped_istream::operator=(std::istream *s)
 {
 	delete stream;
 	stream = s;
+	return *this;
 }
 
 scoped_istream::~scoped_istream()
@@ -917,10 +1082,11 @@ scoped_istream::~scoped_istream()
 	delete stream;
 }
 
-void scoped_ostream::operator=(std::ostream *s)
+scoped_ostream& scoped_ostream::operator=(std::ostream *s)
 {
 	delete stream;
 	stream = s;
+	return *this;
 }
 
 scoped_ostream::~scoped_ostream()
