@@ -1,11 +1,12 @@
-/* $Id: font.cpp 15743 2007-02-13 07:13:20Z sapient $ */
+/* $Id: font.cpp 23985 2008-02-21 08:15:14Z mythological $ */
 /* vim:set encoding=utf-8: */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -24,10 +25,10 @@
 #include "tooltips.hpp"
 #include "util.hpp"
 #include "video.hpp"
-#include "wassert.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
 #include "serialization/string_utils.hpp"
+#include "marked-up_text.hpp"
 
 #include <algorithm>
 #include <cstdio>
@@ -39,6 +40,7 @@
 #include <stack>
 #include <string>
 
+#define DBG_FT LOG_STREAM(debug, display)
 #define LOG_FT LOG_STREAM(info, display)
 #define WRN_FT LOG_STREAM(warn, display)
 #define ERR_FT LOG_STREAM(err, display)
@@ -61,7 +63,6 @@
 
 namespace {
 
-
 // Signed int. Negative values mean "no subset".
 typedef int subset_id;
 
@@ -108,8 +109,10 @@ typedef std::map<std::string,SDL_Rect> line_size_cache_map;
 //map of styles -> sizes -> cache
 std::map<int,std::map<int,line_size_cache_map> > line_size_cache;
 
+}
+
 //Splits the UTF-8 text into text_chunks using the same font.
-std::vector<text_chunk> split_text(std::string const & utf8_text) {
+static std::vector<text_chunk> split_text(std::string const & utf8_text) {
 	text_chunk current_chunk(0);
 	std::vector<text_chunk> chunks;
 
@@ -132,7 +135,7 @@ std::vector<text_chunk> split_text(std::string const & utf8_text) {
 				current_chunk.ucs2_text.clear();
 				current_chunk.subset = font_map[size_t(*ch)];
 			}
-			current_chunk.ucs2_text.push_back((Uint16)*ch);
+			current_chunk.ucs2_text.push_back(static_cast<Uint16>(*ch));
 			current_chunk.text.append(ch.substr().first, ch.substr().second);
 		}
 		if (!current_chunk.text.empty()) {
@@ -145,7 +148,7 @@ std::vector<text_chunk> split_text(std::string const & utf8_text) {
 	return chunks;
 }
 
-TTF_Font* open_font(const std::string& fname, int size)
+static TTF_Font* open_font(const std::string& fname, int size)
 {
 	std::string name;
 	if(!game_config::path.empty()) {
@@ -181,7 +184,7 @@ TTF_Font* open_font(const std::string& fname, int size)
 	return font;
 }
 
-TTF_Font* get_font(font_id id)
+static TTF_Font* get_font(font_id id)
 {
 	const std::map<font_id, TTF_Font*>::iterator it = font_table.find(id);
 	if(it != font_table.end())
@@ -202,7 +205,7 @@ TTF_Font* get_font(font_id id)
 	return font;
 }
 
-void clear_fonts()
+static void clear_fonts()
 {
 	for(std::map<font_id,TTF_Font*>::iterator i = font_table.begin(); i != font_table.end(); ++i) {
 		TTF_CloseFont(i->second);
@@ -214,6 +217,8 @@ void clear_fonts()
 	line_size_cache.clear();
 }
 
+namespace {
+
 struct font_style_setter
 {
 	font_style_setter(TTF_Font* font, int style) : font_(font), old_style_(0)
@@ -277,7 +282,16 @@ manager::~manager()
 	TTF_Quit();
 }
 
-void set_font_list(const std::vector<subset_descriptor>& fontlist)
+//structure used to describe a font, and the subset of the Unicode character
+//set it covers.
+struct subset_descriptor
+{
+	std::string name;
+	std::vector<std::pair<size_t, size_t> > present_codepoints;
+};
+
+//sets the font list to be used.
+static void set_font_list(const std::vector<subset_descriptor>& fontlist)
 {
 	clear_fonts();
 	font_map.reserve(0x10000);
@@ -322,15 +336,17 @@ void set_font_list(const std::vector<subset_descriptor>& fontlist)
 }
 
 const SDL_Color NORMAL_COLOUR = {0xDD,0xDD,0xDD,0},
+                GRAY_COLOUR   = {0x77,0x77,0x77,0},
                 LOBBY_COLOUR  = {0xBB,0xBB,0xBB,0},
                 GOOD_COLOUR   = {0x00,0xFF,0x00,0},
                 BAD_COLOUR    = {0xFF,0x00,0x00,0},
                 BLACK_COLOUR  = {0x00,0x00,0x00,0},
-                DARK_COLOUR   = {0x00,0x00,0x66,0},
                 YELLOW_COLOUR = {0xFF,0xFF,0x00,0},
                 BUTTON_COLOUR = {0xBC,0xB0,0x88,0},
                 STONED_COLOUR = {0xA0,0xA0,0xA0,0},
-                TITLE_COLOUR  = {0xBC,0xB0,0x88,0};
+                TITLE_COLOUR  = {0xBC,0xB0,0x88,0},
+				LABEL_COLOUR  = {0x6B,0x8C,0xFF,0},
+				BIGMAP_COLOUR = {0x00,0xAA,0x00,0};
 const SDL_Color DISABLED_COLOUR = inverse(STONED_COLOUR);
 
 namespace {
@@ -451,7 +467,8 @@ void text_surface::measure() const
 			itor->ucs2_text.push_back(0);
 		}
 
-		TTF_SizeUNICODE(ttfont, (Uint16 const *)&(itor->ucs2_text.front()), &w, &h);
+		TTF_SizeUNICODE(ttfont, 
+			static_cast<Uint16 const *>(&(itor->ucs2_text.front())), &w, &h);
 		w_ += w;
 		h_ = maximum<int>(h_, h);
 	}
@@ -496,7 +513,8 @@ std::vector<surface> const &text_surface::get_surfaces() const
 			continue;
 		font_style_setter const style_setter(ttfont, style_);
 
-		surface s = surface(TTF_RenderUNICODE_Blended(ttfont, (Uint16 const *)&(itor->ucs2_text.front()), color_));
+		surface s = surface(TTF_RenderUNICODE_Blended(ttfont, 
+			static_cast<Uint16 const *>(&(itor->ucs2_text.front())), color_));
 		if(!s.null())
 			surfs_.push_back(s);
 	}
@@ -520,12 +538,16 @@ unsigned int text_cache::max_size_ = 50;
 
 void text_cache::resize(unsigned int size)
 {
+	DBG_FT << "Text cache: resize from: " << max_size_ << " to: "
+		<< size << " items in cache: " << cache_.size() << '\n';
+
 	while(size < cache_.size()) {
 		cache_.pop_back();
 	}
 	max_size_ = size;
 }
 
+
 text_surface &text_cache::find(text_surface const &t)
 {
 	static size_t lookup_ = 0, hit_ = 0;
@@ -540,7 +562,7 @@ text_surface &text_cache::find(text_surface const &t)
 		cache_.push_front(t);
 	}
 	if (++lookup_ % 1000 == 0) {
-		LOG_FT << "Text cache: " << lookup_ << " lookups, " << (hit_ / 10) << "% hits\n";
+		DBG_FT << "Text cache: " << lookup_ << " lookups, " << (hit_ / 10) << "% hits\n";
 		hit_ = 0;
 	}
 	return cache_.front();
@@ -548,7 +570,7 @@ text_surface &text_cache::find(text_surface const &t)
 
 }
 
-surface render_text(const std::string& text, int fontsize, const SDL_Color& colour, int style)
+static surface render_text(const std::string& text, int fontsize, const SDL_Color& colour, int style)
 {
 	const std::vector<std::string> lines = utils::split(text, '\n', utils::REMOVE_EMPTY);
 	std::vector<std::vector<surface> > surfaces;
@@ -609,85 +631,6 @@ surface get_rendered_text(const std::string& str, int size, const SDL_Color& col
 	return render_text(str, size, colour, style);
 }
 
-//Measure a single line of ucs2 text with a specific font_size and
-//style without newline chars.
-//
-//This is INSECURE with lines which are WIDER than UINT16_MAX pixels.
-//
-//It is intended for the command_line widget only which has a line
-//length restriction and small font size.
-SDL_Rect measure_ucs2_text_line(ucs2_string::const_iterator first, ucs2_string::const_iterator last, int font_size, int style) {
-	wassert(last - first >= 0);
-
-	SDL_Rect rect;
-	rect.w = 0;
-	rect.h = 0;
-
-	ucs2_string chunk((last - first ) + 2);
-	ucs2_string::iterator chunk_itor = chunk.begin();
-	*chunk_itor = 0;
-
-	int current_font = 0;
-
-	//set the font for the first char
-	if(*first < font_map.size() && font_map[*first] >= 0) {
-		current_font = font_map[*first];
-	}
-
-	for(;first != last; ++first) {
-		if(*first < font_map.size() && font_map[*first] >= 0 && font_map[*first] != current_font) {
-			TTF_Font* ttfont = get_font(font_id(current_font, font_size));
-			if(ttfont == NULL) {
-				chunk_itor = chunk.begin();
-				*(chunk_itor++) = *first;
-				current_font = font_map[*first];
-				continue;
-			}
-			*(chunk_itor++) = 0;
-
-			font_style_setter const style_setter(ttfont, style);
-
-			int x, y;
-			TTF_SizeUNICODE(ttfont, (Uint16 const *)&chunk.front(), &x, &y);
-			rect.x = x;
-			rect.y = y;
-
-			rect.w += rect.x;
-			rect.h = maximum<Sint16>(rect.h, rect.y);
-
-			chunk_itor = chunk.begin();
-
-			current_font = font_map[*first];
-		}
-		*(chunk_itor++) = *first;
-	}
-	if (chunk_itor != chunk.begin()) {
-		TTF_Font* ttfont = get_font(font_id(current_font, font_size));
-		if(ttfont == NULL) {
-			rect.x = 0;
-			rect.y = 0;
-			return rect;
-		}
-		*(chunk_itor++) = 0;
-
-		font_style_setter const style_setter(ttfont, style);
-
-		int x, y;
-		TTF_SizeUNICODE(ttfont, (Uint16 const *)&chunk.front(), &x, &y);
-		rect.x = x;
-		rect.y = y;
-
-		rect.w += rect.x;
-		rect.h = maximum<Sint16>(rect.h, rect.y);
-	}
-	//reset rect.x and rec.y because we abused it to store the area
-	//of the last chunk
-	rect.x = 0;
-	rect.y = 0;
-	return rect;
-}
-
-
 SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 		   const SDL_Color& colour, const std::string& text,
 		   int x, int y, bool use_tooltips, int style)
@@ -776,15 +719,6 @@ int get_max_height(int size)
 	return TTF_FontHeight(font);
 }
 
-std::string remove_first_space(const std::string& text)
-{
-  if (text.length() > 0 && text[0] == ' ') {
-    return text.substr(1);
-  }
-
-  return text;
-}
-
 int line_width(const std::string& line, int font_size, int style)
 {
 	return line_size(line,font_size,style).w;
@@ -809,15 +743,14 @@ SDL_Rect line_size(const std::string& line, int font_size, int style)
 	res.x = res.y = 0;
 
 	cache.insert(std::pair<std::string,SDL_Rect>(line,res));
-
 	return res;
 }
 
-std::string make_text_ellipsis(const std::string &text, int font_size, int max_width)
+std::string make_text_ellipsis(const std::string &text, int font_size, int max_width, bool with_tags)
 {
 	static const std::string ellipsis = "...";
 
-	if(line_width(text, font_size) <= max_width)
+	if(line_width(with_tags ? text : del_tags(text), font_size) <= max_width)
 		return text;
 	if(line_width(ellipsis, font_size) > max_width)
 		return "";
@@ -831,7 +764,7 @@ std::string make_text_ellipsis(const std::string &text, int font_size, int max_w
 		tmp.append(itor.substr().first, itor.substr().second);
 		tmp += ellipsis;
 
-		if (line_width(tmp, font_size) > max_width) {
+		if (line_width(with_tags ? tmp : del_tags(tmp), font_size) > max_width) {
 			return current_substring + ellipsis;
 		}
 
@@ -865,11 +798,10 @@ public:
 
 	surface create_surface();
 
-	bool expired() const;
+	bool expired() const { return lifetime_ == 0; }
 
-	const std::string& text() const;
 
-	void show(bool value);
+	void show(const bool value) { visible_ = value; }
 
 	bool scroll() const { return scroll_; }
 
@@ -932,18 +864,12 @@ surface floating_label::create_surface()
 				return NULL;
 			}
 
-			SDL_FillRect(tmp,NULL,SDL_MapRGB(tmp->format,bgcolour_.r,bgcolour_.g,bgcolour_.b));
-			if(bgalpha_ != 255) {
-				tmp.assign(adjust_surface_alpha_add(tmp,bgalpha_ - 255));
-				if(tmp == NULL) {
-					return NULL;
-				}
-			}
+			SDL_FillRect(tmp,NULL,SDL_MapRGBA(tmp->format,bgcolour_.r,bgcolour_.g,bgcolour_.b, bgalpha_));
 
 			surf_.assign(tmp);
 		} else {
 			surface background = font::render_text(text_, font_size_, font::BLACK_COLOUR, 0);
-			background = blur_surface(background,4);
+			background = blur_alpha_surface(background,2);
 			background = adjust_surface_alpha(background, ftofxp(4.0));
 
 			surf_.assign(background);
@@ -1006,22 +932,17 @@ void floating_label::undraw(surface screen)
 	if(lifetime_ > 0) {
 		--lifetime_;
 		if(alpha_change_ != 0 && (xmove_ != 0.0 || ymove_ != 0.0)) {
+			// we don't compress these surfaces since they will always change
 			if (!surf_.null()) {
-				surf_.assign(adjust_surface_alpha_add(surf_,alpha_change_));
+				surf_.assign(adjust_surface_alpha_add(surf_,alpha_change_,false));
 			}
 			if (!foreground_.null()) {
-				foreground_.assign(adjust_surface_alpha_add(foreground_,alpha_change_));
+				foreground_.assign(adjust_surface_alpha_add(foreground_,alpha_change_,false));
 			}
 		}
 	}
 }
 
-bool floating_label::expired() const { return lifetime_ == 0; }
-
-const std::string& floating_label::text() const { return text_; }
-
-void floating_label::show(bool value) { visible_ = value; }
-
 }
 
 namespace font {
@@ -1085,20 +1006,8 @@ void show_floating_label(int handle, bool value)
 	}
 }
 
-const std::string& get_floating_label_text(int handle)
-{
-	const label_map::iterator i = labels.find(handle);
-	if(i != labels.end()) {
-		return i->second.text();
-	} else {
-		static const std::string empty_str;
-		return empty_str;
-	}
-}
-
 SDL_Rect get_floating_label_rect(int handle)
 {
-	static const SDL_Rect empty_rect = {0,0,0,0};
 	const label_map::iterator i = labels.find(handle);
 	if(i != labels.end()) {
 		const surface surf = i->second.create_surface();
@@ -1182,8 +1091,7 @@ void undraw_floating_labels(surface screen)
 
 }
 
-namespace {
-	bool add_font_to_fontlist(config* fonts_config, std::vector<font::subset_descriptor>& fontlist, const std::string& name)
+static bool add_font_to_fontlist(config* fonts_config, std::vector<font::subset_descriptor>& fontlist, const std::string& name)
 	{
 		config* font = fonts_config->find_child("font", "name", name);
 		if(font == NULL)
@@ -1210,7 +1118,6 @@ namespace {
 
 		return true;
 	}
-}
 
 namespace font {
 
@@ -1220,7 +1127,7 @@ bool load_font_config()
 	//config when changing languages
 	config cfg;
 	try {
-		scoped_istream stream = preprocess_file("data/fonts.cfg");
+		scoped_istream stream = preprocess_file("data/hardwired/fonts.cfg");
 		read(cfg, *stream);
 	} catch(config::error &e) {
 		ERR_FT << "could not read fonts.cfg:\n"
@@ -1266,4 +1173,5 @@ void cache_mode(CACHE mode)
 	}
 }
 
+
 }
