@@ -1,11 +1,12 @@
-/* $Id: game_errors.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: game_errors.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
-   Copyright (C) 2005 by Yann Dirson <ydirson@altern.org>
+   Copyright (C) 2003 by David White <dave@whitevine.net>
+   Copyright (C) 2005 - 2008 by Yann Dirson <ydirson@altern.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -18,7 +19,9 @@
 
 namespace game {
 struct error {
-	error() {}
+	error() : 
+		message() 
+		{}
 	error(const std::string& msg) : message(msg)
 	{}
 
