@@ -1,31 +1,36 @@
-/* $Id: image.cpp 16259 2007-03-16 18:42:16Z mordante $ */
+/* $Id: image.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file image.cpp
+//! Routines for images: load, scale, re-color, etc.
+
 #include "global.hpp"
 
+#include "color_range.hpp"
 #include "config.hpp"
 #include "filesystem.hpp"
 #include "game_config.hpp"
 #include "image.hpp"
 #include "log.hpp"
 #include "sdl_utils.hpp"
-#include "team.hpp"
 #include "util.hpp"
-#include "wassert.hpp"
 #include "wesconfig.h"
+#include "serialization/string_utils.hpp"
 
 #include "SDL_image.h"
 
+#include <cassert>
 #include <iostream>
 #include <map>
 #include <string>
@@ -38,11 +43,9 @@ typedef std::map<image::locator::value, int> locator_finder_t;
 typedef std::pair<image::locator::value, int> locator_finder_pair;
 locator_finder_t locator_finder;
 
-// Definition of all image maps
-image::image_cache images_,scaled_images_,unmasked_images_,greyed_images_;
-image::image_cache brightened_images_,semi_brightened_images_,darkened_images_;
-
-image::locator_cache alternative_images_;
+//! Definition of all image maps
+image::image_cache images_,hexed_images_,scaled_to_hex_images_,scaled_to_zoom_,unmasked_images_;
+image::image_cache brightened_images_,semi_brightened_images_;
 
 // const int cache_version_ = 0;
 
@@ -52,15 +55,19 @@ std::map<surface, surface> reversed_images_;
 
 int red_adjust = 0, green_adjust = 0, blue_adjust = 0;
 
+//! List of colors used by the TC image modification
+std::vector<std::string> team_colors;
+
 std::string image_mask;
 
-const int tile_size = 72;
-int zoom = tile_size;
+int zoom = image::tile_size;
+int cached_zoom = 0;
 
-//The "pointer to surfaces" vector is not cleared anymore (the surface are
-//still freed, of course.) I do not think it is a problem, as the number of
-//different surfaces the program may lookup has an upper limit, so its
-//memory usage won't grow indefinitely over time
+// The "pointer to surfaces" vector is not cleared anymore
+// (the surface are still freed, of course).
+// I do not think it is a problem, as the number of different surfaces
+// the program may lookup has an upper limit, so its memory usage
+// won't grow indefinitely over time.
 template<typename T>
 void reset_cache(std::vector<image::cache_item<T> >& cache)
 {
@@ -72,24 +79,26 @@ void reset_cache(std::vector<image::cache_item<T> >& cache)
 		beg->item = T();
 	}
 }
-}
+} // end anon namespace
 
 namespace image {
 
 mini_terrain_cache_map mini_terrain_cache;
+mini_terrain_cache_map mini_fogged_terrain_cache;
 
 void flush_cache()
 {
 	reset_cache(images_);
-	reset_cache(scaled_images_);
+	reset_cache(hexed_images_);
+	reset_cache(scaled_to_hex_images_);
+	reset_cache(scaled_to_zoom_);
 	reset_cache(unmasked_images_);
-	reset_cache(greyed_images_);
-	reset_cache(darkened_images_);
 	reset_cache(brightened_images_);
 	reset_cache(semi_brightened_images_);
-	reset_cache(alternative_images_);
 	mini_terrain_cache.clear();
+	mini_fogged_terrain_cache.clear();
 	reversed_images_.clear();
+	image_existance_map.clear();
 }
 
 int locator::last_index_ = 0;
@@ -103,31 +112,18 @@ void locator::init_index()
 		locator_finder.insert(locator_finder_pair(val_, index_));
 
 		images_.push_back(cache_item<surface>());
-		scaled_images_.push_back(cache_item<surface>());
+		hexed_images_.push_back(cache_item<surface>());
+		scaled_to_hex_images_.push_back(cache_item<surface>());
+		scaled_to_zoom_.push_back(cache_item<surface>());
 		unmasked_images_.push_back(cache_item<surface>());
-		greyed_images_.push_back(cache_item<surface>());
-		darkened_images_.push_back(cache_item<surface>());
 		brightened_images_.push_back(cache_item<surface>());
 		semi_brightened_images_.push_back(cache_item<surface>());
 
-		alternative_images_.push_back(cache_item<locator>());
 	} else {
 		index_ = i->second;
 	}
 }
-void locator::get_tc_info(const std::string& field)
-{
-	// field should be of the format "team,team_color_id"
-	size_t comma = field.find(',');
-	if(comma == std::string::npos) {
-		return;
-	}
-	int team_n = lexical_cast_default<int>(field.substr(0,comma));
-	std::string color_id = field.substr(comma+1);
-	val_.new_color = team::get_side_color_range(team_n);
-	val_.swap_colors = game_config::tc_info(color_id);
-	val_.type_ = SUB_FILE;
-}
+
 void locator::parse_arguments()
 {
 	std::string& fn = val_.filename_;
@@ -135,19 +131,11 @@ void locator::parse_arguments()
 		return;
 	}
 	size_t markup_field = fn.find('~');
-	while(markup_field != std::string::npos) {
-		size_t open_field = fn.find('(',markup_field);
-		size_t close_field = fn.find(')',markup_field);
-		if(open_field == std::string::npos || close_field == std::string::npos) {
-			return;
-		}
-		std::string field = fn.substr(open_field+1,close_field-open_field-1);
-		std::string function = fn.substr(markup_field+1,open_field-markup_field-1);
-		fn = fn.substr(0,markup_field) + fn.substr(close_field+1);
-		if(function == "TC") {
-			get_tc_info(field);
-		}
-		markup_field = fn.find('~');
+
+	if(markup_field != std::string::npos) {
+		val_.type_ = SUB_FILE;
+		val_.modifications_ = fn.substr(markup_field, fn.size() - markup_field);
+		fn = fn.substr(0,markup_field);
 	}
 }
 
@@ -156,9 +144,15 @@ locator::locator() :
 {
 }
 
-locator::locator(const locator &a):
-	index_(a.index_), val_(a.val_)
+locator::locator(const locator &a, const std::string& mods):
+	 val_(a.val_)
 {
+	if(mods.size()){
+			val_.modifications_ += mods;
+			val_.type_=SUB_FILE;
+			init_index();
+	}
+	else index_=a.index_;
 }
 
 locator::locator(const char *filename) :
@@ -175,20 +169,20 @@ locator::locator(const std::string &filename) :
 	init_index();
 }
 
-locator::locator(const char *filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
-	val_(filename, new_rgb, swap_rgb)
+locator::locator(const char *filename, const std::string& modifications) :
+	val_(filename, modifications)
 {
 	init_index();
 }
 
-locator::locator(const std::string &filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
-	val_(filename, new_rgb, swap_rgb)
+locator::locator(const std::string &filename, const std::string& modifications) :
+	val_(filename, modifications)
 {
 	init_index();
 }
 
-locator::locator(const std::string &filename, const gamemap::location &loc, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
-	val_(filename, loc, new_rgb, swap_rgb)
+locator::locator(const std::string &filename, const gamemap::location &loc, const std::string& modifications) :
+	val_(filename, loc, modifications)
 {
 	init_index();
 }
@@ -203,7 +197,7 @@ locator& locator::operator=(const locator &a)
 
 locator::value::value(const locator::value& a) :
   type_(a.type_), filename_(a.filename_), loc_(a.loc_),
-  new_color(a.new_color), swap_colors(a.swap_colors)
+  modifications_(a.modifications_)
 {
 }
 
@@ -217,8 +211,8 @@ locator::value::value(const char *filename) :
 }
 
 
-locator::value::value(const char *filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
-  type_(SUB_FILE), filename_(filename), new_color(new_rgb), swap_colors(swap_rgb)
+locator::value::value(const char *filename, const std::string& modifications) :
+  type_(SUB_FILE), filename_(filename), modifications_(modifications)
 {
 }
 
@@ -227,13 +221,13 @@ locator::value::value(const std::string& filename) :
 {
 }
 
-locator::value::value(const std::string& filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
-  type_(SUB_FILE), filename_(filename), new_color(new_rgb), swap_colors(swap_rgb)
+locator::value::value(const std::string& filename, const std::string& modifications) :
+  type_(SUB_FILE), filename_(filename), modifications_(modifications)
 {
 }
 
-locator::value::value(const std::string& filename, const gamemap::location& loc, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb) :
-  type_(SUB_FILE), filename_(filename), loc_(loc), new_color(new_rgb), swap_colors(swap_rgb)
+locator::value::value(const std::string& filename, const gamemap::location& loc, const std::string& modifications) :
+  type_(SUB_FILE), filename_(filename), loc_(loc), modifications_(modifications)
 {
 }
 
@@ -244,7 +238,7 @@ bool locator::value::operator==(const value& a) const
 	} else if(type_ == FILE) {
 		return filename_ == a.filename_;
 	} else if(type_ == SUB_FILE) {
-	  return filename_ == a.filename_ && loc_ == a.loc_ && new_color == a.new_color; //note not checking swap_colors purposely
+	  return filename_ == a.filename_ && loc_ == a.loc_ && modifications_ == a.modifications_;
 	} else {
 		return false;
 	}
@@ -261,7 +255,7 @@ bool locator::value::operator<(const value& a) const
 			return filename_ < a.filename_;
 		if(loc_ != a.loc_)
 		        return loc_ < a.loc_;
-		return(new_color < a.new_color);
+		return(modifications_ < a.modifications_);
 	} else {
 		return false;
 	}
@@ -277,16 +271,7 @@ surface locator::load_image_file() const
 
 	do {
 		if (!location.empty()) {
-#ifdef USE_ZIPIOS
-			std::string const &s = read_file(location);
-			if (!s.empty()) {
-				SDL_RWops* ops = SDL_RWFromMem((void*)s.c_str(), s.size());
-				res = IMG_Load_RW(ops, 0);
-				SDL_FreeRW(ops);
-			}
-#else
 			res = IMG_Load(location.c_str());
-#endif
 		}
 		if (res.null() && (!try_units)) {
 			try_units = true;
@@ -305,8 +290,8 @@ surface locator::load_image_file() const
 
 surface locator::load_image_sub_file() const
 {
-	const surface mother_surface(get_image(val_.filename_, UNSCALED, NO_ADJUST_COLOUR));
-	const surface mask(get_image(game_config::terrain_mask_image, UNSCALED, NO_ADJUST_COLOUR));
+	const surface mother_surface(get_image(val_.filename_, UNSCALED));
+	const surface mask(get_image(game_config::terrain_mask_image, UNSCALED));
 
 	if(mother_surface == NULL)
 		return surface(NULL);
@@ -324,8 +309,82 @@ surface locator::load_image_sub_file() const
 	  surface tmp(cut_surface(mother_surface, srcrect));
 	  surf=mask_surface(tmp, mask);
 	}
-	if(val_.swap_colors.size()){
-	  surf=recolor_image(surf,get_new_color(),get_swap_colors());
+
+	if(val_.modifications_.size()){
+		bool xflip = false;
+		bool yflip = false;
+		std::map<Uint32, Uint32> recolor_map;
+		std::vector<std::string> modlist = utils::paranthetical_split(val_.modifications_,'~');
+		for(std::vector<std::string>::const_iterator i=modlist.begin();
+			i!= modlist.end();i++){
+			std::vector<std::string> tmpmod = utils::paranthetical_split(*i);
+			std::vector<std::string>::const_iterator j=tmpmod.begin();
+			while(j!= tmpmod.end()){
+				std::string function=*j++;
+				if(j==tmpmod.end()){
+					if(function.size()){
+						ERR_DP << "error parsing image modifications: "
+							<< val_.modifications_<< "\n";
+					}
+					break;
+				}
+				std::string field = *j++;
+
+				if("TC" == function){
+					std::vector<std::string> param = utils::split(field,',');
+					if(param.size() < 2)
+						break;
+
+					int side_n = lexical_cast_default<int>(param[0], -1);
+					std::string team_color; 
+					if (side_n < 1) {
+						break;
+					} else if (side_n < static_cast<int>(team_colors.size())) {
+						team_color = team_colors[side_n - 1];
+					} else {
+					// this side is not inialized use default "n"
+						team_color = lexical_cast<std::string>(side_n);
+					}	
+
+					if(game_config::tc_info(param[1]).size()){
+						function="RC";
+						field = param[1] + ">" + team_color;
+					}
+				}
+
+				if("RC" == function){	// Re-color function
+					std::vector<std::string> recolor=utils::split(field,'>');
+					if(recolor.size()>1){
+						std::map<Uint32, Uint32> tmp_map;
+						try {
+							color_range const& new_color = game_config::color_info(recolor[1]);
+							std::vector<Uint32> const& old_color = game_config::tc_info(recolor[0]);
+							tmp_map = recolor_range(new_color,old_color);
+						} catch (config::error& e) {
+							ERR_DP << "caught config::error... " << e.message << std::endl;
+						}
+						for(std::map<Uint32, Uint32>::const_iterator tmp = tmp_map.begin(); tmp!= tmp_map.end(); tmp++){
+							recolor_map[tmp->first] = tmp->second;
+						}
+					}
+				}
+				if("FL" == function){	// Flip layer
+					if(field.empty() || field.find("horiz") != std::string::npos) {
+						xflip = !xflip;
+					}
+					if(field.find("vert") != std::string::npos) {
+						yflip = !yflip;
+					}
+				}
+			}
+		}
+		surf = recolor_image(surf,recolor_map);
+		if(xflip) {
+			surf = flip_surface(surf);
+		}
+		if(yflip) {
+			surf = flop_surface(surf);
+		}
 	}
 	return surf;
 }
@@ -340,7 +399,6 @@ surface locator::load_from_disk() const
 		default:
 			return surface(NULL);
 	}
-	wassert(false);
 }
 
 #if 0
@@ -372,49 +430,6 @@ void locator::add_to_cache(std::vector<cache_item<T> >& cache, const T& item) co
 }
 #endif
 
-bool locator::in_cache(const std::vector<cache_item<surface> >& cache) const
-{
-	if(index_ == -1)
-		return false;
-
-	return cache[index_].loaded;
-}
-surface locator::locate_in_cache(const std::vector<cache_item<surface> >& cache) const
-{
-	if(index_ == -1)
-		return surface();
-
-	return cache[index_].item;
-}
-void locator::add_to_cache(std::vector<cache_item<surface> >& cache, const surface& item) const
-{
-	if(index_ == -1)
-		return;
-
-	cache[index_] = cache_item<surface>(item);
-}
-bool locator::in_cache(const std::vector<cache_item<locator> >& cache) const
-{
-	if(index_ == -1)
-		return false;
-
-	return cache[index_].loaded;
-}
-locator locator::locate_in_cache(const std::vector<cache_item<locator> >& cache) const
-{
-	if(index_ == -1)
-		return locator();
-
-	return cache[index_].item;
-}
-void locator::add_to_cache(std::vector<cache_item<locator> >& cache, const locator& item) const
-{
-	if(index_ == -1)
-		return;
-
-	cache[index_] = cache_item<locator>(item);
-}
-
 manager::manager() {}
 
 manager::~manager()
@@ -446,130 +461,130 @@ void set_colour_adjustment(int r, int g, int b)
 		red_adjust = r;
 		green_adjust = g;
 		blue_adjust = b;
-		reset_cache(scaled_images_);
-		reset_cache(greyed_images_);
-		reset_cache(darkened_images_);
+		reset_cache(scaled_to_hex_images_);
 		reset_cache(brightened_images_);
 		reset_cache(semi_brightened_images_);
-		reset_cache(alternative_images_);
 		reversed_images_.clear();
 	}
 }
 
-
-void get_colour_adjustment(int *r, int *g, int *b)
+void set_team_colors(const std::vector<std::string>* colors)
 {
-	if(r != NULL) {
-		*r = red_adjust;
-	}
-
-	if(g != NULL) {
-		*g = green_adjust;
-	}
-
-	if(b != NULL) {
-		*b = blue_adjust;
+	if (colors == NULL)
+		team_colors.clear();
+	else {
+		team_colors = *colors;
 	}
 }
 
-
-void set_image_mask(const std::string& image)
+void set_image_mask(const std::string& /*image*/)
 {
+
+	// image_mask are blitted in display.cpp
+	// so no need to flush the cache here
+/*
 	if(image_mask != image) {
 		image_mask = image;
-		reset_cache(scaled_images_);
-		reset_cache(greyed_images_);
-		reset_cache(darkened_images_);
+		reset_cache(scaled_to_hex_images_);
+		reset_cache(scaled_to_zoom_);
 		reset_cache(brightened_images_);
 		reset_cache(semi_brightened_images_);
-		reset_cache(alternative_images_);
 		reversed_images_.clear();
 	}
-
+*/
 }
 
 void set_zoom(int amount)
 {
 	if(amount != zoom) {
 		zoom = amount;
-		reset_cache(scaled_images_);
-		reset_cache(greyed_images_);
-		reset_cache(darkened_images_);
+		reset_cache(scaled_to_hex_images_);
 		reset_cache(brightened_images_);
 		reset_cache(semi_brightened_images_);
-		reset_cache(unmasked_images_);
-		reset_cache(alternative_images_);
 		reversed_images_.clear();
+
+		// We keep these caches if:
+		// we use default zoom (it doesn't need those)
+		// or if they are already at the wanted zoom.
+		if (zoom != tile_size && zoom != cached_zoom) {
+			reset_cache(scaled_to_zoom_);
+			reset_cache(unmasked_images_);
+			cached_zoom = zoom;
+		}
 	}
 }
 
-surface get_unmasked(const locator i_locator)
+static surface get_hexed(const locator i_locator)
 {
-	surface image(get_image(i_locator, UNSCALED));
-
-	// Re-cut scaled tiles according to a mask. Check if the surface we try
-	// to get is not the mask itself, to avoid an infinite loop.
-	surface res;
-	if(i_locator != locator(game_config::terrain_mask_image)) {
-		const surface hex(get_image(game_config::terrain_mask_image,
-					UNMASKED, NO_ADJUST_COLOUR));
-		res = mask_surface(scale_surface(image, zoom, zoom), hex);
-	} else {
-		res = scale_surface(image, zoom, zoom);
-	}
-
-	return res;
+	// w.e don't want to add it to the unscaled cache,
+	// since we will normaly never need the non-hexed one.
+	surface image(get_image(i_locator, UNSCALED, false));
+	// Re-cut scaled tiles according to a mask.
+	const surface hex(get_image(game_config::terrain_mask_image,
+					UNSCALED));
+	return mask_surface(image, hex);
 }
 
-surface get_scaled(const locator i_locator, COLOUR_ADJUSTMENT adj)
+static surface get_unmasked(const locator i_locator)
 {
-	surface res(get_image(i_locator, UNMASKED, adj));
+	// If no scaling needed at this zoom level,
+	// we just use the hexed image.
+	surface image(get_image(i_locator, HEXED));
+	if (zoom != tile_size)
+		return scale_surface(image, zoom, zoom);
+	else
+		return image;
+}
+
+static surface get_scaled_to_hex(const locator i_locator)
+{
+	surface res(get_image(i_locator, UNMASKED));
 
 	// Adjusts colour if necessary.
-	if(adj == ADJUST_COLOUR && (red_adjust != 0 ||
-				green_adjust != 0 || blue_adjust != 0)) {
+	if (red_adjust != 0 ||
+				green_adjust != 0 || blue_adjust != 0) {
 		res = surface(adjust_surface_colour(res,
 					red_adjust, green_adjust, blue_adjust));
 	}
-
-	const surface mask(get_image(image_mask,UNMASKED,NO_ADJUST_COLOUR));
+	/*
+	const surface mask(get_image(image_mask,UNMASKED));
 	if(mask != NULL) {
 		SDL_SetAlpha(mask,SDL_SRCALPHA|SDL_RLEACCEL,SDL_ALPHA_OPAQUE);
 		SDL_SetAlpha(res,SDL_SRCALPHA|SDL_RLEACCEL,SDL_ALPHA_OPAQUE);
 
 		//commented out pending reply from SDL team about bug report
 		//SDL_BlitSurface(mask,NULL,result,NULL);
-	}
+	}*/
 	return res;
 }
 
-surface get_greyed(const locator i_locator, COLOUR_ADJUSTMENT adj)
+static surface get_scaled_to_zoom(const locator i_locator)
 {
-	surface image(get_image(i_locator, SCALED, adj));
+	assert(zoom != tile_size);
+	assert(tile_size != 0);
 
-	return surface(greyscale_image(image));
+	surface res(get_image(i_locator, UNSCALED));
+	// For some reason haloes seems to have invalid images, protect against crashing
+	if(!res.null()) {
+		return scale_surface(res, ((res.get()->w * zoom) / tile_size), ((res.get()->h * zoom) / tile_size));
+	} else {
+		return surface(NULL);
+	}
 }
 
-surface get_darkened(const locator i_locator, COLOUR_ADJUSTMENT adj)
+static surface get_brightened(const locator i_locator)
 {
-	surface image(get_image(i_locator, SCALED, adj));
-
-	return surface(darken_image(image));
-}
-
-surface get_brightened(const locator i_locator, COLOUR_ADJUSTMENT adj)
-{
-	surface image(get_image(i_locator, SCALED, adj));
+	surface image(get_image(i_locator, SCALED_TO_HEX));
 	return surface(brighten_image(image, ftofxp(1.5)));
 }
 
-surface get_semi_brightened(const locator i_locator, COLOUR_ADJUSTMENT adj)
+static surface get_semi_brightened(const locator i_locator)
 {
-	surface image(get_image(i_locator, SCALED, adj));
+	surface image(get_image(i_locator, SCALED_TO_HEX));
 	return surface(brighten_image(image, ftofxp(1.25)));
 }
 
-surface get_image(const image::locator& i_locator, TYPE type, COLOUR_ADJUSTMENT adj)
+surface get_image(const image::locator& i_locator, TYPE type, bool add_to_cache )
 {
 	surface res(NULL);
 	image_cache *imap;
@@ -577,21 +592,35 @@ surface get_image(const image::locator& i_locator, TYPE type, COLOUR_ADJUSTMENT
 	if(i_locator.is_void())
 		return surface(NULL);
 
+	bool is_unscaled = false;
+
 	switch(type) {
 	case UNSCALED:
+		is_unscaled = true;
 		imap = &images_;
 		break;
-	case SCALED:
-		imap = &scaled_images_;
+	case SCALED_TO_HEX:
+		imap = &scaled_to_hex_images_;
+		break;
+	case SCALED_TO_ZOOM:
+		// Only use separate cache if scaled
+		if(zoom != tile_size) {
+			imap = &scaled_to_zoom_;
+		} else {
+			is_unscaled = true;
+			imap = &images_;
+		}
+		break;
+	case HEXED:
+		imap = &hexed_images_;
 		break;
 	case UNMASKED:
-		imap = &unmasked_images_;
-		break;
-	case GREYED:
-		imap = &greyed_images_;
-		break;
-	case DARKENED:
-		imap = &darkened_images_;
+		// Only use separate cache if scaled
+		if(zoom != tile_size) {
+			imap = &unmasked_images_;
+		} else {
+			imap = &hexed_images_;
+		}
 		break;
 	case BRIGHTENED:
 		imap = &brightened_images_;
@@ -606,13 +635,13 @@ surface get_image(const image::locator& i_locator, TYPE type, COLOUR_ADJUSTMENT
 	if(i_locator.in_cache(*imap))
 		return i_locator.locate_in_cache(*imap);
 
-	// If type is unscaled, directly load the image from the disk. Else,
-	// create it from the unscaled image
-	if(type == UNSCALED) {
+	// If type is unscaled, directly load the image from the disk.
+	// Else, create it from the unscaled image.
+	if(is_unscaled) {
 		res = i_locator.load_from_disk();
 
 		if(res == NULL) {
-			i_locator.add_to_cache(*imap, surface(NULL));
+			if(add_to_cache) i_locator.add_to_cache(*imap, surface(NULL));
 			return surface(NULL);
 		}
 	} else {
@@ -620,49 +649,35 @@ surface get_image(const image::locator& i_locator, TYPE type, COLOUR_ADJUSTMENT
 		// surface base_image(get_image(i_locator, UNSCALED));
 
 		switch(type) {
-		case SCALED:
-			res = get_scaled(i_locator, adj);
+		case SCALED_TO_HEX:
+			res = get_scaled_to_hex(i_locator);
+			break;
+		case SCALED_TO_ZOOM:
+			res = get_scaled_to_zoom(i_locator);
+			break;
+		case HEXED:
+			res = get_hexed(i_locator);
 			break;
 		case UNMASKED:
 			res = get_unmasked(i_locator);
 			break;
-		case GREYED:
-			res = get_greyed(i_locator, adj);
-			break;
-		case DARKENED:
-			res = get_darkened(i_locator, adj);
-			break;
 		case BRIGHTENED:
-			res = get_brightened(i_locator, adj);
+			res = get_brightened(i_locator);
 			break;
 		case SEMI_BRIGHTENED:
-			res = get_semi_brightened(i_locator, adj);
+			res = get_semi_brightened(i_locator);
 			break;
 		default:
 			return surface(NULL);
 		}
 	}
 
-	// optimizes surface before storing it
+	// Optimizes surface before storing it
 	res = create_optimized_surface(res);
-	i_locator.add_to_cache(*imap, res);
+	if(add_to_cache) i_locator.add_to_cache(*imap, res);
 	return res;
 }
 
-surface get_image_dim(const image::locator& i_locator, size_t x, size_t y)
-{
-	const surface surf(get_image(i_locator,UNSCALED));
-
-	if(surf != NULL && (size_t(surf->w) != x || size_t(surf->h) != y)) {
-		const surface new_image(scale_surface(surf,x,y));
-
-		i_locator.add_to_cache(images_, new_image);
-		return new_image;
-	}
-
-	return surf;
-}
-
 surface reverse_image(const surface& surf)
 {
 	if(surf == NULL) {
@@ -685,43 +700,6 @@ surface reverse_image(const surface& surf)
 	return rev;
 }
 
-locator get_alternative(const image::locator &i_locator, const std::string &alt)
-{
-	if(i_locator.is_void())
-		return locator();
-
-	if(i_locator.in_cache(alternative_images_))
-		return i_locator.locate_in_cache(alternative_images_);
-
-	const std::string &name = i_locator.get_filename();
-	const std::string::size_type pos = name.rfind('.');
-	const std::string alternative = (pos != 0 ? name.substr(0, pos) : "") +
-		alt + (pos != std::string::npos ? name.substr(pos) : "");
-	locator res;
-
-	switch (i_locator.get_type()) {
-	case locator::FILE:
-		res = locator(alternative);
-		break;
-	case locator::SUB_FILE:
-		res = locator(alternative, i_locator.get_loc(), i_locator.get_new_color(), i_locator.get_swap_colors());
-		break;
-	default:
-		wassert(false);
-	}
-
-	i_locator.add_to_cache(alternative_images_, res);
-
-	return res;
-}
-
-
-
-void register_image(const image::locator& id, const surface& surf)
-{
-	id.add_to_cache(images_, surf);
-}
-
 bool exists(const image::locator& i_locator)
 {
 	typedef image::locator loc;
@@ -729,7 +707,7 @@ bool exists(const image::locator& i_locator)
 	if (type != loc::FILE && type != loc::SUB_FILE)
 		return false;
 
-	// the insertion will fail if there is already an element in the cache
+	// The insertion will fail if there is already an element in the cache
 	std::pair< std::map< image::locator, bool >::iterator, bool >
 		it = image_existance_map.insert(std::make_pair(i_locator, false));
 	bool &cache = it.first->second;
@@ -739,4 +717,5 @@ bool exists(const image::locator& i_locator)
 }
 
 
-}
+} // end namespace image
+
