@@ -1,16 +1,20 @@
-/* $Id: intro.cpp 11940 2006-06-08 10:32:49Z rusty $ */
+/* $Id: intro.cpp 23985 2008-02-21 08:15:14Z mythological $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file intro.cpp 
+//! Introduction sequence at start of a scenario, End-screen after end of campaign.
+
 #include "global.hpp"
 
 #include "display.hpp"
@@ -37,15 +41,16 @@
 
 #define LOG_NG LOG_STREAM(info, engine)
 
-bool show_intro_part(display &disp, const config& part,
+static bool show_intro_part(display &disp, const config& part,
 		const std::string& scenario);
 
+//! Show an introduction sequence at the start of a scenario.
 void show_intro(display &disp, const config& data, const config& level)
 {
 	LOG_NG << "showing intro sequence...\n";
 
-	//stop the screen being resized while we're in this function
-	const events::resize_lock stop_resizing;
+	// Stop the screen being resized while we're in this function
+	const resize_lock stop_resizing;
 	const events::event_context context;
 
 	bool showing = true;
@@ -74,6 +79,15 @@ void show_intro(display &disp, const config& data, const config& level)
 	LOG_NG << "intro sequence finished...\n";
 }
 
+//! show_intro_part() is split into two parts, the second part can cause
+//! an utils::invalid_utf8_exception exception and it's to much code
+//! to indent. The solution is not very clean but the entire routine could
+//! use a cleanup.
+static bool show_intro_part_helper(display &disp, const config& part,
+		int textx, int texty,
+		gui::button& next_button, gui::button& skip_button,
+		CKey& key);
+
 bool show_intro_part(display &disp, const config& part,
 		const std::string& scenario)
 {
@@ -82,18 +96,17 @@ bool show_intro_part(display &disp, const config& part,
 	CVideo &video = disp.video();
 	const std::string& music_file = part["music"];
 
-	//play music if available
+	// Play music if available
 	if(music_file != "") {
 		sound::play_music_repeatedly(music_file);
 	}
 
 	CKey key;
-	bool		lang_rtl = current_language_rtl();
 
 	gui::button next_button(video,_("Next") + std::string(">>>"));
 	gui::button skip_button(video,_("Skip"));
 
-	draw_solid_tinted_rectangle(0,0,video.getx()-1,video.gety()-1,
+	draw_solid_tinted_rectangle(0,0,video.getx(),video.gety(),
 			0,0,0,1.0,video.getSurface());
 
 
@@ -102,7 +115,7 @@ bool show_intro_part(display &disp, const config& part,
 
 	surface background(NULL);
 	if(background_name.empty() == false) {
-		background.assign(image::get_image(background_name,image::UNSCALED));
+		background.assign(image::get_image(background_name));
 	}
 
 	int textx = 200;
@@ -111,28 +124,17 @@ bool show_intro_part(display &disp, const config& part,
 	SDL_Rect dstrect;
 
 	if(background.null() || background->w*background->h == 0) {
-		background.assign(SDL_CreateRGBSurface(SDL_SWSURFACE,1,1,32,0xFF0000,0xFF00,0xFF,0xFF000000));
+		background.assign(SDL_CreateRGBSurface(SDL_SWSURFACE,video.getx(),video.gety(),32,0xFF0000,0xFF00,0xFF,0xFF000000));
 	}
 
-	const int source_aspect = (background->w*1000)/background->h;
-	const int dest_aspect = (video.getx()*1000)/video.gety();
+	double xscale = 1.0 * video.getx() / background->w;
+	double yscale = 1.0 * video.gety() / background->h;
+	double scale = minimum<double>(xscale,yscale);
 
-	int xscale = video.getx();
-	int yscale = video.gety();
+	background = scale_surface(background, static_cast<int>(background->w*scale), static_cast<int>(background->h*scale));
 
-	if(dest_aspect >= source_aspect) {
-		xscale = (background->w*yscale)/background->h;
-	} else {
-		yscale = (background->h*xscale)/background->w;
-	}
-
-	const int map_w = background->w;
-	const int map_h = background->h;
-
-	background = scale_surface(background,xscale,yscale);
-
-	dstrect.x = video.getx()/2 - background->w/2;
-	dstrect.y = video.gety()/2 - background->h/2;
+	dstrect.x = (video.getx() - background->w) / 2;
+	dstrect.y = (video.gety() - background->h) / 2;
 	dstrect.w = background->w;
 	dstrect.h = background->h;
 
@@ -141,6 +143,9 @@ bool show_intro_part(display &disp, const config& part,
 #ifdef USE_TINY_GUI
 	textx = 10;
 	int xbuttons = video.getx() - 50;
+
+	// Use the whole screen for text
+	texty = 0;
 #else
 	int xbuttons;
 
@@ -151,22 +156,28 @@ bool show_intro_part(display &disp, const config& part,
 		textx = 200;
 		xbuttons = video.getx() - 200 - 40;
 	}
-#endif
-	texty = dstrect.y + dstrect.h - 200;
 
-	//darken the area for the text and buttons to be drawn on
+	texty = dstrect.y + dstrect.h - 200;
+#endif
+
+	// Darken the area for the text and buttons to be drawn on
 	if(show_title == false) {
 		draw_solid_tinted_rectangle(0,texty,video.getx(),video.gety()-texty,0,0,0,0.5,video.getSurface());
 	}
 
+#ifdef USE_TINY_GUI
+	next_button.set_location(xbuttons,dstrect.y+dstrect.h-40);
+	skip_button.set_location(xbuttons,dstrect.y+dstrect.h-20);
+#else
 	next_button.set_location(xbuttons,dstrect.y+dstrect.h-70);
 	skip_button.set_location(xbuttons,dstrect.y+dstrect.h-40);
+#endif
 
-	//draw title if needed
+	// Draw title if needed
 	if(show_title) {
 		const SDL_Rect area = {0,0,video.getx(),video.gety()};
-		font::draw_text(NULL,area,font::SIZE_XLARGE,font::NORMAL_COLOUR,scenario,0,0);
-		update_rect(font::draw_text(&video,area,font::SIZE_XLARGE,font::NORMAL_COLOUR,scenario,
+		font::draw_text(NULL,area,font::SIZE_XLARGE,font::BIGMAP_COLOUR,scenario,0,0);
+		update_rect(font::draw_text(&video,area,font::SIZE_XLARGE,font::BIGMAP_COLOUR,scenario,
 					    dstrect.x + 20,dstrect.y + 20));
 	}
 
@@ -175,29 +186,27 @@ bool show_intro_part(display &disp, const config& part,
 	disp.flip();
 
 	if(!background.null()) {
-		//draw images
+		// Draw images
 		const config::child_list& images = part.get_children("image");
 
 		bool pass = false;
 
 		for(std::vector<config*>::const_iterator i = images.begin(); i != images.end(); ++i){
-			const std::string& xloc = (**i)["x"];
-			const std::string& yloc = (**i)["y"];
 			const std::string& image_name = (**i)["file"];
-			const std::string& delay_str = (**i)["delay"];
-			const int delay = (delay_str == "") ? 0: atoi(delay_str.c_str());
-			const int x = (atoi(xloc.c_str())*background->w)/map_w;
-			const int y = (atoi(yloc.c_str())*background->h)/map_h;
-
-			if(x < 0 || x >= background->w || y < 0 || y >= background->h)
-				continue;
-
 			if(image_name == "") continue;
-			surface img(image::get_image(image_name,image::UNSCALED));
+			surface img(image::get_image(image_name));
 			if(img.null()) continue;
 
-			if ((**i)["flip"] == "yes")
-				img.assign(image::reverse_image(img));
+			const std::string& xloc = (**i)["x"];
+			const std::string& yloc = (**i)["y"];
+			const std::string& delay_str = (**i)["delay"];
+			const int delay = (delay_str == "") ? 0: atoi(delay_str.c_str());
+			const int x = static_cast<int>(atoi(xloc.c_str())*scale);
+			const int y = static_cast<int>(atoi(yloc.c_str())*scale);
+
+			if ((**i)["scaled"] == "yes"){
+				img = scale_surface(img, static_cast<int>(img->w*scale), static_cast<int>(img->h*scale));
+			}
 
 			SDL_Rect image_rect;
 			image_rect.x = x + dstrect.x;
@@ -205,6 +214,11 @@ bool show_intro_part(display &disp, const config& part,
 			image_rect.w = img->w;
 			image_rect.h = img->h;
 
+			if ((**i)["centered"] == "yes"){
+				image_rect.x -= image_rect.w/2;
+				image_rect.y -= image_rect.h/2;
+			}
+
 			SDL_BlitSurface(img,NULL,video.getSurface(),&image_rect);
 
 			update_rect(image_rect);
@@ -223,7 +237,7 @@ bool show_intro_part(display &disp, const config& part,
 
 					int a, b;
 					const int mouse_flags = SDL_GetMouseState(&a,&b);
-					if(key[SDLK_RETURN] || key[SDLK_SPACE] || mouse_flags) {
+					if(key[SDLK_RETURN] || key[SDLK_KP_ENTER] || key[SDLK_SPACE] || mouse_flags) {
 						pass = true;
 						continue;
 					}
@@ -238,9 +252,30 @@ bool show_intro_part(display &disp, const config& part,
 			}
 		}
 	}
+	try {
+		return show_intro_part_helper(
+			disp, part, textx, texty, next_button, skip_button, key);
+
+	} catch (utils::invalid_utf8_exception&) {
+		LOG_STREAM(err, engine) << "Invalid utf-8 found, story message is ignored.\n";
+		// stop showing on an error, there might be more badly formed utf-8 messages
+		return false;
+	}
+}
+
+static bool show_intro_part_helper(display &disp, const config& part,
+		int textx, int texty,
+		gui::button& next_button, gui::button& skip_button,
+		CKey& key)
+{
+	bool lang_rtl = current_language_rtl();
+	CVideo &video = disp.video();
+
 
 	const int max_width = next_button.location().x - 10 - textx;
-	const std::string story = font::word_wrap_text(part["story"], font::SIZE_PLUS, max_width);
+	const std::string story = 
+		font::word_wrap_text(part["story"], font::SIZE_PLUS, max_width);
+
 	utils::utf8_iterator itor(story);
 
 	bool skip = false, last_key = true;
@@ -257,9 +292,14 @@ bool show_intro_part(display &disp, const config& part,
 
 	if(lang_rtl)
 		textx += max_width;
-	int xpos = textx, ypos = texty + 20;
 
-	//the maximum position that text can reach before wrapping
+#ifdef USE_TINY_GUI
+	int xpos = textx, ypos = texty + 10;
+#else
+	int xpos = textx, ypos = texty + 20;
+#endif
+
+	// The maximum position that text can reach before wrapping
 	size_t height = 0;
 
 	for(;;) {
@@ -270,8 +310,8 @@ bool show_intro_part(display &disp, const config& part,
 				++itor;
 			}
 
-			// output the character
-			// FIXME: this is broken: it does not take kerning into account.
+			// Output the character
+			//! @todo  FIXME: this is broken: it does not take kerning into account.
 			std::string tmp;
 			tmp.append(itor.substr().first, itor.substr().second);
 			if(lang_rtl)
@@ -293,7 +333,7 @@ bool show_intro_part(display &disp, const config& part,
 
 		}
 
-		const bool keydown = key[SDLK_SPACE] || key[SDLK_RETURN];
+		const bool keydown = key[SDLK_SPACE] || key[SDLK_RETURN] || key[SDLK_KP_ENTER];
 
 		if(keydown && !last_key || next_button.pressed()) {
 			if(skip == true || itor == utils::utf8_iterator::end(story)) {
@@ -317,12 +357,13 @@ bool show_intro_part(display &disp, const config& part,
 			disp.delay(20);
 	}
 
-	draw_solid_tinted_rectangle(0,0,video.getx()-1,video.gety()-1,0,0,0,1.0,
+	draw_solid_tinted_rectangle(0,0,video.getx(),video.gety(),0,0,0,1.0,
                                      video.getSurface());
 
 	return true;
 }
 
+//! Black screen with "The End", shown at the end of a campaign.
 void the_end(display &disp)
 {
 	SDL_Rect area = screen_area();
@@ -352,3 +393,4 @@ void the_end(display &disp)
 
 	disp.delay(4000);
 }
+
