@@ -1,17 +1,22 @@
-/* $Id: log.cpp 7645 2005-07-31 18:56:58Z j_daniel $ */
+/* $Id: log.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
-                 2004 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2003 by David White <dave@whitevine.net>
+                 2004 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file log.cpp
+//! Logger for debugging.
+//! See also wesnoth-commandline-params --logdomains and --log-<level>="domain".
+
 #include "global.hpp"
 
 #include "SDL.h"
@@ -38,20 +43,22 @@ public:
 	null_streambuf() {}
 };
 
-} // anonymous namespace
+} // end anonymous namespace
 
 static std::vector< logd > log_domains;
 static std::ostream null_ostream(new null_streambuf);
 static int indent = 0;
-static bool timestamp = false;
+static bool timestamp = true;
 
 namespace lg {
 
 void timestamps(bool t) { timestamp = t; }
 
-logger err("error", 0), warn("warning", 1), info("info", 2);
-log_domain general("general"), ai("ai"), config("config"), display("display"), engine("engine"),
-           network("network"), filesystem("filesystem"), audio("audio");
+logger err("error", 0), warn("warning", 1), info("info", 2), debug("debug", 3);
+log_domain general("general"), ai("ai"), config("config"), display("display"),
+	engine("engine"), network("network"), mp_server("server"),
+	filesystem("filesystem"), audio("audio"), notifs("notifs"),
+	replay("replay");
 
 log_domain::log_domain(char const *name) : domain_(log_domains.size())
 {
@@ -77,6 +84,20 @@ bool set_log_domain_severity(std::string const &name, int severity)
 	return true;
 }
 
+std::string list_logdomains()
+{
+	std::vector< logd >::iterator
+		it_begin = log_domains.begin(),
+		it_end = log_domains.end(),
+		it;
+	std::string domainlist = "";
+	for(it = it_begin; it != it_end; ++it) {
+		if (it != it_begin)
+			domainlist += ", ";
+		domainlist += it->name_;
+	}
+	return domainlist;
+}
 
 bool logger::dont_log(log_domain const &domain) const
 {
@@ -84,18 +105,20 @@ bool logger::dont_log(log_domain const &domain) const
 	return severity_ > d.severity_;
 }
 
+std::string get_timestamp(const time_t& t, const std::string& format) {
+	char buf[100];
+	strftime(buf, 100, format.c_str(), localtime(&t));
+	return buf;
+}
+
 std::ostream &logger::operator()(log_domain const &domain, bool show_names) const
 {
 	logd const &d = log_domains[domain.domain_];
 	if (severity_ > d.severity_)
 		return null_ostream;
 	else {
-		if (timestamp) {
-			time_t t = time(NULL);
-			char buf[100];
-			strftime(buf, 100, "%b %d %H:%M:%S ", localtime(&t));
-			std::cerr << buf;
-		}
+		if (timestamp)
+			std::cerr << get_timestamp(time(NULL));
 		if (show_names)
 			std::cerr << name_ << ' ' << d.name_ << ": ";
 		return std::cerr;
@@ -103,7 +126,7 @@ std::ostream &logger::operator()(log_domain const &domain, bool show_names) cons
 }
 
 scope_logger::scope_logger(log_domain const &domain, const std::string& str)
-	: ticks_(SDL_GetTicks()), str_(str), output_(info(domain, false))
+	: ticks_(SDL_GetTicks()), str_(str), output_(debug(domain, false))
 {
 	do_indent();
 	output_ << "BEGIN: " << str_ << "\n";
@@ -115,6 +138,7 @@ scope_logger::~scope_logger()
 	const int ticks = SDL_GetTicks() - ticks_;
 	--indent;
 	do_indent();
+	if (timestamp) output_ << get_timestamp(time(NULL));
 	output_ << "END: " << str_ << " (took " << ticks << "ms)\n";
 }
 
@@ -124,4 +148,7 @@ void scope_logger::do_indent() const
 		output_ << "  ";
 }
 
-} // namespace lg
+std::stringstream wml_error;
+
+} // end namespace lg
+
