@@ -1,52 +1,56 @@
-/* $Id: multiplayer_lobby.cpp 19129 2007-07-26 10:37:50Z soliton $ */
+/* $Id: multiplayer_lobby.cpp 24101 2008-02-27 00:07:32Z soliton $ */
 /*
-   Copyright (C)
+   Copyright (C) 2007 - 2008
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+/** @file multiplayer_lobby.cpp 
+ * A section on the server where players can chat, create and join games.
+ */
+
 #include "global.hpp"
+
+#include "config.hpp"
 #include "filesystem.hpp"
+#include "game_display.hpp"
 #include "marked-up_text.hpp"
 #include "minimap.hpp"
 #include "multiplayer_lobby.hpp"
 #include "replay.hpp"
-#include "wassert.hpp"
 #include "wml_separators.hpp"
 #include "game_config.hpp"
 #include "gettext.hpp"
+#include "log.hpp"
 #include "playmp_controller.hpp"
 #include "show_dialog.hpp"
 #include "sound.hpp"
+#include "wml_exception.hpp"
+
+#define ERR_CF LOG_STREAM(err, config)
 
 namespace {
-
-std::string games_menu_heading()
-{
-	std::ostringstream str;
-	str << HEADING_PREFIX << _("Map") << COLUMN_SEPARATOR << _("Name")
-		<< COLUMN_SEPARATOR << _("Status") << COLUMN_SEPARATOR << _("Settings");
-	return str.str();
-}
-
+std::vector<std::string> empty_string_vector;
 }
 
 namespace mp {
 	gamebrowser::gamebrowser(CVideo& video, const config* map_hashes) :
 	menu(video, empty_string_vector, false, -1, -1, NULL, &menu::bluebg_style),
-	gold_icon_locator_("misc/gold.png"),
-	xp_icon_locator_("misc/units.png"),
+	gold_icon_locator_("themes/gold.png"),
+	xp_icon_locator_("themes/units.png"),
 	vision_icon_locator_("misc/invisible.png"),
-	time_limit_icon_locator_("misc/sand-clock.png"),
-	observer_icon_locator_("misc/eye.png"), map_hashes_(map_hashes),
-	item_height_(100), margin_(5), h_padding_(5),
-	v_padding_(5), header_height_(20), selected_(0), visible_range_(std::pair<size_t,size_t>(0,0)),
+	time_limit_icon_locator_("themes/sand-clock.png"),
+	observer_icon_locator_(game_config::observer_image),
+	no_observer_icon_locator_("misc/no_observer.png"), map_hashes_(map_hashes),
+	item_height_(100), margin_(5), minimap_size_(item_height_ - 2*margin_),  h_padding_(5),
+	header_height_(20), selected_(0), visible_range_(std::pair<size_t,size_t>(0,0)),
 	double_clicked_(false), ignore_next_doubleclick_(false), last_was_doubleclick_(false)
 {
 	set_numeric_keypress_selection(false);
@@ -111,21 +115,21 @@ void gamebrowser::draw_row(const size_t index, const SDL_Rect& item_rect, ROW_TY
 	int xpos = item_rect.x + margin_;
 	int ypos = item_rect.y + margin_;
 
-	//bg_restore(item_rect);
-	//draw_solid_tinted_rectangle(item_rect.x, item_rect.y, item_rect.w, item_rect.h, 0, 0, 0, 0.2, video().getSurface());
+	// Draw minimaps
+	if (game.mini_map != NULL) {
+		int minimap_x = xpos + (minimap_size_ - game.mini_map->w)/2;
+		int minimap_y = ypos + (minimap_size_ - game.mini_map->h)/2;
+		video().blit_surface(minimap_x, minimap_y, game.mini_map);
+	}
+	xpos += minimap_size_ + margin_;
 
-	// draw mini map
-	video().blit_surface(xpos, ypos, game.mini_map);
-
-	xpos += item_height_ + margin_;
-
-	//set font color
+	// Set font color
 	SDL_Color font_color;
 	if (game.vacant_slots > 0) {
-		if (!game.started) {
-			font_color = font::GOOD_COLOUR;
-		} else {
+		if (game.reloaded || game.started) {
 			font_color = font::YELLOW_COLOUR;
+		} else {
+			font_color = font::GOOD_COLOUR;
 		}
 	} else {
 		if (game.observers) {
@@ -135,83 +139,129 @@ void gamebrowser::draw_row(const size_t index, const SDL_Rect& item_rect, ROW_TY
 		}
 	}
 
-	//draw game name
-	const surface name_surf(font::get_rendered_text(font::make_text_ellipsis(game.name, font::SIZE_PLUS, (item_rect.x + item_rect.w) - xpos - margin_), font::SIZE_PLUS, font_color));
+	const surface status_text(font::get_rendered_text(game.status,
+	    font::SIZE_NORMAL, font_color));
+	const int status_text_width = status_text ? status_text->w : 0;
+
+	// First line: draw game name
+	const surface name_surf(font::get_rendered_text(
+	    font::make_text_ellipsis(game.name, font::SIZE_PLUS,
+	        (item_rect.x + item_rect.w) - xpos - margin_ - status_text_width - h_padding_),
+	    font::SIZE_PLUS, font_color));
 	video().blit_surface(xpos, ypos, name_surf);
 
-	// draw gold icon
-	const surface gold_icon(image::get_image(gold_icon_locator_, image::UNSCALED));
-	ypos = item_rect.y + item_rect.h  - margin_ - gold_icon->h;
+	// Draw status text
+	if(status_text) {
+		// Align the bottom of the text with the game name
+		video().blit_surface(item_rect.x + item_rect.w - margin_ - status_text_width,
+		    ypos + name_surf->h - status_text->h, status_text);
+	}
 
-	video().blit_surface(xpos, ypos, gold_icon);
+	// Second line
+	ypos = item_rect.y + item_rect.h/2;
 
-	// draw map info
-	const surface map_info_surf(font::get_rendered_text(font::make_text_ellipsis(game.map_info, font::SIZE_NORMAL, (item_rect.x + item_rect.w) - xpos - margin_), font::SIZE_NORMAL, font::NORMAL_COLOUR));
-	video().blit_surface(xpos, ypos - map_info_surf->h - 4 * v_padding_, map_info_surf);
+	// Draw map info
+	const surface map_info_surf(font::get_rendered_text(
+	    font::make_text_ellipsis(game.map_info, font::SIZE_NORMAL,
+	        (item_rect.x + item_rect.w) - xpos - margin_),
+	    font::SIZE_NORMAL, font::NORMAL_COLOUR));
+	if(map_info_surf) {
+		video().blit_surface(xpos, ypos - map_info_surf->h/2, map_info_surf);
+	}
 
-	xpos += gold_icon->w + h_padding_;
+	// Third line
+	ypos = item_rect.y + item_rect.h  - margin_;
 
-	// draw gold text
-	const surface gold_text(font::get_rendered_text(game.gold, font::SIZE_NORMAL, game.use_map_settings ? font::DISABLED_COLOUR : font::NORMAL_COLOUR));
-	ypos -= abs(gold_icon->h - gold_text->h) / 2;
-	video().blit_surface(xpos, ypos, gold_text);
+	// Draw observer icon
+	const surface observer_icon(image::get_image(game.observers
+	    ? observer_icon_locator_ : no_observer_icon_locator_));
+	if(observer_icon) {
+		video().blit_surface(xpos, ypos - observer_icon->h, observer_icon);
 
-	xpos += gold_text->w + 2 * h_padding_;
+		// Set ypos to the middle of the line, so that 
+		// all text and icons can be aligned symmetrical to it
+		ypos -= observer_icon->h/2;
+		xpos += observer_icon->w + 2 * h_padding_;
+	}
 
-	// draw xp icon
-	const surface xp_icon(image::get_image(xp_icon_locator_, image::UNSCALED));
-	ypos = item_rect.y + item_rect.h  - margin_ - xp_icon->h;
-	video().blit_surface(xpos, ypos, xp_icon);
+	// Draw gold icon
+	const surface gold_icon(image::get_image(gold_icon_locator_));
+	if(gold_icon) {
+		video().blit_surface(xpos, ypos - gold_icon->h/2, gold_icon);
 
-	xpos += xp_icon->w + h_padding_;
+		xpos += gold_icon->w + h_padding_;
+	}
 
-	// draw xp text
-	const surface xp_text(font::get_rendered_text(game.xp, font::SIZE_NORMAL, font::NORMAL_COLOUR));
-	ypos -= abs(xp_icon->h - xp_text->h) / 2;
-	video().blit_surface(xpos, ypos, xp_text);
+	// Draw gold text
+	const surface gold_text(font::get_rendered_text(game.gold, font::SIZE_NORMAL,
+	    game.use_map_settings ? font::GRAY_COLOUR : font::NORMAL_COLOUR));
+	if(gold_text) {
+		video().blit_surface(xpos, ypos - gold_text->h/2, gold_text);
 
-	xpos += xp_text->w + 2 * h_padding_;
+		xpos += gold_text->w + 2 * h_padding_;
+	}
+
+	// Draw xp icon
+	const surface xp_icon(image::get_image(xp_icon_locator_));
+	if(xp_icon) {
+		video().blit_surface(xpos, ypos - xp_icon->h/2, xp_icon);
+
+		xpos += xp_icon->w + h_padding_;
+	}
+
+	// Draw xp text
+	const surface xp_text(font::get_rendered_text(game.xp, font::SIZE_NORMAL,
+	    font::NORMAL_COLOUR));
+	if(xp_text) {
+		video().blit_surface(xpos, ypos - xp_text->h/2, xp_text);
+
+		xpos += xp_text->w + 2 * h_padding_;
+	}
 
 	if(!game.time_limit.empty()) {
-		// draw time icon
-		const surface time_icon(image::get_image(time_limit_icon_locator_, image::UNSCALED));
-		ypos = item_rect.y + item_rect.h  - margin_ - time_icon->h;
-		video().blit_surface(xpos, ypos, time_icon);
+		// Draw time icon
+		const surface time_icon(image::get_image(time_limit_icon_locator_));
+		video().blit_surface(xpos, ypos - time_icon->h/2, time_icon);
 
 		xpos += time_icon->w + h_padding_;
 
-		// draw time text
-		const surface time_text(font::get_rendered_text(game.time_limit, font::SIZE_NORMAL, font::NORMAL_COLOUR));
-		ypos -= abs(time_icon->h - time_text->h) / 2;
-		video().blit_surface(xpos, ypos, time_text);
+		// Draw time text
+		const surface time_text(font::get_rendered_text(game.time_limit,
+		    font::SIZE_NORMAL, font::NORMAL_COLOUR));
+		video().blit_surface(xpos, ypos - time_text->h/2, time_text);
 
 		xpos += time_text->w + 2 * h_padding_;
 	}
 
-	// draw vision icon
-	const surface vision_icon(image::get_image(vision_icon_locator_, image::UNSCALED));
-	video().blit_surface(xpos, ypos, vision_icon);
+	// Draw vision icon
+	const surface vision_icon(image::get_image(vision_icon_locator_));
+	if(vision_icon) {
+		video().blit_surface(xpos, ypos - vision_icon->h/2, vision_icon);
 
-	xpos += vision_icon->w + h_padding_;
-
-	const surface status_text(font::get_rendered_text(game.status, font::SIZE_NORMAL, font_color));
-	const surface vision_text(font::get_rendered_text(font::make_text_ellipsis(game.vision, font::SIZE_NORMAL, maximum<int>((item_rect.x + item_rect.w - margin_ - status_text->w - 2 * h_padding_) - xpos, 0)),font::SIZE_NORMAL, game.use_map_settings ? font::DISABLED_COLOUR : font::NORMAL_COLOUR));
-	// draw vision text
-	video().blit_surface(xpos, ypos, vision_text);
-
-	// draw map settings text
-	if (game.use_map_settings) {
-		const surface map_settings_text(font::get_rendered_text(_("Use map settings"), font::SIZE_NORMAL, font::NORMAL_COLOUR));
-		xpos += vision_text->w + 4 * h_padding_;
-		video().blit_surface(xpos, ypos, map_settings_text);
+		xpos += vision_icon->w + h_padding_;
 	}
 
-	// draw status text
-	xpos = item_rect.x + item_rect.w - margin_ - status_text->w;
-	video().blit_surface(xpos, ypos, status_text);
+	// Draw vision text
+	const surface vision_text(font::get_rendered_text(
+	    font::make_text_ellipsis(game.vision, font::SIZE_NORMAL,
+	        (item_rect.x + item_rect.w) - xpos - margin_),
+	    font::SIZE_NORMAL,
+	    game.use_map_settings ? font::GRAY_COLOUR : font::NORMAL_COLOUR));
+	if(vision_text) {
+		video().blit_surface(xpos, ypos - vision_text->h/2, vision_text);
+	}
 
-	//if(selected_ == index)
-	//	draw_solid_tinted_rectangle(item_rect.x, item_rect.y, item_rect.w, item_rect.h, 153, 0, 0, 0.3, video().getSurface());
+	// Draw map settings text
+	if (game.use_map_settings) {
+		xpos += vision_text->w + 4 * h_padding_;
+		const surface map_settings_text(font::get_rendered_text(
+		    font::make_text_ellipsis(_("Use map settings"), font::SIZE_NORMAL,
+		        (item_rect.x + item_rect.w) - xpos - margin_),
+		    font::SIZE_NORMAL,
+		    (game.verified && game.vacant_slots > 0)
+				? font::GOOD_COLOUR : font::NORMAL_COLOUR));
+		video().blit_surface(xpos, ypos - map_settings_text->h/2, map_settings_text);
+	}
 }
 
 void gamebrowser::handle_event(const SDL_Event& event)
@@ -297,6 +347,7 @@ void gamebrowser::handle_event(const SDL_Event& event)
 				// If we have a double click as the next event, it means
 				// this double click was generated from a click that
 				// already has helped in generating a double click.
+				// ??
 				SDL_Event ev;
 				SDL_PeepEvents(&ev, 1, SDL_PEEKEVENT,
 							   SDL_EVENTMASK(DOUBLE_CLICK_EVENT));
@@ -310,7 +361,7 @@ void gamebrowser::handle_event(const SDL_Event& event)
 }
 
 struct minimap_cache_item {
-	std::string map_data;
+	std::string map_id;
 	surface mini_map;
 	std::string map_info_size;
 };
@@ -318,102 +369,147 @@ struct minimap_cache_item {
 void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 {
 	const bool scrolled_to_max = (has_scrollbar() && get_position() == get_max_position());
+	const bool selection_visible = (selected_ >= visible_range_.first && selected_ <= visible_range_.second);
+	const std::string selected_game = (selected_ < games_.size()) ? games_[selected_].id : "";
 
-	// don't throw the rendered minimaps away
+	item_height_ = 100;
+
+	// Don't throw the rendered minimaps away
 	std::vector<minimap_cache_item> minimap_cache;
 	for(std::vector<game_item>::iterator oldgame = games_.begin(); oldgame != games_.end(); ++oldgame) {
 		minimap_cache_item item;
-		item.map_data = oldgame->map_data;
+		item.map_id = oldgame->id;
 		item.mini_map = oldgame->mini_map;
 		item.map_info_size = oldgame->map_info_size;
 		minimap_cache.push_back(item);
 	}
-		
+
 	games_.clear();
 	config::child_list games = cfg.get_children("game");
 	config::child_iterator game;
 
 	for(game = games.begin(); game != games.end(); ++game) {
+		bool verified = true;
 		games_.push_back(game_item());
+		games_.back().password_required = (**game)["password"] == "yes";
+		games_.back().reloaded = (**game)["savegame"] == "yes";
 		if((**game)["mp_era"] != "") {
 			const config* const era_cfg = game_config.find_child("era", "id", (**game)["mp_era"]);
-			games_.back().map_info = era_cfg != NULL ? era_cfg->get_attribute("name") : _("Unknown era");
-
+			utils::string_map symbols;
+			symbols["era_id"] = (**game)["mp_era"];
+			if (era_cfg != NULL) {
+				games_.back().map_info = era_cfg->get_attribute("name");
+			} else {
+				games_.back().map_info = vgettext("Unknown era: $era_id", symbols);
+				verified = false;
+			}
 		} else {
 			games_.back().map_info = _("Unknown era");
+			verified = false;
 		}
 		games_.back().map_data = (**game)["map_data"];
-		if(games_.back().map_data == "")
+		if(games_.back().map_data.empty()) {
 			games_.back().map_data = read_map((**game)["map"]);
+		}
 
-		if(games_.back().map_data != "") {
+		if(! games_.back().map_data.empty()) {
 			try {
 				std::vector<minimap_cache_item>::iterator i;
 				bool found = false;
 				for(i = minimap_cache.begin(); i != minimap_cache.end() && !found; ++i) {
-					if (i->map_data == games_.back().map_data) {
+					if (i->map_id == games_.back().id) {
 						found = true;
 						games_.back().map_info_size = i->map_info_size;
 						games_.back().mini_map = i->mini_map;
 					}
 				}
 				if (!found) {
-					// parsing the map and generating the minimap are both cpu expensive
-					gamemap map(game_config, games_.back().map_data);
-					games_.back().mini_map = image::getMinimap(item_height_ - margin_, item_height_ - 2 * margin_, map, 0);
-					games_.back().map_info_size = lexical_cast_default<std::string, int>
-						(map.x(), "??") + std::string("x") + lexical_cast_default<std::string, int>(map.y(), "??");
+					// Parsing the map and generating the minimap are both cpu expensive
+					gamemap map(game_config, games_.back().map_data); 
+					games_.back().mini_map = image::getMinimap(minimap_size_, minimap_size_, map, 0);
+					games_.back().map_info_size = lexical_cast_default<std::string, int>(map.w(), "??")
+						+ std::string("x") + lexical_cast_default<std::string, int>(map.h(), "??");
 				}
-
 				games_.back().map_info += " - " + games_.back().map_info_size;
-				
 			} catch(gamemap::incorrect_format_exception &e) {
-				std::cerr << "illegal map: " << e.msg_ << "\n";
+				ERR_CF << "illegal map: " << e.msg_ << "\n";
+				verified = false;
+			} catch(twml_exception& e) {
+				ERR_CF <<  "map could not be loaded: " << e.dev_message << '\n';
+				verified = false;
 			}
 		} else {
 			games_.back().map_info += " - ??x??";
 		}
 		games_.back().map_info += " ";
-		if((**game)["mp_scenario"] != "" && map_hashes_) {
-			const config* level_cfg = game_config.find_child("generic_multiplayer", "id", (**game)["mp_scenario"]);
-			if(level_cfg == NULL)
-				level_cfg = game_config.find_child("multiplayer", "id", (**game)["mp_scenario"]);
-			games_.back().map_info += level_cfg != NULL ? level_cfg->get_attribute("name") : _("Unknown scenario");
+		if((**game)["mp_scenario"] != "") {
+			// check if it's a multiplayer scenario
+			const config* level_cfg = game_config.find_child("multiplayer", "id", (**game)["mp_scenario"]);
+			if(level_cfg == NULL) {
+				// check if it's a user map
+				level_cfg = game_config.find_child("generic_multiplayer", "id", (**game)["mp_scenario"]);
+			}
 			if(level_cfg) {
-				const std::string& hash = (**game)["hash"];
-				bool hash_found = false;
-				for(string_map::const_iterator i = map_hashes_->values.begin(); i != map_hashes_->values.end(); ++i) {
-					if(i->first == (**game)["mp_scenario"] && i->second == hash) {
-						hash_found = true;
-						break;
+				games_.back().map_info += level_cfg->get_attribute("name");
+				// reloaded games do not match the original scenario hash,
+				// so it makes no sense to test them, they always would appear
+				// as remote scenarios
+				if (map_hashes_ && !games_.back().reloaded) {
+					const std::string& hash = (**game)["hash"];
+					bool hash_found = false;
+					for(string_map::const_iterator i = map_hashes_->values.begin(); i != map_hashes_->values.end(); ++i) {
+						if(i->first == (**game)["mp_scenario"] && i->second == hash) {
+							hash_found = true;
+							break;
+						}
+					}
+					if(!hash_found) {
+						games_.back().map_info += " - ";
+						games_.back().map_info += _("Remote scenario");
+						verified = false;
 					}
 				}
-				if(!hash_found) {
-					games_.back().map_info += " - ";
-					games_.back().map_info += _("Remote scenario");
-				}
+			} else {
+				utils::string_map symbols;
+				symbols["scenario_id"] = (**game)["mp_scenario"];
+				games_.back().map_info += vgettext("Unknown scenario: $scenario_id", symbols);
+				verified = false;
 			}
 		} else {
 			games_.back().map_info += _("Unknown scenario");
+			verified = false;
 		}
+		if (games_.back().reloaded) {
+			games_.back().map_info += " - ";
+			games_.back().map_info += _("Reloaded game");
+			verified = false;
+		}
+		games_.back().id = (**game)["id"];
 		games_.back().name = (**game)["name"];
 		const std::string& turn = (**game)["turn"];
 		const std::string& slots = (**game)["slots"];
 		games_.back().vacant_slots = lexical_cast_default<size_t>(slots, 0);
+		games_.back().current_turn = 0;
 		if(turn != "") {
 			games_.back().started = true;
-			games_.back().status = _("Turn") + (" " + turn);
+			int index = turn.find_first_of('/');
+			if (index > -1){
+				const std::string current_turn = turn.substr(0, index);
+				games_.back().current_turn = lexical_cast<unsigned int>(current_turn);
+			}
+			games_.back().status = _("Turn ") + turn;
 		} else {
 			games_.back().started = false;
-			if(slots != "")
-				games_.back().status = std::string(ngettext(_("Vacant Slot:"), _("Vacant Slots:"), games_.back().vacant_slots)) + " " + slots;
+			if (games_.back().vacant_slots > 0) {
+				games_.back().status = std::string(_n("Vacant Slot:", "Vacant Slots:",
+						games_.back().vacant_slots)) + " " + slots;
+				if (games_.back().password_required) {
+					games_.back().status += std::string(" (") + std::string(_("Password Required")) + ")";
+				}
+			}
 		}
 
-		if((**game)["mp_use_map_settings"] == "yes") {
-			games_.back().use_map_settings = true;
-		} else {
-			games_.back().use_map_settings = false;
-		}
+		games_.back().use_map_settings = ((**game)["mp_use_map_settings"] == "yes");
 		games_.back().gold = (**game)["mp_village_gold"];
 		if((**game)["mp_fog"] == "yes") {
 			games_.back().vision = _("Fog");
@@ -435,22 +531,41 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 			games_.back().shroud = false;
 		}
 		if((**game)["mp_countdown"] == "yes" ) {
-			games_.back().time_limit = (**game)["mp_countdown_init_time"] + " / +" + (**game)["mp_countdown_turn_bonus"] + " " + (**game)["mp_countdown_action_bonus"];
+			games_.back().time_limit =   (**game)["mp_countdown_init_time"] + " / +"
+			                           + (**game)["mp_countdown_turn_bonus"] + " "
+			                           + (**game)["mp_countdown_action_bonus"];
 		} else {
 			games_.back().time_limit = "";
 		}
-
 		games_.back().xp = (**game)["experience_modifier"] + "%";
 		games_.back().observers = (**game)["observer"] != "no" ? true : false;
+		games_.back().verified = verified;
 	}
 	set_full_size(games_.size());
 	set_shown_size(inner_location().h / row_height());
-	if(scrolled_to_max) {
-		set_position(get_max_position());
+
+	// Try to preserve the game selection
+	if (!selected_game.empty()) {
+		for (unsigned int i=0; i < games_.size(); i++) {
+			if (games_[i].id == selected_game) {
+				selected_ = i;
+				break;
+			}
+		}
 	}
-	scroll(get_position());
 	if(selected_ >= games_.size())
 		selected_ = maximum<long>(static_cast<long>(games_.size()) - 1, 0);
+
+	if (scrolled_to_max) {
+		set_position(get_max_position());
+	} else {
+		// Keep the selected game visible if it was visible before
+		if (selection_visible && (visible_range_.first > selected_
+								  || visible_range_.second < selected_)) {
+			set_position(selected_);
+		}
+	}
+	scroll(get_position());
 	set_dirty();
 }
 
@@ -520,23 +635,25 @@ bool lobby::lobby_sorter::less(int column, const gui::menu::item& row1, const gu
 	return false;
 }
 
-lobby::lobby(display& disp, const config& cfg, chat& c, config& gamelist) :
+lobby::lobby(game_display& disp, const config& cfg, chat& c, config& gamelist) :
 	mp::ui(disp, _("Game Lobby"), cfg, c, gamelist),
 
 	observe_game_(disp.video(), _("Observe Game")),
 	join_game_(disp.video(), _("Join Game")),
 	create_game_(disp.video(), _("Create Game")),
 	skip_replay_(disp.video(), _("Quick Replays"), gui::button::TYPE_CHECK),
+#ifndef USE_TINY_GUI
 	game_preferences_(disp.video(), _("Preferences")),
+#endif
 	quit_game_(disp.video(), _("Quit")),
-	sorter_(gamelist),
+	last_selected_game_(-1), sorter_(gamelist),
 	games_menu_(disp.video(),cfg.child("multiplayer_hashes"))
 {
 	skip_replay_.set_check(preferences::skip_mp_replay());
 	skip_replay_.set_help_string(_("Skip quickly to the active turn when observing"));
 	game_config::debug = false;
 	gamelist_updated();
-	sound::play_music_repeatedly(game_config::title_music);
+	sound::play_music_repeatedly(game_config::lobby_music);
 }
 
 void lobby::hide_children(bool hide)
@@ -548,7 +665,9 @@ void lobby::hide_children(bool hide)
 	join_game_.hide(hide);
 	create_game_.hide(hide);
 	skip_replay_.hide(hide);
+#ifndef USE_TINY_GUI
 	game_preferences_.hide(hide);
+#endif
 	quit_game_.hide(hide);
 }
 
@@ -556,12 +675,35 @@ void lobby::layout_children(const SDL_Rect& rect)
 {
 	ui::layout_children(rect);
 
-	join_game_.set_location(xscale(12),yscale(7));
-	observe_game_.set_location(join_game_.location().x + join_game_.location().w + 5,yscale(7));
-	create_game_.set_location(observe_game_.location().x + observe_game_.location().w + 5,yscale(7));
-	skip_replay_.set_location(create_game_.location().x + 2*(create_game_.location().w),yscale(7));
-	game_preferences_.set_location(skip_replay_.location().x + skip_replay_.location().w + 5,yscale(7));
-	quit_game_.set_location(game_preferences_.location().x + game_preferences_.location().w + 5,yscale(7));
+#ifdef USE_TINY_GUI
+	int btn_space = 3;
+	int xborder   = 0;
+	int yborder   = 0;
+#else
+	int btn_space = 5;
+	int xborder   = 10;
+	int yborder   = 7;
+#endif
+
+	// Align to the left border
+	join_game_.set_location(xscale(xborder), yscale(yborder));
+	observe_game_.set_location(join_game_.location().x + join_game_.location().w + btn_space, yscale(yborder));
+	create_game_.set_location(observe_game_.location().x + observe_game_.location().w + btn_space, yscale(yborder));
+
+#ifndef USE_TINY_GUI
+	// Align 'Quit' to the right border
+	quit_game_.set_location(xscale(xscale_base - xborder) - quit_game_.location().w, yscale(yborder));
+
+	// Align in the middle between the right and left buttons
+	int space = (quit_game_.location().x - create_game_.location().x - create_game_.location().w
+	             - skip_replay_.location().w - game_preferences_.location().w - btn_space) / 2;
+	if (space < btn_space) space = btn_space;
+	skip_replay_.set_location(create_game_.location().x + create_game_.location().w + space, yscale(yborder));
+	game_preferences_.set_location(quit_game_.location().x - game_preferences_.location().w - space, yscale(yborder));
+#else
+	skip_replay_.set_location(create_game_.location().x + create_game_.location().w, yscale(yborder));
+	quit_game_.set_location(skip_replay_.location().x + skip_replay_.location().w + btn_space, yscale(yborder));
+#endif
 
 	games_menu_.set_location(client_area().x, client_area().y + title().height());
 	games_menu_.set_measurements(client_area().w, client_area().h
@@ -572,55 +714,72 @@ void lobby::gamelist_updated(bool silent)
 {
 	ui::gamelist_updated(silent);
 
-	std::vector<std::string> game_strings;
 	const config* list = gamelist().child("gamelist");
 	if(list == NULL) {
 		// No gamelist yet. Do not update anything.
 		return;
 	}
 	games_menu_.set_game_items(*list, game_config());
-	join_game_.hide(!games_menu_.selection_is_joinable());
-	observe_game_.hide(!games_menu_.selection_is_observable());
+	join_game_.enable(games_menu_.selection_is_joinable());
+	observe_game_.enable(games_menu_.selection_is_observable());
 }
 
 void lobby::process_event()
 {
-	join_game_.hide(!games_menu_.selection_is_joinable());
-	observe_game_.hide(!games_menu_.selection_is_observable());
+	join_game_.enable(games_menu_.selection_is_joinable());
+	observe_game_.enable(games_menu_.selection_is_observable());
 
 	const bool observe = (observe_game_.pressed() || (games_menu_.selected() && !games_menu_.selection_is_joinable())) && games_menu_.selection_is_observable();
 	const bool join = (join_game_.pressed() || games_menu_.selected()) && games_menu_.selection_is_joinable();
+	games_menu_.reset_selection();
 	preferences::set_skip_mp_replay(skip_replay_.checked());
 	playmp_controller::set_replay_last_turn(0);
+	preferences::set_message_private(false);
+
+	int selected_game = games_menu_.selection();
+	if (selected_game != last_selected_game_) {
+		if (games_menu_.empty()) {
+			set_selected_game("");
+		} else {
+			set_selected_game(games_menu_.selected_game().id);
+		}
+		ui::gamelist_updated();
+		last_selected_game_ = selected_game;
+	}
 
 	if(join || observe) {
-		const config* game = gamelist().child("gamelist");
-
 		const int selected = games_menu_.selection();
-		if(game != NULL && selected >= 0) {
-			const config::const_child_itors i = game->child_range("game");
-			wassert(i.first + selected < i.second);
+		if(!games_menu_.empty() && selected >= 0) {
+			gamebrowser::game_item game = games_menu_.selected_game();
 
-			const config& game_cfg = **(i.first + selected);
-			const std::string& id = game_cfg["id"];
+			std::string password;
+			if(join && game.password_required) {
+				const int res = gui::show_dialog(disp_, NULL, _("Password Required"),
+				          _("Joining this game requires a password."),
+						  gui::OK_CANCEL, NULL, NULL, _("Password: "), &password);
+				if(res != 0) {
+					return;
+				}
+			}
 
 			config response;
 			config& join = response.add_child("join");
-			join["id"] = id;
+			join["id"] = game.id;
 			if (observe){
 				join["observe"] = "yes";
 			}
 			else{
 				join["observe"] = "no";
 			}
-			network::send_data(response);
+
+			if(!password.empty()) {
+				join["password"] = password;
+			}
+			network::send_data(response, 0, true);
 
 			if(observe) {
-				const std::string& str_current_turn = game_cfg["turn"];
-				int index = str_current_turn.find_first_of('/');
-				if (index > -1){
-					const std::string strTest = str_current_turn.substr(0, index);
-					playmp_controller::set_replay_last_turn(lexical_cast<unsigned int>(strTest));
+				if (game.started){
+					playmp_controller::set_replay_last_turn(game.current_turn);
 				}
 				set_result(OBSERVE);
 			} else {
@@ -635,10 +794,12 @@ void lobby::process_event()
 		return;
 	}
 
+#ifndef USE_TINY_GUI
 	if(game_preferences_.pressed()) {
 		set_result(PREFERENCES);
 		return;
 	}
+#endif
 
 	if(quit_game_.pressed()) {
 		recorder.set_skip(false);
@@ -647,10 +808,13 @@ void lobby::process_event()
 	}
 }
 
-
 void lobby::process_network_data(const config& data, const network::connection sock)
 {
 	ui::process_network_data(data, sock);
+
+	// Invalidate game selection for the player list
+	last_selected_game_ = -1;
 }
 
-}
+} // end namespace mp
+
