@@ -1,16 +1,20 @@
-/* $Id: network.cpp 12503 2006-07-11 20:57:03Z noyga $ */
+/* $Id: network.cpp 23927 2008-02-18 08:15:09Z suokko $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file network.cpp
+//! Networking
+
 #include "global.hpp"
 
 #include "serialization/binary_wml.hpp"
@@ -20,17 +24,17 @@
 #include "network.hpp"
 #include "network_worker.hpp"
 #include "thread.hpp"
-#include "util.hpp"
-#include "wassert.hpp"
 
 #include "SDL_net.h"
 
 #include <algorithm>
+#include <cassert>
 #include <cerrno>
 #include <queue>
 #include <iostream>
 #include <set>
 #include <vector>
+#include <ctime>
 
 #include <signal.h>
 #if defined(_WIN32) || defined(__WIN32__) || defined (WIN32)
@@ -49,31 +53,32 @@
 #define SOCKET int
 #endif
 
+#define DBG_NW LOG_STREAM(debug, network)
 #define LOG_NW LOG_STREAM(info, network)
 #define WRN_NW LOG_STREAM(warn, network)
-// only warnings and not errors to avoid DoS by log flooding
+#define ERR_NW LOG_STREAM(err, network)
+// Only warnings and not errors to avoid DoS by log flooding
 
 namespace {
 
-//We store the details of a connection in a map that must be looked up by its handle.
-//This allows a connection to be disconnected and then recovered, but the handle remains
-//the same, so it's all seamless to the user
+// We store the details of a connection in a map that must be looked up by its handle.
+// This allows a connection to be disconnected and then recovered,
+// but the handle remains the same, so it's all seamless to the user.
 struct connection_details {
 	connection_details(TCPsocket sock, const std::string& host, int port)
 		: sock(sock), host(host), port(port), remote_handle(0),
-	      connected_at(SDL_GetTicks()), sent(0), received(0)
+	      connected_at(SDL_GetTicks())
 	{}
 
 	TCPsocket sock;
 	std::string host;
 	int port;
 
-	//the remote handle is the handle assigned to this connection by the remote host.
-	//is 0 before a handle has been assigned.
+	// The remote handle is the handle assigned to this connection by the remote host.
+	// Is 0 before a handle has been assigned.
 	int remote_handle;
 
 	int connected_at;
-	int sent, received;
 };
 
 typedef std::map<network::connection,connection_details> connection_map;
@@ -81,13 +86,18 @@ connection_map connections;
 
 network::connection connection_id = 1;
 
-int create_connection(TCPsocket sock, const std::string& host, int port)
+//! Stores the time of the last server ping we received.
+time_t last_ping, last_ping_check = 0;
+
+} // end anon namespace
+
+static int create_connection(TCPsocket sock, const std::string& host, int port)
 {
 	connections.insert(std::pair<network::connection,connection_details>(connection_id,connection_details(sock,host,port)));
 	return connection_id++;
 }
 
-connection_details& get_connection_details(network::connection handle)
+static connection_details& get_connection_details(network::connection handle)
 {
 	const connection_map::iterator i = connections.find(handle);
 	if(i == connections.end()) {
@@ -97,28 +107,28 @@ connection_details& get_connection_details(network::connection handle)
 	return i->second;
 }
 
-TCPsocket get_socket(network::connection handle)
+static TCPsocket get_socket(network::connection handle)
 {
 	return get_connection_details(handle).sock;
 }
 
-void remove_connection(network::connection handle)
+static void remove_connection(network::connection handle)
 {
 	connections.erase(handle);
 }
 
-bool is_pending_remote_handle(network::connection handle)
+static bool is_pending_remote_handle(network::connection handle)
 {
 	const connection_details& details = get_connection_details(handle);
 	return details.host != "" && details.remote_handle == 0;
 }
 
-void set_remote_handle(network::connection handle, int remote_handle)
+static void set_remote_handle(network::connection handle, int remote_handle)
 {
 	get_connection_details(handle).remote_handle = remote_handle;
 }
 
-void check_error()
+static void check_error()
 {
 	const TCPsocket sock = network_worker_pool::detect_error();
 	if(sock) {
@@ -130,19 +140,43 @@ void check_error()
 	}
 }
 
+//! Check whether too much time since the last server ping has passed and we
+//! timed out. If the last check is too long ago reset the last_ping to 'now'.
+//! This happens when we "freeze" the client one way or another or we just
+//! didn't try to receive data.
+static void check_timeout()
+{
+	if (network::nconnections() == 0) {
+		LOG_NW << "No network connections but last_ping is: " << last_ping;
+		last_ping = 0;
+		return;
+	}
+	const time_t& now = time(NULL);
+	DBG_NW << "Last ping: '" << last_ping << "' Current time: '" << now
+			<< "' Time since last ping: " << now - last_ping << "s\n";
+	// Reset last_ping if we didn't check for the last 10s.
+	if (last_ping_check + 10 <= now) last_ping = now;
+	if (static_cast<time_t>(last_ping + network::ping_timeout) <= now) {
+		int timeout = now - last_ping;
+		ERR_NW << "No server ping since " << timeout
+				<< " seconds. Connection timed out.\n";
+		utils::string_map symbols;
+		symbols["timeout"] = lexical_cast<std::string>(timeout);
+		throw network::error(vngettext("No server ping since $timeout second. "
+				"Connection timed out.", "No server ping since $timeout seconds. "
+				"Connection timed out.", timeout, symbols));
+	}
+	last_ping_check = now;
+}
+
+
+namespace {
+
 SDLNet_SocketSet socket_set = 0;
 std::set<network::connection> waiting_sockets;
 typedef std::vector<network::connection> sockets_list;
 sockets_list sockets;
 
-struct schema_pair
-{
-	compression_schema incoming, outgoing;
-};
-
-typedef std::map<network::connection,schema_pair> schema_map;
-
-schema_map schemas;
 
 struct partial_buffer {
 	partial_buffer() : upto(0) {}
@@ -157,10 +191,15 @@ std::set<network::connection> bad_sockets;
 
 network_worker_pool::manager* worker_pool_man = NULL;
 
-}
+} // end anon namespace
 
 namespace network {
 
+//! Amount of seconds after the last server ping when we assume to have timed out.
+//! When set to '0' ping timeout isn't checked.
+//! Gets set in preferences::manager according to the preferences file.
+unsigned int ping_timeout = 0;
+
 connection_stats::connection_stats(int sent, int received, int connected_at)
        : bytes_sent(sent), bytes_received(received), time_connected(SDL_GetTicks() - connected_at)
 {}
@@ -168,7 +207,7 @@ connection_stats::connection_stats(int sent, int received, int connected_at)
 connection_stats get_connection_stats(connection connection_num)
 {
 	connection_details& details = get_connection_details(connection_num);
-	return connection_stats(details.sent,details.received,details.connected_at);
+	return connection_stats(get_send_stats(connection_num).total,get_receive_stats(connection_num).total,details.connected_at);
 }
 
 error::error(const std::string& msg, connection sock) : message(msg), socket(sock)
@@ -180,31 +219,31 @@ error::error(const std::string& msg, connection sock) : message(msg), socket(soc
 
 void error::disconnect()
 {
-	network::disconnect(socket);
+	if(socket) network::disconnect(socket);
 }
 
-manager::manager(size_t nthreads) : free_(true)
+manager::manager(size_t min_threads, size_t max_threads) : free_(true)
 {
-	//if the network is already being managed
+	// If the network is already being managed
 	if(socket_set) {
 		free_ = false;
 		return;
 	}
 
-	//on Unix-based systems, set sigpipe to be ignored
+	// On Unix-based systems, set sigpipe to be ignored
 #if !(defined(_WIN32) || defined(__APPLE__) || defined(__AMIGAOS4__))
 	WRN_NW << "ignoring SIGPIPE\n";
 	signal(SIGPIPE,SIG_IGN);
 #endif
 
 	if(SDLNet_Init() == -1) {
-		LOG_STREAM(err, network) << "could not initialize SDLNet; throwing error...\n";
+		ERR_NW << "could not initialize SDLNet; throwing error...\n";
 		throw error(SDL_GetError());
 	}
 
 	socket_set = SDLNet_AllocSocketSet(512);
 
-	worker_pool_man = new network_worker_pool::manager(nthreads);
+	worker_pool_man = new network_worker_pool::manager(min_threads, max_threads);
 }
 
 manager::~manager()
@@ -233,7 +272,7 @@ server_manager::server_manager(int port, CREATE_SERVER create_server) : free_(fa
 			}
 		}
 
-		LOG_NW << "server socket initialized: " << server_socket << "\n";
+		DBG_NW << "server socket initialized: " << server_socket << "\n";
 		free_ = true;
 	}
 }
@@ -296,7 +335,7 @@ namespace {
 		IPaddress localAddress;
 		int sflag;
 	};
-}
+} // end namespace
 
 void connect_operation::run()
 {
@@ -314,7 +353,7 @@ void connect_operation::run()
 		return;
 	}
 
-// use non blocking IO
+// Use non blocking IO
 #if defined(_WIN32) || defined(__WIN32__) || defined(WIN32)
 	{
 		unsigned long mode = 1;
@@ -322,7 +361,7 @@ void connect_operation::run()
 	}
 #elif !defined(__BEOS__)
 	int flags;
-	flags = fcntl(((_TCPsocket*)sock)->channel, F_GETFL, 0);
+	flags = fcntl((reinterpret_cast<_TCPsocket*>(sock))->channel, F_GETFL, 0);
 #if defined(O_NONBLOCK)
 	flags |= O_NONBLOCK;
 #elif defined(O_NDELAY)
@@ -330,7 +369,7 @@ void connect_operation::run()
 #elif defined(FNDELAY)
 	flags |= FNDELAY;
 #endif
-	if(fcntl(((_TCPsocket*)sock)->channel, F_SETFL, flags) == -1) {
+	if(fcntl((reinterpret_cast<_TCPsocket*>(sock))->channel, F_SETFL, flags) == -1) {
 		error_ = ("Could not make socket non-blocking: " + std::string(strerror(errno))).c_str();
 		return;
 	}
@@ -342,14 +381,14 @@ void connect_operation::run()
 	}
 #endif
 
-	//if this is a server socket
+	// If this is a server socket
 	if(hostname == NULL) {
 		const threading::lock l(get_mutex());
 		connect_ = create_connection(sock,"",port_);
 		return;
 	}
 
-	//send data telling the remote host that this is a new connection
+	// Send data telling the remote host that this is a new connection
 	char buf[4];
 	SDLNet_Write32(0,buf);
 	const int nbytes = SDLNet_TCP_Send(sock,buf,4);
@@ -359,17 +398,17 @@ void connect_operation::run()
 		return;
 	}
 
-	//no blocking operations from here on
+	// No blocking operations from here on
 	const threading::lock l(get_mutex());
-	LOG_NW << "sent handshake...\n";
+	DBG_NW << "sent handshake...\n";
 
 	if(is_aborted()) {
-		LOG_NW << "connect operation aborted by calling thread\n";
+		DBG_NW << "connect operation aborted by calling thread\n";
 		SDLNet_TCP_Close(sock);
 		return;
 	}
 
-	//allocate this connection a connection handle
+	// Allocate this connection a connection handle
 	connect_ = create_connection(sock,host_,port_);
 
 	const int res = SDLNet_TCP_AddSocket(socket_set,sock);
@@ -382,13 +421,11 @@ void connect_operation::run()
 	waiting_sockets.insert(connect_);
 
 	sockets.push_back(connect_);
-	wassert(schemas.count(connect_) == 0);
-	schemas.insert(std::pair<network::connection,schema_pair>(connect_,schema_pair()));
 
 	while(!notify_finished());
 }
 
-}
+} // end namespace
 
 connection connect(const std::string& host, int port)
 {
@@ -416,18 +453,18 @@ connection accept_connection()
 		return 0;
 	}
 
-	//a connection isn't considered 'accepted' until it has sent its initial handshake.
-	//The initial handshake is a 4 byte value, which is 0 for a new connection, or the
-	//handle of the connection if it's trying to recover a lost connection.
+	// A connection isn't considered 'accepted' until it has sent its initial handshake.
+	// The initial handshake is a 4 byte value, which is 0 for a new connection,
+	// or the handle of the connection if it's trying to recover a lost connection.
 
-	//a list of all the sockets which have connected, but haven't had their initial
-	//handshake received
+	//! A list of all the sockets which have connected,
+	//! but haven't had their initial handshake received.
 	static std::vector<TCPsocket> pending_sockets;
 	static SDLNet_SocketSet pending_socket_set = 0;
 
 	const TCPsocket sock = SDLNet_TCP_Accept(server_socket);
 	if(sock) {
-		LOG_NW << "received connection. Pending handshake...\n";
+		DBG_NW << "received connection. Pending handshake...\n";
 		pending_sockets.push_back(sock);
 		if(pending_socket_set == 0) {
 			pending_socket_set = SDLNet_AllocSocketSet(512);
@@ -447,22 +484,22 @@ connection accept_connection()
 		return 0;
 	}
 
-	LOG_NW << "pending socket activity...\n";
+	DBG_NW << "pending socket activity...\n";
 
 	for(std::vector<TCPsocket>::iterator i = pending_sockets.begin(); i != pending_sockets.end(); ++i) {
 		if(!SDLNet_SocketReady(*i)) {
 			continue;
 		}
 
-		//receive the 4 bytes telling us if they're a new connection or trying to
-		//recover a connection
+		// Receive the 4 bytes telling us if they're a new connection
+		// or trying to recover a connection
 		char buf[4];
 
 		const TCPsocket sock = *i;
 		SDLNet_TCP_DelSocket(pending_socket_set,sock);
 		pending_sockets.erase(i);
 
-		LOG_NW << "receiving data from pending socket...\n";
+		DBG_NW << "receiving data from pending socket...\n";
 
 		const int len = SDLNet_TCP_Recv(sock,buf,4);
 		if(len != 4) {
@@ -473,7 +510,7 @@ connection accept_connection()
 
 		const int handle = SDLNet_Read32(buf);
 
-		LOG_NW << "received handshake from client: '" << handle << "'\n";
+		DBG_NW << "received handshake from client: '" << handle << "'\n";
 
 		const int res = SDLNet_TCP_AddSocket(socket_set,sock);
 		if(res == -1) {
@@ -485,46 +522,45 @@ connection accept_connection()
 
 		const connection connect = create_connection(sock,"",0);
 
-		//send back their connection number
+		// Send back their connection number
 		SDLNet_Write32(connect,buf);
 		const int nbytes = SDLNet_TCP_Send(sock,buf,4);
 		if(nbytes != 4) {
+			SDLNet_TCP_DelSocket(socket_set,sock);
 			SDLNet_TCP_Close(sock);
+			remove_connection(connect);
 			throw network::error(_("Could not send initial handshake"));
 		}
 
 		waiting_sockets.insert(connect);
 		sockets.push_back(connect);
-		wassert(schemas.count(connect) == 0);
-		schemas.insert(std::pair<network::connection,schema_pair>(connect,schema_pair()));
 		return connect;
 	}
 
 	return 0;
 }
 
-bool disconnect(connection s)
+bool disconnect(connection s, bool force)
 {
 	if(s == 0) {
 		while(sockets.empty() == false) {
-			wassert(sockets.back() != 0);
+			assert(sockets.back() != 0);
+//			disconnect(sockets.back(), true);
 			while(disconnect(sockets.back()) == false) {
-				SDL_Delay(10);
+				SDL_Delay(1);
 			}
 		}
-
 		return true;
 	}
+	if (!is_server()) last_ping = 0;
 
 	const connection_map::iterator info = connections.find(s);
 	if(info != connections.end()) {
-		const bool res = network_worker_pool::close_socket(info->second.sock);
-		if(res == false) {
+		if (!network_worker_pool::close_socket(info->second.sock, force)) {
 			return false;
 		}
 	}
 
-	schemas.erase(s);
 	bad_sockets.erase(s);
 
 	std::deque<network::connection>::iterator dqi = std::find(disconnection_queue.begin(),disconnection_queue.end(),s);
@@ -545,7 +581,7 @@ bool disconnect(connection s)
 		remove_connection(s);
 	} else {
 		if(sockets.size() == 1) {
-			LOG_NW << "valid socket: " << (int)*sockets.begin() << "\n";
+			DBG_NW << "valid socket: " << static_cast<int>(*sockets.begin()) << "\n";
 		}
 	}
 
@@ -557,22 +593,23 @@ void queue_disconnect(network::connection sock)
 	disconnection_queue.push_back(sock);
 }
 
-connection receive_data(config& cfg, connection connection_num, int timeout)
+connection receive_data(config& cfg, connection connection_num, unsigned int timeout)
 {
-	int cur_ticks = SDL_GetTicks();
-	while(timeout >= 0) {
+	unsigned int start_ticks = SDL_GetTicks();
+	while(true) {
 		const connection res = receive_data(cfg,connection_num);
 		if(res != 0) {
 			return res;
 		}
 
-		if(timeout > 0) {
+		if(timeout > SDL_GetTicks() - start_ticks) {
 			SDL_Delay(1);
 		}
+		else
+		{
+			break;
+		}
 
-		const int ticks = SDL_GetTicks();
-		timeout -= maximum<int>(1,(ticks - cur_ticks));
-		cur_ticks = ticks;
 	}
 
 	return 0;
@@ -607,8 +644,8 @@ connection receive_data(config& cfg, connection connection_num)
 		const TCPsocket sock = details.sock;
 		if(SDLNet_SocketReady(sock)) {
 
-			//see if this socket is still waiting for it to be assigned its remote handle
-			//if it is, then the first 4 bytes must be the remote handle.
+			// See if this socket is still waiting for it to be assigned its remote handle.
+			// If it is, then the first 4 bytes must be the remote handle.
 			if(is_pending_remote_handle(*i)) {
 				char buf[4];
 				int len = SDLNet_TCP_Recv(sock,buf,4);
@@ -630,10 +667,22 @@ connection receive_data(config& cfg, connection connection_num)
 		}
 	}
 
-	std::vector<char> buf;
+
 	TCPsocket sock = connection_num == 0 ? 0 : get_socket(connection_num);
-	sock = network_worker_pool::get_received_data(sock,buf);
-	if(sock == NULL) {
+	TCPsocket s = sock;
+	sock = network_worker_pool::get_received_data(sock,cfg);
+	if (sock == NULL) {
+		if (!is_server() && last_ping != 0 && ping_timeout != 0)
+		{
+			if (connection_num == 0)
+			{
+				s = get_socket(sockets.back());
+			}
+			if (!network_worker_pool::is_locked(s))
+			{
+				check_timeout();
+			}
+		}
 		return 0;
 	}
 
@@ -646,26 +695,32 @@ connection receive_data(config& cfg, connection connection_num)
 			break;
 		}
 	}
-
-	if(result == 0) {
-		assert(false);
-		return result;
+	if(!cfg.empty()) {
+		DBG_NW << "RECEIVED from: " << result << ": " << cfg; 
 	}
+
+	assert(result != 0);
 	waiting_sockets.insert(result);
-
-	const schema_map::iterator schema = schemas.find(result);
-	wassert(schema != schemas.end());
-
-	std::string buffer(buf.begin(), buf.end());
-	std::istringstream stream(buffer);
-	read_compressed(cfg, stream, schema->second.incoming);
-
+	if(!is_server()) {
+		const time_t& now = time(NULL);
+		const string_map::const_iterator ping = cfg.values.find("ping");
+		if (ping != cfg.values.end()) {
+			LOG_NW << "Lag: " << (now - lexical_cast<time_t>(cfg["ping"])) << "\n";
+			last_ping = now;
+		} else if (last_ping != 0) {
+			last_ping = now;
+		}			
+	}
 	return result;
 }
 
-void send_data(const config& cfg, connection connection_num)
+//! @todo Note the gzipped parameter should be removed later, we want to send
+//! all data gzipped. This can be done once the campaign server is also updated
+//! to work with gzipped data.
+void send_data(const config& cfg, connection connection_num, const bool gzipped)
 {
-	LOG_NW << "in send_data()...\n";
+	DBG_NW << "in send_data()...\n";
+	
 	if(cfg.empty()) {
 		return;
 	}
@@ -674,42 +729,26 @@ void send_data(const config& cfg, connection connection_num)
 		return;
 	}
 
-	log_scope2(network, "sending data");
+//	log_scope2(network, "sending data");
 	if(!connection_num) {
-		LOG_NW << "sockets: " << sockets.size() << "\n";
+		DBG_NW << "sockets: " << sockets.size() << "\n";
 		for(sockets_list::const_iterator i = sockets.begin();
 		    i != sockets.end(); ++i) {
-			LOG_NW << "server socket: " << server_socket << "\ncurrent socket: " << *i << "\n";
-			send_data(cfg,*i);
+			DBG_NW << "server socket: " << server_socket << "\ncurrent socket: " << *i << "\n";
+			send_data(cfg,*i, gzipped);
 		}
-
 		return;
 	}
 
-	const schema_map::iterator schema = schemas.find(connection_num);
-	wassert(schema != schemas.end());
-
-	std::ostringstream compressor;
-	write_compressed(compressor, cfg, schema->second.outgoing);
-	std::string const &value = compressor.str();
-
-//	std::cerr << "--- SEND DATA to " << ((int)connection_num) << ": '"
-//	          << cfg.write() << "'\n--- END SEND DATA\n";
-
-	std::vector<char> buf(4 + value.size() + 1);
-	SDLNet_Write32(value.size()+1,&buf[0]);
-	std::copy(value.begin(),value.end(),buf.begin()+4);
-	buf.back() = 0;
-
 	const connection_map::iterator info = connections.find(connection_num);
-	wassert(info != connections.end());
+	if (info == connections.end()) {
+		ERR_NW << "Error: socket: " << connection_num
+			<< "\tnot found in connection_map. Not sending...\n";
+		return;
+	}
 
-	network_worker_pool::queue_data(info->second.sock,buf);
-}
-
-void queue_data(const config& cfg, connection connection_num)
-{
-	send_data(cfg,connection_num);
+	LOG_NW << "SENDING to: " << connection_num << ": " << cfg;
+	network_worker_pool::queue_data(info->second.sock, cfg, gzipped);
 }
 
 void process_send_queue(connection, size_t)
@@ -717,14 +756,15 @@ void process_send_queue(connection, size_t)
 	check_error();
 }
 
-void send_data_all_except(const config& cfg, connection connection_num)
+//! @todo Note the gzipped parameter should be removed later.
+void send_data_all_except(const config& cfg, connection connection_num, const bool gzipped)
 {
 	for(sockets_list::const_iterator i = sockets.begin(); i != sockets.end(); ++i) {
 		if(*i == connection_num) {
 			continue;
 		}
 
-		send_data(cfg,*i);
+		send_data(cfg,*i, gzipped);
 	}
 }
 
@@ -746,6 +786,12 @@ std::string ip_address(connection connection_num)
 	return str.str();
 }
 
+
+std::pair<unsigned int,size_t> get_thread_state()
+{
+	return network_worker_pool::thread_state();
+}
+
 statistics get_send_stats(connection handle)
 {
 	return network_worker_pool::get_current_transfer_stats(handle == 0 ? get_socket(sockets.back()) : get_socket(handle)).first;
@@ -755,4 +801,4 @@ statistics get_receive_stats(connection handle)
 	return network_worker_pool::get_current_transfer_stats(handle == 0 ? get_socket(sockets.back()) : get_socket(handle)).second;
 }
 
-} //end namespace network
+} // end namespace network
