@@ -1,10 +1,11 @@
-/* $Id: network_worker.hpp 8584 2005-10-16 00:21:44Z dave $ */
+/* $Id: network_worker.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -16,20 +17,16 @@
 
 #include <map>
 #include <vector>
-
+#include "config.hpp"
+#include "network.hpp"
 #include "SDL_net.h"
 
-namespace network
-{
-	struct statistics;
-}
-
 namespace network_worker_pool
 {
 
 struct manager
 {
-	explicit manager(size_t nthreads=1);
+	explicit manager(size_t min_threads,size_t max_threads);
 	~manager();
 
 private:
@@ -39,14 +36,15 @@ private:
 	bool active_;
 };
 
-//function to asynchronously received data to the given socket
+//! Function to asynchronously received data to the given socket.
 void receive_data(TCPsocket sock);
 
-TCPsocket get_received_data(TCPsocket sock, std::vector<char>& buf);
+TCPsocket get_received_data(TCPsocket sock, config& cfg);
 
-void queue_data(TCPsocket sock, std::vector<char>& buf);
-bool socket_locked(TCPsocket sock);
-bool close_socket(TCPsocket sock);
+void queue_data(TCPsocket sock, const config& buf, const bool gzipped);
+bool is_locked(const TCPsocket sock);
+bool close_socket(TCPsocket sock, bool force=false);
+std::pair<unsigned int,size_t> thread_state();
 TCPsocket detect_error();
 
 std::pair<network::statistics,network::statistics> get_current_transfer_stats(TCPsocket sock);
