@@ -1,10 +1,11 @@
-/* $Id: reports.hpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: reports.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -18,23 +19,22 @@
 #include <string>
 #include <vector>
 
-class gamestatus;
-
-#include "map.hpp"
+#include "image.hpp"
 #include "team.hpp"
-#include "unit.hpp"
 
 //this module is responsible for outputting textual reports of
 //various game and unit statistics
 namespace reports {
 
-	enum TYPE { UNIT_DESCRIPTION, UNIT_TYPE, UNIT_LEVEL, UNIT_TRAITS, UNIT_STATUS,
-	            UNIT_ALIGNMENT, UNIT_ABILITIES, UNIT_HP, UNIT_XP, UNIT_MOVES, UNIT_WEAPONS,
-				UNIT_IMAGE, UNIT_PROFILE, TIME_OF_DAY,
-				TURN, GOLD, VILLAGES, NUM_UNITS, UPKEEP, EXPENSES, INCOME, TERRAIN, POSITION,
-		    SIDE_PLAYING, OBSERVERS, REPORT_CLOCK,
-		    SELECTED_TERRAIN, EDIT_LEFT_BUTTON_FUNCTION,
-				NUM_REPORTS};
+	enum TYPE { UNIT_DESCRIPTION, UNIT_TYPE, UNIT_RACE, UNIT_LEVEL,
+		    UNIT_AMLA, UNIT_TRAITS, UNIT_STATUS,
+		    UNIT_ALIGNMENT, UNIT_ABILITIES, UNIT_HP, UNIT_XP,
+		    UNIT_ADVANCEMENT_OPTIONS, UNIT_MOVES, UNIT_WEAPONS,
+		    UNIT_IMAGE, UNIT_PROFILE, TIME_OF_DAY,
+		    TURN, GOLD, VILLAGES, NUM_UNITS, UPKEEP, EXPENSES,
+		    INCOME, TERRAIN, POSITION, SIDE_PLAYING, OBSERVERS,
+		    REPORT_COUNTDOWN, REPORT_CLOCK, SELECTED_TERRAIN,
+		    EDIT_LEFT_BUTTON_FUNCTION, NUM_REPORTS};
 
 	enum { UNIT_REPORTS_BEGIN=UNIT_DESCRIPTION, UNIT_REPORTS_END=UNIT_PROFILE+1 };
 	enum { STATUS_REPORTS_BEGIN=TIME_OF_DAY, STATUS_REPORTS_END=NUM_REPORTS};
@@ -42,19 +42,31 @@ namespace reports {
 	const std::string& report_name(TYPE type);
 
 	struct element {
+	private:
+		//! Shouldn't be used so only declared.
+		element();
+	public:		
+		explicit element(const std::string& text) : 
+				image(),
+				text(text),
+				tooltip()
+				{}
+
 		// Invariant: either text or image should be empty
 		// It would be okay to create a class for this, but it's a pretty simple
 		// invariant so I left it like the original report class.
-		std::string image;
+		image::locator image;
 		std::string text;
 
 		std::string tooltip;
-
-		element() {}
-		explicit element(const std::string& text) : text(text) {}
 		element(const std::string& text, const std::string& image, const std::string& tooltip) :
 			image(image), text(text), tooltip(tooltip) {}
 
+		element(const std::string& text, const image::locator& image, const std::string& tooltip) :
+			image(image), text(text), tooltip(tooltip) {}
+		element(const std::string& text, const char* image, const std::string& tooltip) :
+			image(image), text(text), tooltip(tooltip) {}
+
 		bool operator==(const element& o) const {
 			return o.text == text && o.image == image && o.tooltip == tooltip;
 		}
@@ -66,6 +78,12 @@ namespace reports {
 		report(const std::string& text, const std::string& image, const std::string& tooltip) {
 			this->push_back(element(text, image, tooltip));
 		}
+		report(const std::string& text, const char* image, const std::string& tooltip) {
+			this->push_back(element(text, image, tooltip));
+		}
+		report(const std::string& text, const image::locator& image, const std::string& tooltip) {
+			this->push_back(element(text, image, tooltip));
+		}
 
 		// Convenience functions
 		void add_text(std::stringstream& text, std::stringstream& tooltip);
@@ -74,17 +92,14 @@ namespace reports {
 		void add_image(std::stringstream& image, std::stringstream& tooltip);
 	};
 
-	report generate_report(TYPE type, const gamemap& map, const unit_map& units,
-	                       const std::vector<team>& teams, const team& current_team,
-	                       int current_side, int active_side,
-	                       const gamemap::location& loc, const gamemap::location& mouseover,
-	                       const gamestatus& status, const std::set<std::string>& observers);
-	// Set what will be shown for the report with type
-	// which_report. Note that this only works for some reports,
-	// i.e. reports that can not be deducted from the supplied arguments
-	// to generate_report.
-	// Currently: SELECTED_TERRAIN, EDIT_LEFT_BUTTON_FUNCTION
-	void set_report_content(const TYPE which_report, const std::string &content);
+	report generate_report(TYPE type,
+			       std::map<reports::TYPE, std::string> report_contents,
+			       const gamemap& map, unit_map& units,
+			       const std::vector<team>& teams, const team& current_team,
+			       unsigned int current_side, int unsigned active_side,
+			       const gamemap::location& loc, const gamemap::location& mouseover, const gamemap::location& displayed_unit_hex,
+			       const gamestatus& status, const std::set<std::string>& observers,
+			       const config& level);
 }
 
 #endif
