@@ -1,16 +1,20 @@
-/* $Id: sdl_utils.cpp 12002 2006-06-10 16:20:25Z torangan $ */
+/* $Id: sdl_utils.cpp 23876 2008-02-17 12:18:58Z alink $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file sdl_utils.cpp
+//! Support-routines for the SDL-graphics-library.
+
 #include "global.hpp"
 
 #include "config.hpp"
@@ -18,13 +22,29 @@
 #include "sdl_utils.hpp"
 #include "util.hpp"
 #include "video.hpp"
-#include "wassert.hpp"
 
 #include <algorithm>
+#include <cassert>
 #include <cmath>
 #include <iostream>
 #include <map>
 
+SDL_Color int_to_color(const Uint32 rgb) {
+	SDL_Color to_return = {
+		(0x00FF0000 & rgb)>>16,
+		(0x0000FF00 & rgb)>>8,
+		(0x000000FF & rgb), 0
+	};
+	return to_return;
+}
+
+Uint32 color_to_int(const SDL_Color& color) {
+	Uint32 to_return = color.r;
+	to_return = (to_return << 8) & color.g;
+	to_return = (to_return << 8) & color.b;
+	return to_return;
+}
+
 SDLKey sdl_keysym_from_name(std::string const &keyname)
 {
 	static bool initialized = false;
@@ -54,8 +74,8 @@ bool point_in_rect(int x, int y, const SDL_Rect& rect)
 
 bool rects_overlap(const SDL_Rect& rect1, const SDL_Rect& rect2)
 {
-	return point_in_rect(rect1.x,rect1.y,rect2) || point_in_rect(rect2.x+rect2.w,rect2.y,rect1) ||
-	       point_in_rect(rect2.x,rect2.y+rect2.h,rect1) || point_in_rect(rect1.x+rect1.w,rect1.y+rect1.h,rect2);
+	return (rect1.x < rect2.x+rect2.w && rect2.x < rect1.x+rect1.w &&
+			rect1.y < rect2.y+rect2.h && rect2.y < rect1.y+rect1.h);
 }
 
 SDL_Rect intersect_rects(SDL_Rect const &rect1, SDL_Rect const &rect2)
@@ -73,8 +93,7 @@ bool operator<(const surface& a, const surface& b)
 	return a.get() < b.get();
 }
 
-namespace {
-	SDL_PixelFormat& get_neutral_pixel_format()
+static SDL_PixelFormat& get_neutral_pixel_format()
 	{
 		static bool first_time = true;
 		static SDL_PixelFormat format;
@@ -89,8 +108,6 @@ namespace {
 		return format;
 	}
 
-}
-
 surface make_neutral_surface(surface const &surf)
 {
 	if(surf == NULL) {
@@ -124,9 +141,10 @@ surface create_optimized_surface(surface const &surf)
 	return result;
 }
 
-// don't pass this function 0 scaling arguments
+// NOTE: Don't pass this function 0 scaling arguments.
 surface scale_surface(surface const &surf, int w, int h)
 {
+	// Since SDL version 1.1.5 0 is transparent, before 255 was transparent.
 	assert(SDL_ALPHA_TRANSPARENT==0);
 
 	if(surf == NULL)
@@ -135,11 +153,18 @@ surface scale_surface(surface const &surf, int w, int h)
 	if(w == surf->w && h == surf->h) {
 		return surf;
 	}
-	wassert(w != 0);
-	wassert(h != 0);
+	assert(w >= 0);
+	assert(h >= 0);
 
 	surface dst(SDL_CreateRGBSurface(SDL_SWSURFACE,w,h,32,0xFF0000,0xFF00,0xFF,0xFF000000));
+		
+	if (w == 0 || h ==0) {
+		std::cerr << "Create an empty image\n";
+		return create_optimized_surface(dst);
+	}
+
 	surface src(make_neutral_surface(surf));
+	// Now both surfaces are always in the "neutral" pixel format
 
 	if(src == NULL || dst == NULL) {
 		std::cerr << "Could not create surface to scale onto\n";
@@ -191,7 +216,7 @@ surface scale_surface(surface const &surf, int w, int h)
 				const fixed_t w = 0xFF - e;
 
 				pix[0] = *src_word;              // northwest
-			        pix[1] = *(src_word + dx);       // northeast
+				pix[1] = *(src_word + dx);       // northeast
 				pix[2] = *(src_word + dy);       // southwest
 				pix[3] = *(src_word + dx + dy);  // southeast
 
@@ -207,7 +232,10 @@ surface scale_surface(surface const &surf, int w, int h)
 				avg_r = avg_g = avg_b = avg_a = 0;
 				int loc;
 				for (loc=0; loc<4; loc++) {
-				  SDL_GetRGBA(pix[loc],src->format,&r,&g,&b,&a);
+				  a = pix[loc] >> 24;
+				  r = pix[loc] >> 16;
+				  g = pix[loc] >> 8;
+				  b = pix[loc] >> 0;
 				  if (a != 0) {
 				    avg_r += r;
 				    avg_g += g;
@@ -223,35 +251,34 @@ surface scale_surface(surface const &surf, int w, int h)
 				  avg_a /= count;
 				}
 
-				// Perform modified bilinear
-				// interpolation. Don't trust any
-				// color information from an RGBA
-				// sample when the alpha channel is
-				// set to fully transparent.
+				// Perform modified bilinear interpolation.
+				// Don't trust any color information from
+				// an RGBA sample when the alpha channel
+				// is set to fully transparent.
 				//
-				// Some of the input images are hex
-				// tiles, created using a hexagon
-				// shaped alpha channel that is either
-				// set to full-on or full-off. If
-				// intermediate alpha values are
-				// introduced along a hex edge, it
-				// produces a gametime artifact.
-				// Moving the mouse around will leave
-				// behind "hexagon halos" from the
-				// temporary highlighting. In other
-				// words the Wesnoth rendering engine
+				// Some of the input images are hex tiles,
+				// created using a hexagon shaped alpha channel
+				// that is either set to full-on or full-off.
+				//
+				// If intermediate alpha values are introduced
+				// along a hex edge, it produces a gametime artifact.
+				// Moving the mouse around will leave behind
+				// "hexagon halos" from the temporary highlighting.
+				// In other words, the Wesnoth rendering engine
 				// freaks out.
 				//
-				// The alpha thresholding step
-				// attempts to accomodates this
-				// limitation. There is a small loss
-				// of quality. For example, skeleton
-				// bowstrings are not as good as they
-				// could be.
+				// The alpha thresholding step attempts
+				// to accomodates this limitation.
+				// There is a small loss of quality.
+				// For example, skeleton bowstrings
+				// are not as good as they could be.
 
 				rr = gg = bb = aa = 0;
 				for (loc=0; loc<4; loc++) {
-				  SDL_GetRGBA(pix[loc],src->format,&r,&g,&b,&a);
+				  a = pix[loc] >> 24;
+				  r = pix[loc] >> 16;
+				  g = pix[loc] >> 8;
+				  b = pix[loc] >> 0;
 				  if (a == 0) {
 				    r = avg_r;
 				    g = avg_g;
@@ -267,7 +294,7 @@ surface scale_surface(surface const &surf, int w, int h)
 				b = bb >> 16;
 				a = aa >> 16;
 				a = (a < avg_a/2) ? 0 : avg_a;
-				*dst_word = SDL_MapRGBA(dst->format,r,g,b,a);
+				*dst_word = (a << 24) + (r << 16) + (g << 8) + b;
 			}
 		}
 	}
@@ -283,8 +310,16 @@ surface scale_surface_blended(surface const &surf, int w, int h)
 	if(w == surf->w && h == surf->h) {
 		return surf;
 	}
+	assert(w >= 0);
+	assert(h >= 0);
 
 	surface dst(SDL_CreateRGBSurface(SDL_SWSURFACE,w,h,32,0xFF0000,0xFF00,0xFF,0xFF000000));
+
+	if (w == 0 || h ==0) {
+		std::cerr << "Create an empty image\n";
+		return create_optimized_surface(dst);
+	}
+
 	surface src(make_neutral_surface(surf));
 
 	if(src == NULL || dst == NULL) {
@@ -312,8 +347,8 @@ surface scale_surface_blended(surface const &surf, int w, int h)
 
 				double summation = 0.0;
 
-				//we now have a rectangle, (xsrc,ysrc,xratio,yratio) which we
-				//want to derive the pixel from
+				// We now have a rectangle, (xsrc,ysrc,xratio,yratio)
+				// which we want to derive the pixel from
 				for(double xloc = xsrc; xloc < xsrc+xratio; xloc += 1.0) {
 					const double xsize = minimum<double>(std::floor(xloc+1.0)-xloc,xsrc+xratio-xloc);
 					for(double yloc = ysrc; yloc < ysrc+yratio; yloc += 1.0) {
@@ -370,13 +405,17 @@ surface adjust_surface_colour(surface const &surf, int r, int g, int b)
 
 		while(beg != end) {
 			Uint8 red, green, blue, alpha;
-			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
+			alpha = (*beg) >> 24;
+			red   = (*beg) >> 16;
+			green = (*beg) >> 8;
+			blue  = (*beg) >> 0;
+
 
 			red = maximum<int>(8,minimum<int>(255,int(red)+r));
 			green = maximum<int>(0,minimum<int>(255,int(green)+g));
 			blue  = maximum<int>(0,minimum<int>(255,int(blue)+b));
 
-			*beg = SDL_MapRGBA(nsurf->format,red,green,blue,alpha);
+			*beg = (alpha << 24) + (red << 16) + (green << 8) + blue;
 
 			++beg;
 		}
@@ -407,13 +446,14 @@ surface greyscale_image(surface const &surf)
 
 			//const Uint8 avg = (red+green+blue)/3;
 
-			//use the correct formula for RGB to grayscale
-			//conversion. ok, this is no big deal :)
-			//the correct formula being:
-			//gray=0.299red+0.587green+0.114blue
-			const Uint8 avg = (Uint8)((77*(Uint16)red +
-						   150*(Uint16)green +
-						   29*(Uint16)blue) / 256);
+			// Use the correct formula for RGB to grayscale conversion.
+			// Ok, this is no big deal :)
+			// The correct formula being:
+			// gray=0.299red+0.587green+0.114blue
+			const Uint8 avg = static_cast<Uint8>((
+				77 * static_cast<Uint16>(red) +
+				150 * static_cast<Uint16>(green) +
+				29 * static_cast<Uint16>(blue)) / 256);
 
 
 			*beg = SDL_MapRGBA(nsurf->format,avg,avg,avg,alpha);
@@ -447,17 +487,18 @@ surface darken_image(surface const &surf)
 
 			//const Uint8 avg = (red+green+blue)/3;
 
-			//use the correct formula for RGB to grayscale
-			//conversion. ok, this is no big deal :)
-			//the correct formula being:
-			//gray=0.299red+0.587green+0.114blue
-			const Uint8 avg = (Uint8)((77*(Uint16)red +
-						   150*(Uint16)green +
-						   29*(Uint16)blue) / 256);
+			// Use the correct formula for RGB to grayscale conversion.
+			// Ok, this is no big deal :)
+			// The correct formula being:
+			// gray=0.299red+0.587green+0.114blue
+			const Uint8 avg = static_cast<Uint8>((
+				77 * static_cast<Uint16>(red) +
+				150 * static_cast<Uint16>(green) +
+				29 * static_cast<Uint16>(blue)) / 256);
 			// then tint 77%, 67%, 72%
-			const Uint8 r=(Uint8)((avg*196)>>8);
-			const Uint8 g=(Uint8)((avg*171)>>8);
-			const Uint8 b=(Uint8)((avg*184)>>8);
+			const Uint8 r= ((avg * 196) >> 8);
+			const Uint8 g= ((avg * 171) >> 8);
+			const Uint8 b= ((avg * 184) >> 8);
 
 
 			*beg = SDL_MapRGBA(nsurf->format,r,g,b,alpha);
@@ -469,95 +510,43 @@ surface darken_image(surface const &surf)
 	return create_optimized_surface(nsurf);
 }
 
-surface recolor_image(surface surf, const color_range& new_rgb, const std::vector<Uint32>& old_rgb)
-{
-  // function to replace vectors of colors with new color-range
-  // parallel input vectors chosen to prevent need to include SDL structures
-  // elsewhere in code or create new rgb structure.  It might be better to
-  // just pass a vector of SDL colors instead...
+surface recolor_image(surface surf, const std::map<Uint32, Uint32>& map_rgb){
+	if(map_rgb.size()){
+		if(surf == NULL)
+		return NULL;
 
-  if(old_rgb.size()){
-    Uint16 new_red  = (new_rgb.mid() & 0x00FF0000)>>16;
-    Uint16 new_green= (new_rgb.mid() & 0x0000FF00)>>8;
-    Uint16 new_blue = (new_rgb.mid() & 0x000000FF);
-    Uint16 max_red  = (new_rgb.max() & 0x00FF0000)>>16;
-    Uint16 max_green= (new_rgb.max() & 0x0000FF00)>>8 ;
-    Uint16 max_blue = (new_rgb.max() & 0x000000FF)    ;
-    Uint16 min_red  = (new_rgb.min() & 0x00FF0000)>>16;
-    Uint16 min_green= (new_rgb.min() & 0x0000FF00)>>8 ;
-    Uint16 min_blue = (new_rgb.min() & 0x000000FF)    ;
+	     surface nsurf(make_neutral_surface(surf));
+	     if(nsurf == NULL) {
+			std::cerr << "failed to make neutral surface\n";
+			return NULL;
+	     }
 
-    if(surf == NULL)
-      return NULL;
+		surface_lock lock(nsurf);
+		Uint32* beg = lock.pixels();
+		Uint32* end = beg + nsurf->w*surf->h;
 
-        surface nsurf(make_neutral_surface(surf));
-        if(nsurf == NULL) {
-	  std::cerr << "failed to make neutral surface\n";
-	  return NULL;
-        }
+		while(beg != end) {
+			Uint8 red, green, blue, alpha;
+			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
 
-	Uint16 reference_avg=0;
-	//map first color in vector to exact new color
-	Uint32 temp_rgb=old_rgb[0];
-	int old_r=(temp_rgb & 0X00FF0000)>>16;
-	int old_g=(temp_rgb & 0X0000FF00)>>8;
-	int old_b=(temp_rgb & 0X000000FF);
-	reference_avg = (Uint16)(((Uint16) old_r + (Uint16)old_g + (Uint16)old_b)
-			   / 3);
-
-	{
-	  for(std::vector<Uint32>::const_iterator temp_rgb = old_rgb.begin();
-	      temp_rgb!=old_rgb.end();temp_rgb++)
-	    {
-	      int old_r=((*temp_rgb) & 0X00FF0000)>>16;
-	      int old_g=((*temp_rgb) & 0X0000FF00)>>8;
-	      int old_b=((*temp_rgb) & 0X000000FF);
-
-	      //	    std::cout<<"recolor:"<<old_r<<","<<old_g<<","<<old_b<<"\n";
-	      surface_lock lock(nsurf);
-	      Uint32* beg = lock.pixels();
-	      Uint32* end = beg + nsurf->w*surf->h;
-
-	      //const Uint16 old_grey = (Uint16)((77*(Uint16) old_r +
-	      //                                  150*(Uint16)old_g +
-	      //                                  29*(Uint16)old_b) / 256);
-
-	      const Uint16 old_avg = (Uint16)(((Uint16) old_r +
-						(Uint16) old_g +
-						(Uint16) old_b) / 3);
-	      //calculate new color
-	      Uint8 new_r, new_g, new_b;
-
-	      if(reference_avg && old_avg <= reference_avg){
-		float old_rat = ((float)old_avg)/reference_avg;
-		new_r=(Uint8)( old_rat * new_red   + (1 - old_rat) * min_red);
-		new_g=(Uint8)( old_rat * new_green + (1 - old_rat) * min_green);
-		new_b=(Uint8)( old_rat * new_blue  + (1 - old_rat) * min_blue);
-	      }else if(255 - reference_avg){
-		float old_rat = ((float) 255 - old_avg) / (255 - reference_avg);
-		new_r=(Uint8)( old_rat * new_red   + (1 - old_rat) * max_red);
-		new_g=(Uint8)( old_rat * new_green + (1 - old_rat) * max_green);
-		new_b=(Uint8)( old_rat * new_blue  + (1 - old_rat) * max_blue);
-	      }else{
-		      new_r=0; new_g=0; new_b=0; //supress warning
-		      wassert(false);
-		//should never get here
-		//would imply old_avg > reference_avg = 255
-	      }
-
-	      while(beg != end) {
-                        Uint8 red, green, blue, alpha;
-                        SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
-                        if(red==old_r && green==old_g && blue==old_b){
-                          *beg = SDL_MapRGBA(nsurf->format,new_r,new_g,new_b,alpha);
-                        }
-                        ++beg;
-	      }
-	    }
+			if(alpha){	// don't recolor invisible pixels.
+				Uint32 oldrgb = (red<<16) + (green<<8) + (blue);
+				for(std::map<Uint32, Uint32>::const_iterator i=map_rgb.begin(); i!= map_rgb.end(); i++){
+					if(oldrgb==i->first){
+						Uint32 new_rgb = i->second;
+						Uint8 new_r = (new_rgb & 0x00FF0000)>>16;
+						Uint8 new_g = (new_rgb & 0x0000FF00)>>8;
+						Uint8 new_b = (new_rgb & 0x000000FF);
+					*beg = SDL_MapRGBA(nsurf->format,new_r,new_g,new_b,alpha);
+					break; // no double replacements.
+					}
+				}
+			}
+		++beg;
+		}
+		return create_optimized_surface(nsurf);
 	}
-        return create_optimized_surface(nsurf);
-  }
-      return NULL;
+	return surf;
 }
 
 surface brighten_image(surface const &surf, fixed_t amount)
@@ -581,13 +570,16 @@ surface brighten_image(surface const &surf, fixed_t amount)
 		if (amount < 0) amount = 0;
 		while(beg != end) {
 			Uint8 red, green, blue, alpha;
-			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
+			alpha = (*beg) >> 24;
+			red   = (*beg) >> 16;
+			green = (*beg) >> 8;
+			blue  = (*beg) >> 0;
 
 			red = minimum<unsigned>(unsigned(fxpmult(red,amount)),255);
 			green = minimum<unsigned>(unsigned(fxpmult(green,amount)),255);
 			blue = minimum<unsigned>(unsigned(fxpmult(blue,amount)),255);
 
-			*beg = SDL_MapRGBA(nsurf->format,red,green,blue,alpha);
+			*beg = (alpha << 24) + (red << 16) + (green << 8) + blue;
 
 			++beg;
 		}
@@ -596,7 +588,7 @@ surface brighten_image(surface const &surf, fixed_t amount)
 	return create_optimized_surface(nsurf);
 }
 
-surface adjust_surface_alpha(surface const &surf, fixed_t amount)
+surface adjust_surface_alpha(surface const &surf, fixed_t amount, bool optimize)
 {
 	if(surf== NULL) {
 		return NULL;
@@ -617,20 +609,27 @@ surface adjust_surface_alpha(surface const &surf, fixed_t amount)
 		if (amount < 0) amount = 0;
 		while(beg != end) {
 			Uint8 red, green, blue, alpha;
-			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
+			alpha = (*beg) >> 24;
+			red   = (*beg) >> 16;
+			green = (*beg) >> 8;
+			blue  = (*beg) >> 0;
 
 			alpha = minimum<unsigned>(unsigned(fxpmult(alpha,amount)),255);
 
-			*beg = SDL_MapRGBA(nsurf->format,red,green,blue,alpha);
+			*beg = (alpha << 24) + (red << 16) + (green << 8) + blue;
 
 			++beg;
 		}
 	}
 
-	return create_optimized_surface(nsurf);
+	if (optimize) {
+		return create_optimized_surface(nsurf);
+	} else {
+		return nsurf;
+	}
 }
 
-surface adjust_surface_alpha_add(surface const &surf, int amount)
+surface adjust_surface_alpha_add(surface const &surf, int amount, bool optimize)
 {
 	if(surf== NULL) {
 		return NULL;
@@ -650,20 +649,23 @@ surface adjust_surface_alpha_add(surface const &surf, int amount)
 
 		while(beg != end) {
 			Uint8 red, green, blue, alpha;
-			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
+			alpha = (*beg) >> 24;
+			red   = (*beg) >> 16;
+			green = (*beg) >> 8;
+			blue  = (*beg) >> 0;
 
 			alpha = Uint8(maximum<int>(0,minimum<int>(255,int(alpha) + amount)));
 
-			*beg = SDL_MapRGBA(nsurf->format,red,green,blue,alpha);
+			*beg = (alpha << 24) + (red << 16) + (green << 8) + blue;
 
 			++beg;
 		}
 	}
 
-	return create_optimized_surface(nsurf);
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-// Applies a mask on a surface
+//! Applies a mask on a surface.
 surface mask_surface(surface const &surf, surface const &mask)
 {
 	if(surf == NULL) {
@@ -677,7 +679,16 @@ surface mask_surface(surface const &surf, surface const &mask)
 		std::cerr << "could not make neutral surface...\n";
 		return NULL;
 	}
-
+	if (nsurf->w !=  nmask->w) {
+		// we don't support efficiently different width.
+		// (different height is not a real problem)
+		// This function is used on all hexes and usually only for that
+		// so better keep it simple and efficient for the normal case
+		std::cerr << "Detected an image with bad dimensions :" << nsurf->w << "x" << nsurf->h << "\n";
+		std::cerr << "It will not be masked, please use :"<< nmask->w << "x" << nmask->h << "\n";
+		return nsurf;
+	}
+	
 	{
 		surface_lock lock(nsurf);
 		surface_lock mlock(nmask);
@@ -689,14 +700,18 @@ surface mask_surface(surface const &surf, surface const &mask)
 
 		while(beg != end && mbeg != mend) {
 			Uint8 red, green, blue, alpha;
-			Uint8 mred, mgreen, mblue, malpha;
+			Uint8 malpha;
 
-			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
-			SDL_GetRGBA(*mbeg,nmask->format,&mred,&mgreen,&mblue,&malpha);
+			alpha = (*beg) >> 24;
+			red   = (*beg) >> 16;
+			green = (*beg) >> 8;
+			blue  = (*beg) >> 0;
+
+			malpha = (*mbeg) >> 24;
 
 			alpha = Uint8(minimum<int>(malpha, alpha));
 
-			*beg = SDL_MapRGBA(nsurf->format,red,green,blue,alpha);
+			*beg = (alpha << 24) + (red << 16) + (green << 8) + blue;
 
 			++beg;
 			++mbeg;
@@ -707,91 +722,249 @@ surface mask_surface(surface const &surf, surface const &mask)
 	//return create_optimized_surface(nsurf);
 }
 
-// Cross-fades a surface
+//! Cross-fades a surface.
 surface blur_surface(surface const &surf, int depth)
 {
 	if(surf == NULL) {
 		return NULL;
 	}
 
-	surface nsurf = make_neutral_surface(surf);
-	surface res = create_compatible_surface(nsurf, surf->w, surf->h);
+	surface res = make_neutral_surface(surf);
 
-	if(nsurf == NULL || res == NULL) {
+	if(res == NULL) {
 		std::cerr << "could not make neutral surface...\n";
 		return NULL;
 	}
 
-	for(int count = 0; count < depth; ++count) {
-		surface_lock lock(nsurf);
-		surface_lock rlock(res);
+	const int max_blur = 256;
+	if(depth > max_blur) {
+		depth = max_blur;
+	}
 
-		{
-			Uint32* p = lock.pixels();
-			Uint32* left = p - 1;
-			Uint32* right = p + 1;
-			Uint32* top = p - res->w;
-			Uint32* bottom = p + res->w;
-			Uint32* rp = rlock.pixels();
+	Uint32 queue[max_blur];
+	const Uint32* end_queue = queue + max_blur;
 
-			for(int y = 0; y < res->h; ++y) {
-				for(int x = 0; x < res->w; ++x) {
-					Uint8 red, green, blue, alpha;
-					Uint16 rred=0, rgreen=0, rblue=0, ralpha=0;
+	const Uint32 ff = 0xff;
 
-					if (x != 0) {
-						SDL_GetRGBA(*left, nsurf->format, &red, &green, &blue, &alpha);
-						rred += red;
-						rgreen += green;
-						rblue += blue;
-						ralpha += alpha;
-					}
-
-					if (x != res->w - 1) {
-						SDL_GetRGBA(*right, nsurf->format, &red, &green, &blue, &alpha);
-						rred += red;
-						rgreen += green;
-						rblue += blue;
-						ralpha += alpha;
-					}
-
-					if (y != 0) {
-						SDL_GetRGBA(*top, nsurf->format, &red, &green, &blue, &alpha);
-						rred += red;
-						rgreen += green;
-						rblue += blue;
-						ralpha += alpha;
-					}
-
-					if (y != res->h - 1) {
-						SDL_GetRGBA(*bottom, nsurf->format, &red, &green, &blue, &alpha);
-						rred += red;
-						rgreen += green;
-						rblue += blue;
-						ralpha += alpha;
-					}
-
-					rred /= 4;
-					rgreen /= 4;
-					rblue /= 4;
-					ralpha /= 4;
-
-					*rp = SDL_MapRGBA(res->format, (Uint8)rred, (Uint8)rgreen, (Uint8)rblue, (Uint8)ralpha);
-
-					++left; ++right; ++top; ++bottom; ++rp;
-				}
+	surface_lock lock(res);
+	int x, y;
+	for(y = 0; y < res->h; ++y) {
+		const Uint32* front = &queue[0];
+		Uint32* back = &queue[0];
+		Uint32 red = 0, green = 0, blue = 0, avg = 0;
+		Uint32* p = lock.pixels() + y*res->w;
+		for(x = 0; x <= depth && x < res->w; ++x, ++p) {
+			red += ((*p) >> 16)&0xFF;
+			green += ((*p) >> 8)&0xFF;
+			blue += (*p)&0xFF;
+			++avg;
+			*back++ = *p;
+			if(back == end_queue) {
+				back = &queue[0];
 			}
 		}
 
-		if (count < depth - 1) {
-			nsurf = res;
+		p = lock.pixels() + y*res->w;
+		for(x = 0; x < res->w; ++x, ++p) {
+			*p = 0xFF000000 | (minimum(red/avg,ff) << 16) | (minimum(green/avg,ff) << 8) | minimum(blue/avg,ff);
+			if(x >= depth) {
+				red -= ((*front) >> 16)&0xFF;
+				green -= ((*front) >> 8)&0xFF;
+				blue -= *front&0xFF;
+				--avg;
+				++front;
+				if(front == end_queue) {
+					front = &queue[0];
+				}
+			}
+
+			if(x + depth+1 < res->w) {
+				Uint32* q = p + depth+1;
+				red += ((*q) >> 16)&0xFF;
+				green += ((*q) >> 8)&0xFF;
+				blue += (*q)&0xFF;
+				++avg;
+				*back++ = *q;
+				if(back == end_queue) {
+					back = &queue[0];
+				}
+			}
+		}
+	}
+
+	for(x = 0; x < res->w; ++x) {
+		const Uint32* front = &queue[0];
+		Uint32* back = &queue[0];
+		Uint32 red = 0, green = 0, blue = 0, avg = 0;
+		Uint32* p = lock.pixels() + x;
+		for(y = 0; y <= depth && y < res->h; ++y, p += res->w) {
+			red += ((*p) >> 16)&0xFF;
+			green += ((*p) >> 8)&0xFF;
+			blue += *p&0xFF;
+			++avg;
+			*back++ = *p;
+			if(back == end_queue) {
+				back = &queue[0];
+			}
+		}
+
+		p = lock.pixels() + x;
+		for(y = 0; y < res->h; ++y, p += res->w) {
+			*p = 0xFF000000 | (minimum(red/avg,ff) << 16) | (minimum(green/avg,ff) << 8) | minimum(blue/avg,ff);
+			if(y >= depth) {
+				red -= ((*front) >> 16)&0xFF;
+				green -= ((*front) >> 8)&0xFF;
+				blue -= *front&0xFF;
+				--avg;
+				++front;
+				if(front == end_queue) {
+					front = &queue[0];
+				}
+			}
+
+			if(y + depth+1 < res->h) {
+				Uint32* q = p + (depth+1)*res->w;
+				red += ((*q) >> 16)&0xFF;
+				green += ((*q) >> 8)&0xFF;
+				blue += (*q)&0xFF;
+				++avg;
+				*back++ = *q;
+				if(back == end_queue) {
+					back = &queue[0];
+				}
+			}
 		}
 	}
 
 	return create_optimized_surface(res);
 }
 
-// Cuts a rectangle from a surface.
+//! Cross-fades a surface with alpha channel.
+//! @todo FIXME: This is just an adapted copy-paste
+//! of the normal blur but with blur alpha channel too
+surface blur_alpha_surface(surface const &surf, int depth)
+{
+	if(surf == NULL) {
+		return NULL;
+	}
+
+	surface res = make_neutral_surface(surf);
+
+	if(res == NULL) {
+		std::cerr << "could not make neutral surface...\n";
+		return NULL;
+	}
+
+	const int max_blur = 256;
+	if(depth > max_blur) {
+		depth = max_blur;
+	}
+
+	Uint32 queue[max_blur];
+	const Uint32* end_queue = queue + max_blur;
+
+	const Uint32 ff = 0xff;
+
+	surface_lock lock(res);
+	int x, y;
+	for(y = 0; y < res->h; ++y) {
+		const Uint32* front = &queue[0];
+		Uint32* back = &queue[0];
+		Uint32 alpha=0, red = 0, green = 0, blue = 0, avg = 0;
+		Uint32* p = lock.pixels() + y*res->w;
+		for(x = 0; x <= depth && x < res->w; ++x, ++p) {
+			alpha += ((*p) >> 24)&0xFF;
+			red += ((*p) >> 16)&0xFF;
+			green += ((*p) >> 8)&0xFF;
+			blue += (*p)&0xFF;
+			++avg;
+			*back++ = *p;
+			if(back == end_queue) {
+				back = &queue[0];
+			}
+		}
+
+		p = lock.pixels() + y*res->w;
+		for(x = 0; x < res->w; ++x, ++p) {
+			*p = (minimum(alpha/avg,ff) << 24) | (minimum(red/avg,ff) << 16) | (minimum(green/avg,ff) << 8) | minimum(blue/avg,ff);
+			if(x >= depth) {
+				alpha -= ((*front) >> 24)&0xFF;
+				red -= ((*front) >> 16)&0xFF;
+				green -= ((*front) >> 8)&0xFF;
+				blue -= *front&0xFF;
+				--avg;
+				++front;
+				if(front == end_queue) {
+					front = &queue[0];
+				}
+			}
+
+			if(x + depth+1 < res->w) {
+				Uint32* q = p + depth+1;
+				alpha += ((*q) >> 24)&0xFF;
+				red += ((*q) >> 16)&0xFF;
+				green += ((*q) >> 8)&0xFF;
+				blue += (*q)&0xFF;
+				++avg;
+				*back++ = *q;
+				if(back == end_queue) {
+					back = &queue[0];
+				}
+			}
+		}
+	}
+
+	for(x = 0; x < res->w; ++x) {
+		const Uint32* front = &queue[0];
+		Uint32* back = &queue[0];
+		Uint32 alpha=0, red = 0, green = 0, blue = 0, avg = 0;
+		Uint32* p = lock.pixels() + x;
+		for(y = 0; y <= depth && y < res->h; ++y, p += res->w) {
+			alpha += ((*p) >> 24)&0xFF;
+			red += ((*p) >> 16)&0xFF;
+			green += ((*p) >> 8)&0xFF;
+			blue += *p&0xFF;
+			++avg;
+			*back++ = *p;
+			if(back == end_queue) {
+				back = &queue[0];
+			}
+		}
+
+		p = lock.pixels() + x;
+		for(y = 0; y < res->h; ++y, p += res->w) {
+			*p = (minimum(alpha/avg,ff) << 24) | (minimum(red/avg,ff) << 16) | (minimum(green/avg,ff) << 8) | minimum(blue/avg,ff);
+			if(y >= depth) {
+				alpha -= ((*front) >> 24)&0xFF;
+				red -= ((*front) >> 16)&0xFF;
+				green -= ((*front) >> 8)&0xFF;
+				blue -= *front&0xFF;
+				--avg;
+				++front;
+				if(front == end_queue) {
+					front = &queue[0];
+				}
+			}
+
+			if(y + depth+1 < res->h) {
+				Uint32* q = p + (depth+1)*res->w;
+				alpha += ((*q) >> 24)&0xFF;
+				red += ((*q) >> 16)&0xFF;
+				green += ((*q) >> 8)&0xFF;
+				blue += (*q)&0xFF;
+				++avg;
+				*back++ = *q;
+				if(back == end_queue) {
+					back = &queue[0];
+				}
+			}
+		}
+	}
+
+	return create_optimized_surface(res);
+}
+
+//! Cuts a rectangle from a surface.
 surface cut_surface(surface const &surf, SDL_Rect const &r)
 {
 	surface res = create_compatible_surface(surf, r.w, r.h);
@@ -815,7 +988,7 @@ surface cut_surface(surface const &surf, SDL_Rect const &r)
 		Uint8* line_dest = dest + y * rpitch;
 		size_t size = r.w + r.x <= surf->w ? r.w : surf->w - r.x;
 
-		wassert(rpitch >= r.w * rbpp);
+		assert(rpitch >= r.w * rbpp);
 		memcpy(line_dest, line_src, size * rbpp);
 	}
 
@@ -962,15 +1135,15 @@ void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, surface const &
 
 surface get_surface_portion(surface const &src, SDL_Rect &area)
 {
-	if(area.x >= src->w || area.y >= src->h) {
-		std::cerr << "illegal surface portion...\n";
+	// Check if there is something in the portion
+	if(area.x >= src->w || area.y >= src->h || area.x + area.w < 0 || area.y + area.h < 0) {
+		//std::cerr << "illegal surface portion...\n";
 		return NULL;
 	}
 
 	if(area.x + area.w > src->w) {
 		area.w = src->w - area.x;
 	}
-
 	if(area.y + area.h > src->h) {
 		area.h = src->h - area.y;
 	}
@@ -981,9 +1154,7 @@ surface get_surface_portion(surface const &src, SDL_Rect &area)
 		return NULL;
 	}
 
-	SDL_Rect dstarea = {0,0,0,0};
-
-	SDL_BlitSurface(src,&area,dst,&dstarea);
+	SDL_BlitSurface(src,&area,dst, NULL);
 
 	return dst;
 }
@@ -1006,7 +1177,7 @@ private:
 
 }
 
-SDL_Rect get_non_transperant_portion(surface const &surf)
+SDL_Rect get_non_transparent_portion(surface const &surf)
 {
 	SDL_Rect res = {0,0,0,0};
 	const surface nsurf(make_neutral_surface(surf));
@@ -1039,8 +1210,9 @@ SDL_Rect get_non_transperant_portion(surface const &surf)
 			break;
 	}
 
-	//the height is the height of the surface, minus the distance from the top and the
-	//distance from the bottom
+	// The height is the height of the surface,
+	// minus the distance from the top and
+	// the distance from the bottom.
 	res.h = nsurf->h - res.y - n;
 
 	for(n = 0; n != nsurf->w; ++n) {
@@ -1122,10 +1294,6 @@ void pixel_data::read(const config& cfg) {
 		b = atoi(blue.c_str());
 }
 
-namespace {
-	const SDL_Rect empty_rect = {0,0,0,0};
-}
-
 surface_restorer::surface_restorer() : target_(NULL), rect_(empty_rect), surface_(NULL)
 {
 }
@@ -1199,3 +1367,20 @@ void draw_solid_tinted_rectangle(int x, int y, int w, int h,
 	SDL_Rect rect = {x,y,w,h};
 	fill_rect_alpha(rect,SDL_MapRGB(target->format,r,g,b),Uint8(alpha*255),target);
 }
+
+void draw_centered_on_background(surface surf, const SDL_Rect& rect, const SDL_Color& color, surface target)
+{
+	clip_rect_setter clip_setter(target, rect);
+
+	Uint32 col = SDL_MapRGBA(target->format, color.r, color.g, color.b, color.unused);
+	//TODO: only draw background outside the image
+	SDL_Rect r = rect;
+	SDL_FillRect(target, &r, col);
+
+	if (surf != NULL) {
+		r.x = rect.x + (rect.w-surf->w)/2;
+		r.y = rect.y + (rect.h-surf->h)/2;
+		SDL_BlitSurface(surf, NULL, target, &r);
+	}
+	update_rect(rect);
+}
