@@ -1,18 +1,22 @@
-/* $Id: parser.cpp 12583 2006-07-15 04:24:57Z rusty $ */
+/* $Id: parser.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 by David White <dave@whitevine.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Copyright (C) 2005 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2008 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file serialization/parser.cpp 
+//! Read/Write & analyse WML- and config-files.
+
 #include "serialization/parser.hpp"
 
 #include "config.hpp"
@@ -22,16 +26,19 @@
 #include "loadscreen.hpp"
 #include "log.hpp"
 #include "util.hpp"
-#include "wassert.hpp"
 #include "wesconfig.h"
 #include "serialization/binary_wml.hpp"
 #include "serialization/preprocessor.hpp"
 #include "serialization/string_utils.hpp"
 #include "serialization/tokenizer.hpp"
 
+#include <cassert>
 #include <sstream>
 #include <stack>
 
+#include <boost/iostreams/filtering_stream.hpp>
+#include <boost/iostreams/filter/gzip.hpp>
+
 #define ERR_CF LOG_STREAM(err, config)
 #define WRN_CF LOG_STREAM(warn, config)
 #define LOG_CF LOG_STREAM(info, config)
@@ -42,8 +49,12 @@ namespace {
 
 class parser
 {
+	parser();
+	parser(const parser&);
+	parser& operator=(const parser&);
 public:
 	parser(config& cfg, std::istream& in);
+	~parser();
 	void operator() (std::string* error_log=NULL);
 
 private:
@@ -55,11 +66,16 @@ private:
 	void error(const std::string& message);
 
 	config& cfg_;
-	tokenizer tok_;
+	tokenizer *tok_;
 
 	struct element {
-		element(config *cfg, std::string const &name, std::string const &start_line)
-			: cfg(cfg), name(name), start_line(start_line) {}
+		element(config *cfg, std::string 
+			const &name, std::string const &start_line) :
+				cfg(cfg), 
+				name(name),
+				last_element_map(),
+				start_line(start_line) 
+			{}
 
 		config* cfg;
 		std::string name;
@@ -71,11 +87,21 @@ private:
 	std::stack<element> elements;
 };
 
-parser::parser(config &cfg, std::istream &in)
-	: cfg_(cfg), tok_(in)
+parser::parser(config &cfg, std::istream &in) :
+		cfg_(cfg), 
+		tok_(new tokenizer(in)),
+		elements()
 {
 }
 
+
+parser::~parser()
+{
+	if(tok_) {
+		delete tok_;
+	}
+}
+
 void parser::operator()(std::string* error_log)
 {
 	cfg_.clear();
@@ -83,9 +109,9 @@ void parser::operator()(std::string* error_log)
 
 	do {
 		try {
-			tok_.next_token();
+			tok_->next_token();
 
-			switch(tok_.current_token().type) {
+			switch(tok_->current_token().type) {
 			case token::LF:
 				continue;
 			case '[':
@@ -105,18 +131,18 @@ void parser::operator()(std::string* error_log)
 				throw;
 
 			// On error, dump tokens to the next LF
-			while(tok_.current_token().type != token::LF &&
-					tok_.current_token().type != token::END) {
-				tok_.next_token();
+			while(tok_->current_token().type != token::LF &&
+					tok_->current_token().type != token::END) {
+				tok_->next_token();
 			}
 
 			*error_log += e.message + '\n';
 		}
 		increment_parser_progress();
-	} while (tok_.current_token().type != token::END);
+	} while (tok_->current_token().type != token::END);
 
 	// The main element should be there. If it is not, this is a parser error.
-	wassert(!elements.empty());
+	assert(!elements.empty());
 
 	if(elements.size() != 1) {
 		utils::string_map i18n_symbols;
@@ -128,28 +154,28 @@ void parser::operator()(std::string* error_log)
 
 void parser::parse_element()
 {
-	tok_.next_token();
+	tok_->next_token();
 	std::string elname;
 	config* current_element = NULL;
 	std::map<std::string, config*>::const_iterator last_element_itor;
 
-	switch(tok_.current_token().type) {
+	switch(tok_->current_token().type) {
 	case token::STRING: // [element]
-		elname = tok_.current_token().value;
-		if (tok_.next_token().type != ']')
+		elname = tok_->current_token().value;
+		if (tok_->next_token().type != ']')
 			error(_("Unterminated [element] tag"));
 
 		// Add the element
 		current_element = &(elements.top().cfg->add_child(elname));
 		elements.top().last_element_map[elname] = current_element;
-		elements.push(element(current_element, elname, tok_.get_line()));
+		elements.push(element(current_element, elname, tok_->get_line()));
 		break;
 
 	case '+': // [+element]
-		if (tok_.next_token().type != token::STRING)
+		if (tok_->next_token().type != token::STRING)
 			error(_("Invalid tag name"));
-		elname = tok_.current_token().value;
-		if (tok_.next_token().type != ']')
+		elname = tok_->current_token().value;
+		if (tok_->next_token().type != ']')
 			error(_("Unterminated [+element] tag"));
 
 		// Find the last child of the current element whose name is
@@ -161,14 +187,14 @@ void parser::parse_element()
 			current_element = last_element_itor->second;
 		}
 		elements.top().last_element_map[elname] = current_element;
-		elements.push(element(current_element, elname, tok_.get_line()));
+		elements.push(element(current_element, elname, tok_->get_line()));
 		break;
 
 	case '/': // [/element]
-		if(tok_.next_token().type != token::STRING)
+		if(tok_->next_token().type != token::STRING)
 			error(_("Invalid closing tag name"));
-		elname = tok_.current_token().value;
-		if(tok_.next_token().type != ']')
+		elname = tok_->current_token().value;
+		if(tok_->next_token().type != ']')
 			error(_("Unterminated closing tag"));
 		if(elements.size() <= 1)
 			error(_("Unexpected closing tag"));
@@ -193,12 +219,12 @@ void parser::parse_variable()
 	std::vector<std::string> variables;
 	variables.push_back("");
 
-	while (tok_.current_token().type != '=') {
-		switch(tok_.current_token().type) {
+	while (tok_->current_token().type != '=') {
+		switch(tok_->current_token().type) {
 		case token::STRING:
 			if(!variables.back().empty())
 				variables.back() += ' ';
-			variables.back() += tok_.current_token().value;
+			variables.back() += tok_->current_token().value;
 			break;
 		case ',':
 			if(variables.back().empty()) {
@@ -211,7 +237,7 @@ void parser::parse_variable()
 			error(_("Unexpected characters after variable name (expected , or =)"));
 			break;
 		}
-		tok_.next_token();
+		tok_->next_token();
 	}
 	if(variables.back().empty())
 		error(_("Empty variable name"));
@@ -226,10 +252,10 @@ void parser::parse_variable()
 
 	bool ignore_next_newlines = false;
 	while(1) {
-		tok_.next_token();
-		wassert(curvar != variables.end());
+		tok_->next_token();
+		assert(curvar != variables.end());
 
-		switch (tok_.current_token().type) {
+		switch (tok_->current_token().type) {
 		case ',':
 			if ((curvar+1) != variables.end()) {
 				curvar++;
@@ -240,17 +266,17 @@ void parser::parse_variable()
 			}
 			break;
 		case '_':
-			tok_.next_token();
-			switch (tok_.current_token().type) {
+			tok_->next_token();
+			switch (tok_->current_token().type) {
 			case token::UNTERMINATED_QSTRING:
 				error(_("Unterminated quoted string"));
 				break;
 			case token::QSTRING:
-				cfg[*curvar] += t_string(tok_.current_token().value, tok_.textdomain());
+				cfg[*curvar] += t_string(tok_->current_token().value, tok_->textdomain());
 				break;
 			default:
 				cfg[*curvar] += "_";
-				cfg[*curvar] += tok_.current_token().value;
+				cfg[*curvar] += tok_->current_token().value;
 				break;
 			case token::END:
 			case token::LF:
@@ -261,10 +287,10 @@ void parser::parse_variable()
 			// Ignore this
 			break;
 		default:
-			cfg[*curvar] += tok_.current_token().leading_spaces + tok_.current_token().value;
+			cfg[*curvar] += tok_->current_token().leading_spaces + tok_->current_token().value;
 			break;
 		case token::QSTRING:
-			cfg[*curvar] += tok_.current_token().value;
+			cfg[*curvar] += tok_->current_token().value;
 			break;
 		case token::UNTERMINATED_QSTRING:
 			error(_("Unterminated quoted string"));
@@ -277,9 +303,9 @@ void parser::parse_variable()
 			return;
 		}
 
-		if (tok_.current_token().type == '+') {
+		if (tok_->current_token().type == '+') {
 			ignore_next_newlines = true;
-		} else if (tok_.current_token().type != token::LF) {
+		} else if (tok_->current_token().type != token::LF) {
 			ignore_next_newlines = false;
 		}
 	}
@@ -310,15 +336,31 @@ void parser::error(const std::string& error_type)
 	i18n_symbols["error"] = error_type;
 
 	throw config::error(
-		lineno_string(i18n_symbols, tok_.get_line(),
+		lineno_string(i18n_symbols, tok_->get_line(),
 		              N_("$error at $pos")));
 }
 
 } // end anon namespace
 
-void read(config &cfg, std::istream &data_in, std::string* error_log)
+void read(config &cfg, std::istream &in, std::string* error_log)
 {
-	parser(cfg, data_in)(error_log);
+	parser(cfg, in)(error_log);
+}
+
+void read(config &cfg, std::string &in, std::string* error_log)
+{
+	std::stringstream ss(in);
+	parser(cfg, ss)(error_log);
+}
+
+void read_gz(config &cfg, std::istream &file, std::string* error_log)
+{
+
+	boost::iostreams::filtering_stream<boost::iostreams::input> filter;
+	filter.push(boost::iostreams::gzip_decompressor());
+	filter.push(file);
+
+	parser(cfg, filter)(error_log);
 }
 
 static char const *AttributeEquals = "=";
@@ -341,65 +383,83 @@ static char const *EndElementPostfix = "]\n";
 static std::string escaped_string(const std::string& value) {
 	std::vector<char> res;
 	for(std::string::const_iterator i = value.begin(); i != value.end(); ++i) {
-		//double interior quotes
+		// double interior quotes
 		if(*i == '\"') res.push_back(*i);
 		res.push_back(*i);
 	}
 	return std::string(res.begin(), res.end());
 }
 
+void write_key_val(std::ostream &out, const std::string &key, const t_string &value, unsigned int level, std::string& textdomain)
+{
+	bool first = true;
+	if (value.empty()) {
+		return;
+	}
+
+	for(t_string::walker w(value); !w.eos(); w.next()) {
+		std::string part(w.begin(), w.end());
+
+		if(w.translatable()) {
+			if(w.textdomain() != textdomain) {
+				out << TextdomainPrefix
+					<< w.textdomain()
+					<< TextdomainPostfix;
+				textdomain = w.textdomain();
+			}
+
+			if(first) {
+				out << std::string(level, '\t')
+					<< key
+					<< AttributeEquals;
+			}
+
+			out << TranslatableAttributePrefix
+				<< escaped_string(part)
+				<< AttributePostfix;
+
+		} else {
+			if(first) {
+				out << std::string(level, '\t')
+					<< key
+					<< AttributeEquals;
+			}
+
+			out << AttributePrefix
+				<< escaped_string(part)
+				<< AttributePostfix;
+		}
+
+		if(w.last()) {
+			out << AttributeEndPostfix;
+		} else {
+			out << AttributeContPostfix;
+			out << std::string(level+1, '\t');
+		}
+
+		first = false;
+	}
+}
+
+void write_open_child(std::ostream &out, const std::string &child, unsigned int level)
+{
+	out << std::string(level, '\t')
+		<< ElementPrefix << child << ElementPostfix;
+}
+
+void write_close_child(std::ostream &out, const std::string &child, unsigned int level)
+{
+	out << std::string(level, '\t')
+		<< EndElementPrefix << child << EndElementPostfix;
+}
+
 static void write_internal(config const &cfg, std::ostream &out, std::string& textdomain, size_t tab = 0)
 {
 	if (tab > max_recursion_levels)
 		throw config::error("Too many recursion levels in config write");
 
 	for(string_map::const_iterator i = cfg.values.begin(), i_end = cfg.values.end(); i != i_end; ++i) {
-		if (!i->second.empty()) {
-			bool first = true;
-
-			for(t_string::walker w(i->second); !w.eos(); w.next()) {
-				std::string part(w.begin(), w.end());
-
-				if(w.translatable()) {
-					if(w.textdomain() != textdomain) {
-						out << TextdomainPrefix
-							<< w.textdomain()
-							<< TextdomainPostfix;
-						textdomain = w.textdomain();
-					}
-
-					if(first) {
-						out << std::string(tab, '\t')
-							<< i->first
-							<< AttributeEquals;
-					}
-
-					out << TranslatableAttributePrefix
-						<< escaped_string(part)
-						<< AttributePostfix;
-
-				} else {
-					if(first) {
-						out << std::string(tab, '\t')
-							<< i->first
-							<< AttributeEquals;
-					}
-
-					out << AttributePrefix
-						<< escaped_string(part)
-						<< AttributePostfix;
-				}
-
-				if(w.last()) {
-					out << AttributeEndPostfix;
-				} else {
-					out << AttributeContPostfix;
-					out << std::string(tab+1, '\t');
-				}
-
-				first = false;
-			}
-		}
+		write_key_val(out, i->first, i->second, tab, textdomain);
 	}
 
 	for(config::all_children_iterator j = cfg.ordered_begin(), j_end = cfg.ordered_end(); j != j_end; ++j) {
@@ -407,16 +467,15 @@ static void write_internal(config const &cfg, std::ostream &out, std::string& te
 		const std::string& name = *item.first;
 		const config& cfg = *item.second;
 
-		out << std::string(tab, '\t')
-		    << ElementPrefix << name << ElementPostfix;
+		write_open_child(out, name, tab);
 		write_internal(cfg, out, textdomain, tab + 1);
-		out << std::string(tab, '\t')
-		    << EndElementPrefix << name << EndElementPostfix;
+		write_close_child(out, name, tab);
 	}
 }
 
-void write(std::ostream &out, config const &cfg)
+void write(std::ostream &out, config const &cfg, unsigned int level)
 {
 	std::string textdomain = PACKAGE;
-	write_internal(cfg, out, textdomain);
+	write_internal(cfg, out, textdomain, level);
 }
+
