@@ -1,16 +1,20 @@
-/* $Id: metrics.cpp 9251 2005-12-20 21:39:15Z darthfool $ */
+/* $Id: metrics.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file server/metrics.cpp
+//! Various server-statistics.
+
 #include "../global.hpp"
 
 #include "metrics.hpp"
@@ -55,7 +59,7 @@ std::ostream& operator<<(std::ostream& out, metrics& met)
 	const int days = time_up/(60*60*24);
 	const int requests_immediate = met.nrequests_ - met.nrequests_waited_;
 	const int percent_immediate = (requests_immediate*100)/(met.nrequests_ > 0 ? met.nrequests_ : 1);
-	out << "METRICS\n----\nUp " << days << " days, " << hours << " hours, "
+	out << "METRICS\nUp " << days << " days, " << hours << " hours, "
 	    << minutes << " minutes, " << seconds << " seconds\n"
 	    << met.nrequests_ << " requests serviced. " << requests_immediate
 	    << " (" << percent_immediate << "%) "
@@ -69,6 +73,5 @@ std::ostream& operator<<(std::ostream& out, metrics& met)
 		}
 	}
 
-	out << "----\n";
 	return out;
 }
