@@ -1,10 +1,11 @@
-/* $Id: video.hpp 11647 2006-05-26 01:03:08Z rusty $ */
+/* $Id: video.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,6 +14,7 @@
 #ifndef VIDEO_HPP_INCLUDED
 #define VIDEO_HPP_INCLUDED
 
+#include "events.hpp"
 #include "SDL.h"
 #include "sdl_utils.hpp"
 
@@ -134,4 +136,15 @@ private:
 	bool unlock;
 };
 
+class resize_monitor : public events::pump_monitor {
+	void process(events::pump_info &info);
+};
+
+//an object which prevents resizing of the screen occuring during
+//its lifetime.
+struct resize_lock {
+	resize_lock();
+	~resize_lock();
+};
+
 #endif
