@@ -1,10 +1,11 @@
-/* $Id: label.cpp 12603 2006-07-17 09:53:55Z isaac $ */
+/* $Id: label.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -19,7 +20,7 @@
 
 namespace gui {
 
-label::label(CVideo& video, const std::string& text, int size, const SDL_Color& colour) : widget(video), text_(text), size_(size), colour_(colour)
+label::label(CVideo& video, const std::string& text, int size, const SDL_Color& colour, const bool auto_join) : widget(video, auto_join), text_(text), size_(size), colour_(colour)
 {
 	update_label_size();
 }
