@@ -1,10 +1,12 @@
+/* $Id: menu_style.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   wesnoth menu styles Copyright (C) 2006 by Patrick Parker <patrick_x99@hotmail.com>
-   wesnoth menu Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   wesnoth menu styles Copyright (C) 2006 - 2008 by Patrick Parker <patrick_x99@hotmail.com>
+   wesnoth menu Copyright (C) 2003-5 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -35,19 +37,24 @@
 namespace gui {
 
 	//static initializations
-menu::imgsel_style menu::slateborder_style("misc/selection", false,
-										   0x000000, 0x4a4440, 0x999999,
-										   0.0, 0.2, 0.2);
 menu::imgsel_style menu::bluebg_style("misc/selection2", true,
 										   0x000000, 0x000000, 0x333333,
 										   0.35, 0.0, 0.3);
+menu::style menu::simple_style;
+
+#ifdef USE_TINY_GUI
+menu::style &menu::default_style = menu::simple_style;
+#else
 menu::style &menu::default_style = menu::bluebg_style;
+#endif
+menu *empty_menu = NULL;
 
 	//constructors
 menu::style::style() : font_size_(font::SIZE_NORMAL),
 		cell_padding_(font::SIZE_NORMAL * 3/5), thickness_(0),
 		normal_rgb_(0x000000), selected_rgb_(0x000099), heading_rgb_(0x333333),
-		normal_alpha_(0.2),  selected_alpha_(0.6), heading_alpha_(0.3)
+		normal_alpha_(0.2),  selected_alpha_(0.6), heading_alpha_(0.3),
+		max_img_w_(-1), max_img_h_(-1)
 {}
 
 menu::style::~style()
@@ -66,11 +73,36 @@ size_t menu::style::get_font_size() const { return font_size_; }
 size_t menu::style::get_cell_padding() const { return cell_padding_; }
 size_t menu::style::get_thickness() const { return thickness_; }
 
+void menu::style::scale_images(int max_width, int max_height)
+{
+	max_img_w_ = max_width;
+	max_img_h_ = max_height;
+}
+
+surface menu::style::get_item_image(const image::locator& img_loc) const
+{
+	surface surf = image::get_image(img_loc);
+	if(!surf.null())
+	{
+		int scale = 100;
+		if(max_img_w_ > 0 && surf->w > max_img_w_) {
+			scale = (max_img_w_ * 100) / surf->w;
+		}
+		if(max_img_h_ > 0 && surf->h > max_img_h_) {
+			scale = minimum<int>(scale, ((max_img_h_ * 100) / surf->h));
+		}
+		if(scale != 100)
+		{
+			return scale_surface(surf, (scale * surf->w)/100, (scale * surf->h)/100);
+		}
+	}
+	return surf;
+}
 
 bool menu::imgsel_style::load_image(const std::string &img_sub)
 {
 	std::string path = img_base_ + "-" + img_sub + ".png";
-	const surface image = image::get_image(path,image::UNSCALED);
+	const surface image = image::get_image(path);
 	img_map_[img_sub] = image;
 	return(!image.null());
 }
