@@ -1,16 +1,19 @@
-/* $Id: scrollpane.cpp 13460 2006-09-08 08:49:00Z dirus $ */
+/* $Id: scrollpane.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file widgets/scrollpane.cpp
+
 #include "global.hpp"
 
 #include <algorithm>
@@ -107,7 +110,7 @@ void scrollpane::draw()
 
 void scrollpane::scroll(unsigned int pos)
 {
-	if ((int)pos == content_pos_.y)
+	if (static_cast<int>(pos) == content_pos_.y)
 		return;
 
 	content_pos_.y = pos;
@@ -178,4 +181,5 @@ void scrollpane::update_content_size()
 	set_dirty();
 }
 
-}
+} // namespace gui
+
