@@ -1,10 +1,11 @@
-/* $Id: widget.cpp 15989 2007-02-21 21:13:25Z mordante $ */
-/*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+/* $Id: widget.cpp 23842 2008-02-16 08:47:16Z mordante $
+
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -22,21 +23,18 @@ namespace {
 
 namespace gui {
 
-	widget::widget(const widget &o)
-	: events::handler(), focus_(o.focus_), video_(o.video_), restorer_(o.restorer_),
-	  rect_(o.rect_), needs_restore_(o.needs_restore_),
-	  state_(o.state_), hidden_override_(o.hidden_override_),
-	  enabled_(o.enabled_), clip_(o.clip_), clip_rect_(o.clip_rect_),
-	  volatile_(o.volatile_), help_text_(o.help_text_),
-	  help_string_(o.help_string_), align_(o.align_), id_(o.id_)
+widget::widget(const widget &o)
+	: events::handler(), focus_(o.focus_), video_(o.video_), restorer_(o.restorer_), rect_(o.rect_),
+	   needs_restore_(o.needs_restore_), state_(o.state_), hidden_override_(o.hidden_override_),
+	  enabled_(o.enabled_), clip_(o.clip_), clip_rect_(o.clip_rect_), volatile_(o.volatile_),
+	  help_text_(o.help_text_), help_string_(o.help_string_), id_(o.id_)
 {
 }
 
-widget::widget(CVideo& video, bool auto_join)
-	: handler(auto_join), focus_(true), video_(&video), rect_(EmptyRect), 
-	  needs_restore_(false), state_(UNINIT), hidden_override_(false),
-	  enabled_(true), clip_(false), clip_rect_(EmptyRect), volatile_(false),
-	  help_string_(0), align_(RIGHT_ALIGN)
+widget::widget(CVideo& video, const bool auto_join)
+	: handler(auto_join), focus_(true), video_(&video), rect_(EmptyRect), needs_restore_(false),
+	  state_(UNINIT), hidden_override_(false), enabled_(true), clip_(false),
+	  clip_rect_(EmptyRect), volatile_(false), help_string_(0)
 {
 }
 
@@ -100,22 +98,12 @@ void widget::set_height(unsigned h)
 	set_location(rect);
 }
 
-void widget::set_align(ALIGN a)
-{
-	align_ = a;
-}
-
 void widget::set_measurements(unsigned w, unsigned h)
 {
 	SDL_Rect rect = { rect_.x, rect_.y, w, h };
 	set_location(rect);
 }
 
-ALIGN widget::align() const
-{
-	return align_;
-}
-
 unsigned widget::width() const
 {
 	return rect_.w;
